<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Controller\Adminhtml\System\Config;

use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Semknox\Semknox\Exception\InvalidStoreException;
use Semknox\Semknox\Helper\InitialUpload;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;

/**
 * Class Upload
 *
 * JSON-based backend controller for controlling and retriving the state
 * of the initial upload process
 *
 * @package Semknox\Semknox\Controller\Adminhtml\System\Config
 */
class Upload extends \Magento\Backend\App\Action
{

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var InfoHelperInterface
     */
    protected $uploadInfoHelper;

    /**
     * @var InitialUpload\InfoHelperFactory
     */
    protected $uploadInfoHelperFactory;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param JsonFactory                         $resultJsonFactory
     * @param InitialUpload\InfoHelperFactory     $uploadInfoHelperFactory
     *
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        JsonFactory $resultJsonFactory,
        InitialUpload\InfoHelperFactory $uploadInfoHelperFactory
    ) {
        parent::__construct($context);
        $this->resultJsonFactory = $resultJsonFactory;
        $this->uploadInfoHelperFactory = $uploadInfoHelperFactory;
    }

    /**
     * @param array $resultArray
     *
     * @return Json
     */
    protected function result($resultArray)
    {
        $resultJson = $this->resultJsonFactory->create();
        return $resultJson->setData($resultArray);
    }

    /**
     * Perform the requested upload action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {

        $storeId = $this->getRequest()->getParam('storeId', -1);
        try {
            $this->uploadInfoHelper = $this->uploadInfoHelperFactory->create(['storeId' => $storeId]);
        } catch (InvalidStoreException $exception) {
            return $this->result([
             'error' => 1,
             'msg' => 'Invalid Request'
            ]);
        }

        $action = $this->getRequest()->getParam('action');
        switch ($action) {
            case 'poll':
                return $this->pollAction();
             break;
            case 'start':
                return $this->startAction();
             break;
            case 'stop':
                return $this->stopAction();
             break;
            case 'pause':
                return $this->pauseAction();
             break;
        }
        return $this->result([
        'error' => 1,
        'msg' => 'Invalid Request'
        ]);
    }

    /**
     * Return the current state of the initial upload.
     * If the upload is running or paused, this will also return the progress.
     *
     * @return Json
     */
    protected function pollAction()
    {

        if ($this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_RUNNING ||
        $this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_PAUSED ||
        $this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_QUEUED
        ) {
            return $this->result([
                'state' => $this->uploadInfoHelper->getState(),
                'exceptions' => $this->uploadInfoHelper->getExceptions(),
                'progress' => $this->uploadInfoHelper->getProgress()
            ]);
        }
        return $this->result([
            'state' => $this->uploadInfoHelper->getState(),
            'exceptions' => $this->uploadInfoHelper->getExceptions(),
            'lastSuccess' => $this->uploadInfoHelper->getLastSuccess()->getTimestamp()
        ]);
    }

    /**
     * Queue the initial product upload.
     * If the current state is not STOPPED or QUEUED, the state will not be changed.
     *
     * @return Json
     */
    protected function startAction()
    {
        if ($this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_STOPPED ||
        $this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_QUEUED ||
        $this->uploadInfoHelper->getState() == InfoHelperInterface::STATE_PAUSED
        ) {
            $this->uploadInfoHelper->setState(InfoHelperInterface::STATE_QUEUED);
            $this->uploadInfoHelper->clearExceptions();
            return $this->result([
             'error' => 0
            ]);
        }
        return $this->result([
        'error' => 1,
        'message' => 'An upload is already running.'
        ]);
    }

    /**
     * Stop the current process or fail if the process is marked as not interruptable.
     *
     * @return Json
     */
    protected function stopAction()
    {
        if ($this->uploadInfoHelper->isInterruptable()) {
            $this->uploadInfoHelper->setState(InfoHelperInterface::STATE_STOPPED);
            // Reset the whole uploading flags to initial values
            $this->uploadInfoHelper->setToInitial();
            return $this->result([
             'error' => 0
            ]);
        }
        return $this->result([
        'error' => 1,
        'message' => 'The proccess is currently not interruptable.'
        ]);
    }

    /**
     * Pause the current process or fail if the process is marked as not interruptable or QUEUED.
     *
     * @return Json
     */
    protected function pauseAction()
    {
        if ($this->uploadInfoHelper->isInterruptable()) {
            $this->uploadInfoHelper->setState(InfoHelperInterface::STATE_PAUSED);
            return $this->result([
             'error' => 0
            ]);
        }
        return $this->result([
        'error' => 1,
        'message' => 'The proccess is currently not interruptable.'
        ]);
    }
}
