<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\StoreManagerInterface;
use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException;
use Semknox\Semknox\Library\LibEntrypointInterface;
use Semknox\Semknox\Library\LibEntrypointInterfaceFactory;
use Magento\Framework\Stdlib\StringUtils;

use Magento\Framework\App\Request\Http as HttpRequest;
use Semknox\Semknox\Library\SearchResult\LibSearchResultInterface;

/**
 * Class ApiConnection
 *
 * Connector to the Search Service Library API entrypoint
 *
 * @package Semknox\Semknox\Helper
 */
class ApiConnection extends \Magento\Framework\App\Helper\AbstractHelper
{

    const PREFIX_SEARCH_RESULT_ORDER = 'semknox_';

    /**
     * @var LibEntrypointInterfaceFactory
     */
    protected $libEntrypointFactory;

    /**
     * @var LibEntrypointInterface[]
     */
    protected $api = [];

    /**
     * @var bool[]
     */
    protected $isConnectedCache = [];

    /** @var LibSearchResultInterface */
    protected $search;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var ConfigInterface
     */
    protected $config;

    /** @var StringUtils  */
    protected $stringUtils;

    /**
     * ApiConnection constructor.
     *
     * @param LibEntrypointInterfaceFactory $libEntrypointFactory
     * @param Context                       $context
     * @param StoreManagerInterface         $storeManager
     * @param ConfigInterface               $config
     * @param StringUtils                   $stringUtils
     * @param HttpRequest                   $request
     */
    public function __construct(
        LibEntrypointInterfaceFactory $libEntrypointFactory,
        Context $context,
        StoreManagerInterface $storeManager,
        ConfigInterface $config,
        StringUtils $stringUtils,
        HttpRequest $request
    ) {
        $this->libEntrypointFactory = $libEntrypointFactory;
        $this->storeManager = $storeManager;
        $this->config = $config;
        $this->stringUtils = $stringUtils;
        $this->request = $request;
        parent::__construct($context);
    }

    /**
     * Get the library entrypoint
     * This loads the user information and the library, if not already loaded.
     *
     * @param int [$storeId]  Store ID to get the API connection for or null for current
     *
     * @return LibEntrypointInterface
     */
    public function get($storeId = null)
    {
        if ($storeId == null) {
            $storeId = $this->storeManager->getStore()->getId();
        }
        if (!array_key_exists($storeId, $this->api)) {
            $this->api[$storeId] = $this->libEntrypointFactory->create();
            // Configure the API connection.
            $this->api[$storeId]->auth()->setConfiguration($this->config->dumpConfigurationForLibrary($storeId));
        }
        return $this->api[$storeId];
    }

    /**
     * Returns true if the API is enabled, authenticated and available.
     * Returns false otherwise.
     *
     * @return bool
     */
    public function isEnabledAndAvailable()
    {
        // TODO: Currently there is no way to explicitly disable the search.
        try {
            return $this->isAuthenticated();
        } catch (ApiException $ex) {
            // TODO: Might consider logging here.
            return false;
        }
    }

    /**
     * Returns whether or not the current user for the specified
     * (or current if null) Store-View is authenticated at the
     * Search Service API.
     * May throw ApiException on non-authentication related errors.
     *
     * @param int [$storeId]  Store ID to get the API connection for or null for current
     *
     * @return bool User Authentication Status
     *
     * @throws ApiException On internal API errors or general connection issues.
     */
    public function isAuthenticated($storeId = null)
    {
        if ($storeId == null) {
            $storeId = $this->storeManager->getStore()->getId();
        }

        if (!array_key_exists($storeId, $this->isConnectedCache)) {
            $this->isConnectedCache[$storeId] = false;
            $lib = $this->get($storeId);
            try {
                $lib->auth()->testConnection();
                $this->isConnectedCache[$storeId] = true;
            } catch (ApiNotAuthenticatedException $anae) {
                // Failed, keep at false
            } catch (ApiException $ex) {
                // 400 means customer not found on this endpoint and also marks failed Authentication.
                // All other codes mean failure, in this case just throw the exception.
                if ($ex->getCode() != 400) {
                    throw $ex;
                }
            }
        }

        return $this->isConnectedCache[$storeId];
    }

    /**
     * @param null $storeId
     * @return bool
     */
    public function isOpened($storeId = null)
    {
        $opened = false;

        try {
            $opened = $this->isAuthenticated($storeId);
        } catch (ApiException $ex) {
            $opened = false;
        }

        return $opened;
    }

    /**
     * @param string $query
     * @param array $filters
     * @param array $rangeFilters
     * @param string $order
     * @param string $dir
     * @return LibSearchResultInterface
     */
    public function search($query, $filters, $rangeFilters, $order, $dir)
    {
        if ($this->search === null) {
            $this->search = $this->get()->search()->getProducts($query, $filters, $rangeFilters, $order, $dir);
        }
        return $this->search;
    }

    /**
     * @return LibSearchResultInterface
     */
    public function getSearch()
    {
        return $this->search;
    }

    /**
     * @return array
     */
    public function getSearchResultOrders()
    {
        $availableOrders = [];
        if ($this->getSearch()) {
            foreach ($this->getSearch()->getOrders() as $order) {
                if ($order['name']) {
                    $availableOrders[self::PREFIX_SEARCH_RESULT_ORDER . $order['id']] = $order['name'];
                }
            }
        }
        return $availableOrders;
    }

    /**
     * @param string $magentoResultOrder
     * @return null
     */
    public function convertSearchResultOrder($magentoResultOrder)
    {
        $smkxOrder = null;
        if ($this->stringUtils->strpos($magentoResultOrder, self::PREFIX_SEARCH_RESULT_ORDER) !== false) {
            $smkxOrder = str_replace(self::PREFIX_SEARCH_RESULT_ORDER, '', $magentoResultOrder);
        }
        return $smkxOrder;
    }

    /**
     * @param string $magentoResultDirection
     * @return string
     */
    public function convertSearchResultDirection($magentoResultDirection)
    {
        return strtolower($magentoResultDirection) === 'asc' ? 'ASC' : 'DESC';
    }
}
