<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Helper;

use Semknox\Semknox\Helper\ConfigInterface;
use Semknox\Semknox\Model\Config\Source\Environment;

class Config extends \Magento\Framework\App\Helper\AbstractHelper implements ConfigInterface
{
    const XML_PATH_API_ACCESS_CUSTOMER_ID   = 'semknox/api_access/customer_id';
    const XML_PATH_API_ACCESS_API_KEY       = 'semknox/api_access/api_key';
    const XML_PATH_API_ACCESS_ENV           = 'semknox/api_access/environment';

    const XML_PATH_ADVANCED_SEO_URL_ENABLED = 'semknox/advanced/seo_urls';
    const XML_PATH_ADVANCED_SEO_URL_PREFIX = 'semknox/advanced/seo_url_prefix';
    const XML_PATH_ADVANCED_NO_ROUTE = 'semknox/advanced/no_route';
    const XML_PATH_ADVANCED_DATA_UPLOAD_DISABLED = 'semknox/advanced/data_upload_disabled';
    const XML_PATH_ADVANCED_REMOVE_PUB_FROM_URL = 'semknox/advanced/remove_pub_from_url';
    const XML_PATH_ADVANCED_FULL_UPLOAD_STRATEGY = 'semknox/advanced/full_upload_strategy';
    const XML_PATH_ADVANCED_ALLOW_ROBOTS = 'semknox/advanced/allow_robots';
    const XML_PATH_ADVANCED_SEARCH_BATCH_SIZE = 'semknox/advanced/search_batch_size';

    /**
     * Get the currently configured customer id
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return mixed
     */
    public function getCustomerId($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_API_ACCESS_CUSTOMER_ID,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get the currently configured API-Key
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getApiKey($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_API_ACCESS_API_KEY,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get the environment mode
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getEnvironment($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_API_ACCESS_ENV,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get the url to the SEMKNOX api
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getApiUrl($storeId = null)
    {
        switch ($this->getEnvironment($storeId)) {
            case Environment::ENV_STAGE_VALUE:
                return Environment::ENV_STAGE_URL;
                break;
            case Environment::ENV_LIVE_VALUE:
                return Environment::ENV_LIVE_URL;
                break;
        }
        return null;
    }

    /**
     * Is the options for SEO URLs currently enabled?
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getSeoUrlEnabled($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_SEO_URL_ENABLED,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        ) ? true : false;
    }

    /**
     * Get the prefix to be used for SEO urls (bla ==> /bla-my-search)
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getSeoUrlPrefix($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_SEO_URL_PREFIX,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Get the noroute. Format: route/controller/action
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getNoRoute($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_NO_ROUTE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Check whether the product upload is enabled or not
     *
     * @param int $storeId  Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getDataUploadDisabled($storeId = null)
    {
        // XXX:  Clean the scope config to ensure newest config data from the DB - there is probably a better way!
        $this->scopeConfig->clean();
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_DATA_UPLOAD_DISABLED,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        ) ? true : false;
    }

    /**
     * Whether or not to remove "pub/" prefix from image urls.
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getRemovePubFromUrl($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_REMOVE_PUB_FROM_URL,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        ) ? true : false;
    }

    /**
     * If false:
     * - Use indexer to process partial product updates
     * If true:
     * - The indexer does nothing
     * - Schedule a full product upload every night
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getFullUploadStrategy($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_FULL_UPLOAD_STRATEGY,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        ) ? true : false;
    }

    /**
     * Whether the meta-robots tag für CatalogSearch result pages is set to:
     * (true) => INDEX,FOLLOW
     * (false) => NOINDEX, NOFOLLOW
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return mixed
     */
    public function getAreRobotsAllowedOnSearchResults($storeId = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_ALLOW_ROBOTS,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        ) ? true : false;
    }

    /**
     * Search batch size
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return mixed
     */
    public function getSearchBatchSize($storeId)
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_ADVANCED_SEARCH_BATCH_SIZE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * Dump the configuration needed for the LibAuthenticationInterface
     * implementation. The actual needed configuration depends on the library used.
     *
     * @see \Semknox\Semknox\Library\LibAuthenticationInterface::setConfiguration
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return array
     */
    public function dumpConfigurationForLibrary($storeId = null)
    {
        return [
            'customer_id'   => $this->getCustomerId($storeId),
            'api_key'       => $this->getApiKey($storeId),
            'api_url'       => $this->getApiUrl($storeId),
            'search_batch_size' => $this->getSearchBatchSize($storeId)
        ];
    }
}
