<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Helper\InitialUpload;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Intl\DateTimeFactory;
use Semknox\Semknox\Exception\InvalidStoreException;

/**
 * Class InfoHelper
 *
 * Helper class for retreiving current state and other misc. information
 * about the initial upload process
 *
 * @package Semknox\Semknox\Helper
 */
class InfoHelper extends \Magento\Framework\App\Helper\AbstractHelper implements InfoHelperInterface
{

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var \Semknox\Semknox\Model\InitialUpload\StateInfoFlag
     */
    protected $uploadStateFlag;

    /**
     * @var array
     */
    protected $flagData;

    /**
     * @var DateTimeFactory
     */
    protected $dateTimeFactory;

    /**
     * @var \Semknox\Semknox\Model\InitialUpload\StateInfoFlagFactory
     */
    protected $stateInfoFlagFactory;

    /**
     * UploadFlagHelper constructor.
     *
     * @param int                                                      $storeId
     * @param \Semknox\Semknox\Model\InitialUpload\StateInfoFlagFactory $stateInfoFlagFactory
     * @param \Magento\Store\Model\StoreManagerInterface               $storeManager
     * @param DateTimeFactory                                          $dateTimeFactory
     * @param Context                                                  $context
     *
     * @throws InvalidStoreException
     */
    public function __construct(
        $storeId,
        \Semknox\Semknox\Model\InitialUpload\StateInfoFlagFactory $stateInfoFlagFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        DateTimeFactory $dateTimeFactory,
        Context $context
    ) {
        $this->storeId = $storeId;
        $this->dateTimeFactory = $dateTimeFactory;
        $this->stateInfoFlagFactory = $stateInfoFlagFactory;

        try {
            $storeManager->getStore($this->storeId);
        } catch (NoSuchEntityException $exception) {
            throw new InvalidStoreException("No such store exists");
        }

        $this->refreshFlagData();

        parent::__construct($context);
    }

    /**
     * Reload the flag data.
     * @return void
     */
    public function refreshFlagData()
    {

        $this->uploadStateFlag = $this->stateInfoFlagFactory->create()->loadSelf();
        // Defaults
        $flagDataDefaults = [
            'state' => InfoHelperInterface::STATE_STOPPED,
            'interruptable' => true,
            'progress' => 0,
            'lastSuccess' => 0,
            'batch' => 1,
            'runnerState' => InfoHelperInterface::RUNNER_WAITING,
            'lastErrors' => []
        ];

        // Step one: Load the entire flag data
        $flagDataFromModel = json_decode($this->uploadStateFlag->getFlagData(), true);
        if ($flagDataFromModel == null) {
            $flagDataFromModel = [];
        }

        // Step two: Reduce this data to the current store and merge with defaults
        $flagDataFromStore = array_key_exists($this->storeId, $flagDataFromModel) ?
            $flagDataFromModel[$this->storeId] : [];
        if ($flagDataFromStore == null) {
            $flagDataFromStore = [];
        }
        $this->flagData = array_merge($flagDataDefaults, $flagDataFromStore);
    }

    /**
     * @return int
     */
    public function getStoreId()
    {
        return $this->storeId;
    }

    /**
     * Return the current flag code
     *
     * If the flag code got corrupted somehow (or isn't set yet), returns the STATE_STOPPED code.
     * @return string returns a value from the state variables
     */
    public function getState()
    {
        $flagCode = $this->flagData['state'];
        if ($flagCode != InfoHelperInterface::STATE_RUNNING
        && $flagCode != InfoHelperInterface::STATE_PAUSED
        && $flagCode != InfoHelperInterface::STATE_STOPPED
        && $flagCode != InfoHelperInterface::STATE_QUEUED
        ) {
            return InfoHelperInterface::STATE_STOPPED;
        }
        return $flagCode;
    }

    /**
     * Set the flag code, setting the value to anything other than
     * the constants of this class will thrown an exception.
     * @param string $flagCode
     * @return void
     */
    public function setState($flagCode)
    {
        $this->refreshFlagData();

        if ($flagCode != InfoHelperInterface::STATE_RUNNING
            && $flagCode != InfoHelperInterface::STATE_PAUSED
            && $flagCode != InfoHelperInterface::STATE_STOPPED
            && $flagCode != InfoHelperInterface::STATE_QUEUED
        ) {
            throw new \InvalidArgumentException('The flagCode must be a STATE_* value');
        }
        $this->flagData['state'] = $flagCode;
        $this->save();
    }

    /**
     * Get the interruptable state.
     * @return bool
     */
    public function isInterruptable()
    {
        $interruptable = $this->flagData['interruptable'];
        if ($interruptable !== true && $interruptable !== false) {
            return true;
        }
        return $interruptable;
    }

    /**
     * Set the interruptable state.
     *
     * @param boolean   $interruptable
     * @return void
     */
    public function setInterruptable($interruptable)
    {
        $this->refreshFlagData();

        if ($interruptable !== true && $interruptable !== false) {
            throw new \InvalidArgumentException('The interruptable value can only be set to true or false.');
        }
        $this->flagData['interruptable'] = $interruptable;
        $this->save();
    }

    /**
     * Get the progress.
     * @return float
     */
    public function getProgress()
    {
        $progress = $this->flagData['progress'];
        if (!is_numeric($progress)) {
            return 0;
        }
        return $progress;
    }

    /**
     * Set the progress.
     *
     * @param number   $progress
     * @return void
     */
    public function setProgress($progress)
    {
        $this->refreshFlagData();

        if (!is_numeric($progress)) {
            throw new \InvalidArgumentException('The progress can only be set to numeric values.');
        }
        $this->flagData['progress'] = $progress;
        $this->save();
    }

    /**
     * Get last success.
     * @return \DateTime
     */
    public function getLastSuccess()
    {
        $lastSuccess = $this->flagData['lastSuccess'];
        if (!is_numeric($lastSuccess)) {
            return $this->dateTimeFactory->create('now', new \DateTimeZone('UTC'))->setTimestamp(0);
        }
        return $this->dateTimeFactory->create('now', new \DateTimeZone('UTC'))->setTimestamp($lastSuccess);
    }

    /**
     * Set last success
     *
     * @param \DateTime $lastSuccess
     * @return void
     */
    public function setLastSuccess(\DateTime $lastSuccess)
    {
        $this->refreshFlagData();

        $this->flagData['lastSuccess'] = $lastSuccess->getTimestamp();
        $this->save();
    }

    /**
     * Get the batch id.
     * @return int
     */
    public function getBatch()
    {
        $batch = $this->flagData['batch'];
        if (!is_numeric($batch)) {
            return 0;
        }
        return $batch;
    }

    /**
     * Set the batch id
     *
     * @param number $batch
     * @return void
     */
    public function setBatch($batch)
    {
        $this->refreshFlagData();

        if (!is_numeric($batch)) {
            throw new \InvalidArgumentException('The batch id can only be set to numeric values.');
        }
        $this->flagData['batch'] = $batch;
        $this->save();
    }

    /**
     * Get the runner state.
     * @return string
     */
    public function getRunnerState()
    {
        $runnerState = $this->flagData['runnerState'];
        return $runnerState;
    }

    /**
     * Set the runner state
     *
     * @param string    $runnerState
     * @return void
     */
    public function setRunnerState($runnerState)
    {
        $this->refreshFlagData();
        $this->flagData['runnerState'] = $runnerState;
        $this->save();
    }

    /**
     * Adds an exception to the upload state.
     * On the third added exception an upload process is stopped.
     *
     * @param \Throwable $exception
     */
    public function addException($exception)
    {
        $this->refreshFlagData();
        if (!is_array($this->flagData['lastErrors'])) {
            $this->flagData['lastErrors'] = [];
        }
        $this->flagData['lastErrors'][] = [
            "message" => $exception->getMessage(),
            "trace" => $exception->__toString(),
        ];
        if (count($this->flagData['lastErrors']) > 3) {
            $this->setState(InfoHelperInterface::STATE_STOPPED);
            $this->setBatch(1);
            $this->setProgress(0);
            $this->setInterruptable(true);
        }
        $this->save();
    }

    /**
     * Clears all exceptions of the upload state and resets the exception counter to zero.
     */
    public function clearExceptions()
    {
        $this->refreshFlagData();
        $this->flagData['lastErrors'] = [];
        $this->save();
    }

    /**
     * Get's the exceptions from this run.
     *
     * @return array [['trace' => string, 'message' => string]]
     */
    public function getExceptions()
    {
        return $this->flagData['lastErrors'];
    }

    /**
     * Resets all upload related flags to a point that represents the initial state of the process.
     * This does however not modify the overall state flag.
     * @return void
     */
    public function setToInitial()
    {
        $this->setRunnerState(InfoHelperInterface::RUNNER_WAITING);
        $this->setBatch(1);
        $this->setProgress(0);
        $this->setInterruptable(true);
    }

    /**
     * Save the flag data to the database
     * @return void
     */
    protected function save()
    {
        $newFlagData = json_decode($this->uploadStateFlag->getFlagData(), true);
        $newFlagData[$this->storeId] = $this->flagData;
        $this->uploadStateFlag->setFlagData(json_encode($newFlagData))->save();
    }
}
