<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Library\SearchResult;

use Semknox\Semknox\Library\SearchResult\LibFilterFactory;
use Semknox\Semknox\Library\SearchResult\LibFilterInterface;
use Semknox\Semknox\Library\SearchResult\LibFilterOptionFactory;
use Semknox\Semknox\Library\SearchResult\LibSearchResultInterface;
use Semknox\Semknox\Library\Semknox\Api\Data\ProductsInterface;
use Semknox\Semknox\Library\Semknox\Api\Data\Result\FlagInterface;
use Semknox\Semknox\Library\Semknox\Api\Data\ResultInterface;

class LibSearchResult implements LibSearchResultInterface
{
    /**
     * @var ProductsInterface
     */
    protected $products;

    /**
     * @var LibFilterInterface[]
     */
    protected $filterCache;

    /**
     * @var LibFilterFactory
     */
    protected $libFilterFactory;

    /**
     * @var LibFilterOptionFactory
     */
    protected $libFilterOptionFactory;

    /**
     * LibSearchResult constructor.
     *
     * @param ProductsInterface      $products
     * @param LibFilterFactory       $libFilterFactory
     * @param LibFilterOptionFactory $libFilterOptionFactory
     */
    public function __construct(
        ProductsInterface $products,
        LibFilterFactory $libFilterFactory,
        LibFilterOptionFactory $libFilterOptionFactory
    ) {
        $this->products = $products;
        $this->libFilterFactory = $libFilterFactory;
        $this->libFilterOptionFactory = $libFilterOptionFactory;
    }

    /**
     * Returns the ids of all products found by this search
     * Returns empty array if no products were found
     *
     * @return int[]
     */
    public function getResultProductIds()
    {
        return array_map(function ($entry) {
            /** @var ResultInterface $entry */
            return $entry->getArticleNumber();
        }, $this->products->getResults());
    }

    /**
     * If no products were found and this library supports "suggested products on no search result"
     * this may return an array of product ids to show.
     * Otherwise this will return an empty array
     *
     * @return int[]
     */
    public function getNoResultProductIds()
    {
        return array_map(function ($entry) {
            /** @var ResultInterface $entry */
            return $entry->getArticleNumber();
        }, $this->products->getZeroResults() ?: []);
    }

    /**
     * Returns the product flags (array of string to show on top of product image in product list)
     * for the given product if
     * 1. the library supports product flags
     * 2. the product was in the search results and has a flag
     *
     * Otherwise returns empty array
     *
     * @param int $productId
     *
     * @return string[]
     */
    public function getProductFlag($productId)
    {
        $productsIndexed = $this->products->getResultsIndexed();
        if (array_key_exists($productId, $productsIndexed) && $productsIndexed[$productId]->getFlags()) {
            return array_map(function ($entry) {
                /** @var FlagInterface $entry */
                return $entry->getValue();
            }, $productsIndexed[$productId]->getFlags());
        }
        return [];
    }

    /**
     * Return the available filters that can be performed on this search query
     *
     * @return LibFilterInterface[]
     */
    public function getFilters()
    {
        if (!$this->filterCache) {
            $this->filterCache = [];
            $filtersSknox = $this->products->getFilters();
            // TODO: Put in seperate classes or properly refactor the whole data layer to end this interface hell
            foreach ($filtersSknox as $filter) {
                $libFilter = $this->libFilterFactory->create();
                $libFilter->setId($filter->getId());
                $libFilter->setName($filter->getViewName());
                $libFilter->setType($filter->getType());
                $libFilter->setMax($filter->getMax());
                $libFilter->setMin($filter->getMin());
                $libFilter->setUnitName($filter->getUnitName());
                $libOptions = [];
                foreach ($filter->getOptionsObjects() as $filterOption) {
                    $libOption = $this->libFilterOptionFactory->create();
                    $libOption->setId($filterOption->getId());
                    $libOption->setName($filterOption->getViewName());
                    $libOption->setCount($filterOption->getCount());
                    $libOptions[] = $libOption;
                }
                $libFilter->setOptions($libOptions);
                $this->filterCache[] = $libFilter;
            }
        }
        return $this->filterCache;
    }

    /**
     * Additional orders / sortings resulting from this search.
     * May return empty array if the library does not support this feature.
     *
     * @return array Array in the form  ['id' => <Identifier of order>, 'name' => <human readable name>]
     */
    public function getOrders()
    {
        $orders = $this->products->getOrders();
        $collect = [];
        foreach ($orders as $order) {
            $collect[] = ['id' => $order->getId(), 'name' => $order->getViewName()];
        }
        return $collect;
    }
}
