<?php
/**
 * SEMKNOX API - PHP Library
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Library\Semknox;

use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException;

class ApiCommunicator
{
    /**
     * @var \Zend\Http\Client
     */
    protected $httpClient;

    /**
     * @var ApiConfig
     */
    protected $apiConfig;

    /**
     * ApiCommunicator constructor.
     *
     * @param ApiConfig         $apiConfig
     * @param \Zend\Http\Client $httpClient
     */
    public function __construct(
        ApiConfig $apiConfig,
        \Zend\Http\Client $httpClient
    ) {
        $this->apiConfig = $apiConfig;
        $this->httpClient = $httpClient;
        $this->httpClient->setAdapter('\Zend\Http\Client\Adapter\Curl');

        $this->httpClient->setOptions([
            'timeout' => 60
        ]);
    }

    /**
     * Send a POST Request to the API
     *
     * @param string       $requestPath
     * @param array|string $params      Array of paramaters or raw string to set body to
     * @param string       $contentType (Optional) Content type to use. Default is Form-Urlencoded.
     *
     * @return array    JSON result as array
     * @throws ApiException
     * @throws ApiNotAuthenticatedException
     */
    public function requestPost($requestPath, $params = [], $contentType = 'application/x-www-form-urlencoded')
    {
        return $this->request('post', $requestPath, $params, $contentType);
    }

    /**
     * Send a GET Request to the API
     *
     * @param string $requestPath
     * @param array  $params Array of GET Parameters (?key1=value1&key2=value2...)
     *
     * @return array    JSON result as array
     * @throws ApiException
     * @throws ApiNotAuthenticatedException
     */
    public function requestGet($requestPath, $params = [])
    {
        return $this->request('get', $requestPath, $params);
    }

    /**
     * Send a PUT Request to the API
     *
     * @param string       $requestPath
     * @param array|string $params Array of PUT Parameters or raw string to set body to
     *
     * @return array    JSON result as array
     * @throws ApiException
     * @throws ApiNotAuthenticatedException
     */
    public function requestPut($requestPath, $params = [])
    {
        return $this->request('put', $requestPath, $params);
    }

    /**
     * Send a DELETE Request to the API
     *
     * @param string $requestPath
     * @param array  $params Array of parameters
     *
     * @return array    JSON result as array
     * @throws ApiException
     * @throws ApiNotAuthenticatedException
     */
    public function requestDelete($requestPath, $params = [])
    {
        return $this->request('delete', $requestPath, $params);
    }

    /**
     * Send any request to the API
     *
     * @param string       $method          HTTP Method
     * @param string       $requestPath
     * @param array|string $params          Array of parameters or (if not GET or delete) raw string to set
     * @param              $contentType     (Optional) Content type to use. Default is Form-Urlencoded.
     *
     * @return array                        JSON result as array
     * @throws ApiException Thrown if the server returns any other status code other than 200
     * @throws ApiNotAuthenticatedException Thrown if status code is 401, 403 or 404.
     */
    private function request($method, $requestPath, $params, $contentType = 'application/x-www-form-urlencoded')
    {
        try {
            $response = $this->rawRequest($method, $requestPath, $params, $contentType);
        } catch (\Exception $exception) {
            throw new ApiException("There was an error, processing the request (0).", '', 0, $exception);
        }

        if ($response->isForbidden() || $response->getStatusCode() == 401 || $response->isNotFound()) {
            throw new ApiNotAuthenticatedException(
                "The user is not authenticated or the resource was not found.",
                $response->getBody(),
                $response->getStatusCode()
            );
        } elseif (!$response->isOk()) {
            throw new ApiException(
                "There was an error, processing the request (1)",
                $response->getBody(),
                $response->getStatusCode()
            );
        }

        $jsonResult = json_decode($response->getBody(), true);
        if ($jsonResult === null) {
            throw new ApiException(
                "There was an error, processing the request (2)",
                $response->getBody(),
                $response->getStatusCode()
            );
        }
        return $jsonResult;
    }

    /**
     * Send a raw request without error checking
     *
     * @param string       $method       HTTP Method
     * @param string       $requestPath
     * @param array|string $paramsOrBody Array of parameters or (if not GET or delete) raw string to set
     * @param string       $contentType  Content type to use.
     *
     * @return \Zend\Http\Response
     */
    private function rawRequest($method, $requestPath, $paramsOrBody, $contentType)
    {
        $this->httpClient->reset();
        $this->httpClient->setRawBody("");
        $baseParams = [
            'customerId' => $this->apiConfig->getCustomerId(),
            'apiKey' => $this->apiConfig->getApiKey()
        ];
        if (is_array($paramsOrBody)) {
            $params = array_merge($paramsOrBody, $baseParams);
        } else {
            $params = $baseParams;
        }

        // replace :customerId by real customerId in url
        if (strpos($requestPath, ':customerId') !== false) {
            $requestPath = str_replace(':customerId', $this->apiConfig->getCustomerId(), $requestPath);
        }

        $this->httpClient
            ->setUri($this->apiConfig->getApiUrl() . $requestPath)
            ->setHeaders(['Content-Type: '.$contentType])
            ->setMethod($method);

        if (in_array(strtolower($method), ['get', 'delete'])) {
            // If Method is GET or DELETE:
            // Set GET style paramaters
            $this->httpClient->setParameterGet($params);
        } elseif (!is_array($paramsOrBody)) {
            // Else if $paramsOrBody was a string: Set base parameters as get parameters and body to $paramsOrBody
            $this->httpClient->setParameterGet($params);
            $this->httpClient->setRawBody($paramsOrBody);
        } else {
            // Else: Encode all parameters in body
            $body = [];
            foreach ($params as $name => $value) {
                if (is_array($value)) {
                    $value = urlencode(json_encode($value));
                }

                $body[] = $name . '=' . $value;
            }

            $this->httpClient->setRawBody(join('&', $body));
        }

        $this->httpClient->send();
        return $this->httpClient->getResponse();
    }
}
