<?php
/**
 * SEMKNOX API - PHP Library
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Artsiom Bruneuski <artsiom.bruneuski@tudock.de>
 */

namespace Semknox\Semknox\Library\Semknox\Query;

use Semknox\Semknox\Library\Semknox\Api\Data\ProductsInterface;

use Semknox\Semknox\Library\Semknox\Api\Data\ResultInterface;
use Semknox\Semknox\Library\Semknox\Api\Data\ResultInterfaceFactory;

use Semknox\Semknox\Library\Semknox\Api\Data\Result\OrderInterfaceFactory as ResultOrderInterfaceFactory;

use Semknox\Semknox\Library\Semknox\Api\Data\Result\FilterInterfaceFactory as ResultFilterInterfaceFactory;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Framework\DataObject;
use Semknox\Semknox\Library\Semknox\ApiCommunicator;

class Products extends DataObject implements ProductsInterface
{
    /**
     * Cached response array
     * @var array
     */
    protected $response;

    /** @var ApiCommunicator  */
    protected $api;

    /** @var DataObjectHelper  */
    protected $dataObjectHelper;

    /** @var DataObjectProcessor  */
    protected $dataObjectProcessor;

    /** @var ResultInterfaceFactory  */
    protected $resultFactory;

    /** @var ResultOrderInterfaceFactory */
    protected $resultOrderFactory;

    /** @var ResultFilterInterfaceFactory */
    protected $resultFilterFactory;

    /** @var string */
    protected $query;

    /** @var array */
    protected $filters;

    /** @var array */
    protected $rangeFilters;

    /** @var string  */
    protected $order;

    /** @var string  */
    protected $dir;

    /**
     * @var int
     */
    protected $searchBatchSize;

    /**
     * Products constructor.
     * @param ApiCommunicator $apiCommunicator
     * @param DataObjectHelper $dataObjectHelper
     * @param ResultInterfaceFactory $resultFactory
     * @param ResultOrderInterfaceFactory $resultOrderFactory
     * @param ResultFilterInterfaceFactory $resultFilterFactory
     * @param DataObjectProcessor $dataObjectProcessor
     * @param string $query
     * @param array $filters
     * @param array $rangeFilters
     * @param int $searchBatchSize
     * @param string $order
     * @param string $dir
     */
    public function __construct(
        ApiCommunicator $apiCommunicator,
        DataObjectHelper $dataObjectHelper,
        ResultInterfaceFactory $resultFactory,
        ResultOrderInterfaceFactory $resultOrderFactory,
        ResultFilterInterfaceFactory $resultFilterFactory,
        DataObjectProcessor $dataObjectProcessor,
        $query,
        $filters,
        $rangeFilters,
        $searchBatchSize,
        $order,
        $dir
    ) {
        $this->api = $apiCommunicator;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->resultFactory = $resultFactory;
        $this->resultOrderFactory = $resultOrderFactory;
        $this->resultFilterFactory = $resultFilterFactory;
        $this->query = $query;
        $this->filters = $filters;
        $this->rangeFilters = $rangeFilters;
        $this->order = $order;
        $this->dir = $dir;
        $this->searchBatchSize = $searchBatchSize;

        $this->query();
    }

    /**
     * Returns the products as an array ordered by SEMKNOX
     * @return ResultInterface []
     */
    public function getResults()
    {
        return $this->_getData(self::VAR_RESULTS);
    }

    /**
     * Returns the products as an associative array where the keys are product ids.
     * @return ResultInterface []
     */
    public function getResultsIndexed()
    {
        return $this->_getData(self::VAR_RESULTS_INDEXED);
    }

    /**
     * @return ResultInterface []
     */
    public function getZeroResults()
    {
        return $this->_getData(self::VAR_ZERO_RESULTS);
    }

    /**
     * @return mixed
     */
    public function getOrders()
    {
        return $this->_getData(self::VAR_ORDERS);
    }

    /**
     * @return mixed
     */
    public function getFilters()
    {
        return $this->_getData(self::VAR_FILTERS);
    }

    /**
     * Perform the requested query and return the result.
     * @return mixed
     */
    protected function query()
    {
        if (!$this->response) {
            $this->response = [];

            $params = [
                'query' => $this->query,
                // TODO: Add proper paging!
                'limit' => (int)$this->searchBatchSize >= 0 ? (int)$this->searchBatchSize : 100
            ];

            if ($this->order !== null || $this->filters !== null || $this->rangeFilters !== null) {
                if ($this->order !== null) {
                    $params['order'] = urlencode(json_encode([
                        'id' => $this->order,
                        'direction' => $this->dir
                    ]));
                }

                if ($this->filters !== null || $this->rangeFilters !== null) {
                    $params['filters'] = urlencode(json_encode($this->_getFilters()));
                }

                $this->response = $this->api->requestPost('products/search', $params);
            } else {
                $this->response = $this->api->requestGet('products', $params);
            }

            $this->_storeResponse();
        }

        return $this->response;
    }

    /**
     * @return array
     */
    protected function _getFilters()
    {
        $filters = [];
        if ($this->filters !== null) {
            foreach ($this->filters as $id => $values) {
                $filters[] = [
                    'id' => (string)$id,
                    'values' => $values
                ];
            }
        }

        if ($this->rangeFilters !== null) {
            foreach ($this->rangeFilters as $id => $values) {
                list($min, $max) = $values;
                $filters[] = [
                    'id' => (string)$id,
                    'minValue' => $min,
                    'maxValue' => $max
                ];
            }
        }
        return $filters;
    }

    /**
     * Process store response
     * @return void
     */
    protected function _storeResponse()
    {
        if (is_array($this->response)) {
            if (array_key_exists('searchResults', $this->response)) {
                $this->_storeSearchResults($this->response['searchResults']);
            }

            if (array_key_exists('zeroResults', $this->response)) {
                $this->_storeZeroResults($this->response['zeroResults']);
            }

            if (array_key_exists('order', $this->response)) {
                $this->_storeOrders($this->response['order']);
            }

            if (array_key_exists('filters', $this->response)) {
                $this->_storeFilters($this->response['filters']);
            }
        }
    }

    /**
     * @param array $searchResults
     * @return $this
     */
    protected function _storeSearchResults(array $searchResults)
    {
        $results = [];
        $resultsIndexed = [];
        foreach ($searchResults as $searchResults) {
            foreach ($searchResults as $searchResult) {
                /** @var ResultInterface $result */
                $result = $this->resultFactory->create();

                $this->dataObjectHelper->populateWithArray(
                    $result,
                    $searchResult,
                    'Semknox\Semknox\Library\Semknox\Api\Data\ResultInterface'
                );

                if ($result->getHead()) {
                    $results[] = $result;
                    $resultsIndexed[$result->getArticleNumber()] = $result;
                }
            }
        }
        $this->setData(self::VAR_RESULTS, $results);
        $this->setData(self::VAR_RESULTS_INDEXED, $resultsIndexed);

        return $this;
    }

    /**
     * @param array $zeroResults
     * @return $this
     */
    protected function _storeZeroResults(array $zeroResults)
    {
        $results = [];
        foreach ($zeroResults as $zeroResult) {
            $result = $this->resultFactory->create();

            $this->dataObjectHelper->populateWithArray(
                $result,
                $zeroResult,
                'Semknox\Semknox\Library\Semknox\Api\Data\ResultInterface'
            );
            $results[] = $result;
        }
        $this->setData(self::VAR_ZERO_RESULTS, $results);

        return $this;
    }

    /**
     * @param array $orders
     * @return $this
     */
    protected function _storeOrders(array $orders)
    {
        $results = [];
        foreach ($orders as $orderData) {
            $order = $this->resultOrderFactory->create();

            $this->dataObjectHelper->populateWithArray(
                $order,
                $orderData,
                'Semknox\Semknox\Library\Semknox\Api\Data\Result\OrderInterface'
            );
            $results[] = $order;
        }
        $this->setData(self::VAR_ORDERS, $results);

        return $this;
    }

    /**
     * @param array $filters
     * @return $this
     */
    protected function _storeFilters(array $filters)
    {
        $results = [];
        foreach ($filters as $filterData) {
            $filter = $this->resultFilterFactory->create();

            $this->dataObjectHelper->populateWithArray(
                $filter,
                $filterData,
                'Semknox\Semknox\Library\Semknox\Api\Data\Result\FilterInterface'
            );
            $results[] = $filter;
        }
        $this->setData(self::VAR_FILTERS, $results);

        return $this;
    }
}
