<?php
/**
 * SEMKNOX API - PHP Library
 *
 * @copyright   Copyright (c) 2018 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Library\Semknox\Query;

use Semknox\Semknox\Library\Semknox\ApiCommunicator;

/**
 * Class ProductsBatch
 *
 * Batch products update endpoints:
 *
 * - a) GET: /products/clearInput -- clear Input File
 * - b) PUT: /products/batchInput -- batch upload
 * - c) GET: /products/fullUpdate -- initiate fullUpdate
 *
 * @package Semknox\Semknox\Library\Query
 */
class ProductsBatch
{

    /**
     * @var ApiCommunicator
     */
    protected $api;

    /**
     * ProductsBatch constructor.
     *
     * @param ApiCommunicator $apiCommunicator
     */
    public function __construct(
        ApiCommunicator $apiCommunicator
    ) {
        $this->api = $apiCommunicator;
    }

    /**
     * Clears the input file for product upload and starts the full upload sequence.
     * Use @see batchInput to add files.
     *
     * @api
     *
     * @return void
     * @throws \Semknox\Semknox\Library\Exception\ApiException
     * @throws \Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException
     */
    public function clearInput() {
        $this->api->requestGet('products/clearInput');
    }

    /**
     * Add products to the batch import file.
     *
     * @param array $productsArray An array of product data, according to the API specification.
     *
     * @return void
     * @throws \Semknox\Semknox\Library\Exception\ApiException
     * @throws \Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException
     * @api
     *
     */
    public function batchInput($productsArray) {
        $this->uploadProducts(json_encode($productsArray));
    }

    /**
     * Add products to the batch import file.
     *
     * @param string $jsonProductData JSON encoded product data, according to the API specification.
     *
     * @return void
     * @throws \Semknox\Semknox\Library\Exception\ApiException
     * @throws \Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException
     * @api
     *
     */
    public function batchInputByJson($jsonProductData) {
        $this->uploadProducts($jsonProductData);
    }

    /**
     * Delete all products from the search index, and use the products added by batchInput
     * as new search index.
     *
     * @api
     *
     * @return void
     * @throws \Semknox\Semknox\Library\Exception\ApiException
     * @throws \Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException
     */
    public function fullUpdate() {
        $this->api->requestGet('products/fullUpdate');
    }

    /**
     * Upload the data via PUT request
     *
     * @param string $jsonProductData
     *
     * @return void
     * @throws \Semknox\Semknox\Library\Exception\ApiException
     * @throws \Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException
     */
    protected function uploadProducts($jsonProductData)
    {
        $this->api->requestPut(
            'products/batchInput',
            [
                'productsJsonArray' => urlencode($jsonProductData)
            ]
        );
    }
}