<?php
/**
 * SEMKNOX API - PHP Library
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Library\Semknox;

use Semknox\Semknox\Library\Semknox\Api\Data\ProductsInterface;
use Semknox\Semknox\Library\Semknox\Query\CustomerStatus;
use Semknox\Semknox\Library\Semknox\Query\CustomerStatusFactory;
use Semknox\Semknox\Library\Semknox\Query\ProductsBatch;
use Semknox\Semknox\Library\Semknox\Query\ProductsBatchFactory;
use Semknox\Semknox\Library\Semknox\Query\ProductsDelete;
use Semknox\Semknox\Library\Semknox\Query\ProductsDeleteFactory;
use Semknox\Semknox\Library\Semknox\Query\ProductsPutFactory;
use Semknox\Semknox\Library\Semknox\Query\Suggest;
use Semknox\Semknox\Library\Semknox\Query\SuggestFactory;
use Semknox\Semknox\Library\Semknox\Query\ProductsFactory;
use Semknox\Semknox\Library\Semknox\Query\UpdateCcssFactory;
use Semknox\Semknox\Library\Semknox\Query\ProductsPut;
use Semknox\Semknox\Library\Semknox\Query\UpdateCcss;

/**
 * Class SemknoxApi
 *
 * API entrypoint
 *
 * @package Semknox\Semknox\Library
 */
class SemknoxApi
{

    /**
     * @var ApiCommunicator
     */
    protected $api;

    /**
     * @var ApiConfig
     */
    protected $apiConfig;

    /**
     * @var CustomerStatus
     */
    protected $customerStatus;

    /**
     * @var ProductsPut
     */
    protected $productsPut;

    /**
     * @var ProductsDelete
     */
    protected $productsDelete;

    /**
     * @var UpdateCcss
     */
    protected $updateCcss;

    /**
     * @var ProductsBatch
     */
    protected $productsBatch;

    /**
     * SemknoxApi constructor.
     *
     * @param ApiConfigFactory       $apiConfigFactory
     * @param ApiCommunicatorFactory $apiCommunicatorFactory
     * @param CustomerStatusFactory  $customerStatusFactory
     * @param ProductsPutFactory     $productsPutFactory
     * @param ProductsDeleteFactory  $productsDeleteFactory
     * @param ProductsFactory        $productsFactory
     * @param UpdateCcssFactory      $updateCcssFactory
     * @param ProductsBatchFactory   $productsBatchFactory
     */
    public function __construct(
        ApiConfigFactory $apiConfigFactory,
        ApiCommunicatorFactory $apiCommunicatorFactory,
        // DI
        CustomerStatusFactory $customerStatusFactory,
        // DI
        ProductsPutFactory $productsPutFactory,
        // DI
        ProductsDeleteFactory $productsDeleteFactory,
        // DI
        ProductsFactory $productsFactory,
        // DI
        UpdateCcssFactory $updateCcssFactory,
        // DI
        ProductsBatchFactory $productsBatchFactory
    ) {
        $this->apiConfig       = $apiConfigFactory->create();
        $this->api             = $apiCommunicatorFactory->create($this->apiConfig);
        $this->customerStatus  = $customerStatusFactory->create(['apiCommunicator' => $this->api]);
        $this->productsPut     = $productsPutFactory->create(['apiCommunicator' => $this->api]);
        $this->productsDelete  = $productsDeleteFactory->create(['apiCommunicator' => $this->api]);
        $this->updateCcss      = $updateCcssFactory->create(['apiCommunicator' => $this->api]);
        $this->productsFactory = $productsFactory;
        $this->productsBatch   = $productsBatchFactory->create(['apiCommunicator' => $this->api]);
    }

    /**
     * @return ApiConfig
     */
    public function getApiConfig()
    {
        return $this->apiConfig;
    }

    /**
     * Get the status information of the current customer
     * @return CustomerStatus
     */
    public function getCustomerStatus()
    {
        return $this->customerStatus;
    }

    /**
     * Get the list of products by query
     * @param string $query
     * @param array $filters
     * @param array $rangeFilters
     * @param string $order
     * @param string $dir
     * @return ProductsInterface
     */
    public function getProducts($query, $filters, $rangeFilters, $order, $dir)
    {
        return $this->productsFactory->create([
            'apiCommunicator' => $this->api,
            'query' => $query,
            'filters' => $filters,
            'rangeFilters' => $rangeFilters,
            'searchBatchSize' => $this->apiConfig->getSearchBatchSize(),
            'order' => $order,
            'dir' => $dir
        ]);
    }

    /**
     * Get the product update endpoint to upload new products
     * @return ProductsPut
     */
    public function putProducts()
    {
        return $this->productsPut;
    }

    /**
     * Get the product update endpoint to upload new products using the new batch method
     * @return ProductsBatch
     */
    public function putProductsBatch()
    {
        return $this->productsBatch;
    }

    /**
     * Get the product delete endpoint to delete products
     * @return ProductsDelete
     */
    public function deleteProducts()
    {
        return $this->productsDelete;
    }

    /**
     * Update Ccss information
     * @return UpdateCcss
     */
    public function updateCcss()
    {
        return $this->updateCcss;
    }
}
