<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Model\Indexer;

use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Helper\ConfigInterface;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterfaceFactory;
use Semknox\Semknox\Service\Converter\ProductConverterInterface;
use Semknox\Semknox\Service\ProductConverter;
use Semknox\Semknox\Service\UploadCollectionCreator;

/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
class Product implements \Magento\Framework\Indexer\ActionInterface, \Magento\Framework\Mview\ActionInterface
{

    const INDEXER_ID = 'semknox_product_indexer';

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var ProductConverterInterface
     */
    protected $productConverter;

    /**
     * @var \Semknox\Semknox\Helper\ApiConnection
     */
    protected $apiConnection;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var InfoHelperInterfaceFactory
     */
    protected $uploadInfoHelperFactory;

    /**
     * @var ConfigInterface
     */
    protected $config;

    /**
     * @var UploadCollectionCreator
     */
    protected $uploadCollectionCreator;

    /**
     * Product indexer constructor.
     *
     * @param \Psr\Log\LoggerInterface                   $logger
     * @param ApiConnection                              $apiConnection
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param InfoHelperInterfaceFactory                 $uploadInfoHelperFactory
     * @param ProductConverterInterface                  $productConverter
     * @param ConfigInterface                            $config
     * @param UploadCollectionCreator                    $uploadCollectionCreator
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        ApiConnection $apiConnection,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        InfoHelperInterfaceFactory $uploadInfoHelperFactory,
        ProductConverterInterface $productConverter,
        ConfigInterface $config,
        UploadCollectionCreator $uploadCollectionCreator
    ) {
        $this->logger = $logger;
        $this->apiConnection = $apiConnection;
        $this->storeManager = $storeManager;
        $this->uploadInfoHelperFactory = $uploadInfoHelperFactory;
        $this->productConverter = $productConverter;
        $this->config = $config;
        $this->uploadCollectionCreator = $uploadCollectionCreator;
    }

    /**
     * Execute materialization on ids entities
     *
     * @param int[] $ids
     *
     * @return void
     * @api
     */
    public function execute($ids)
    {
        $this->executeList($ids);
    }

    /**
     * Execute full indexation
     *
     * @return void
     */
    public function executeFull()
    {
        foreach ($this->storeManager->getStores() as $store) {
            if (!$this->apiConnection->isAuthenticated($store->getId())
                || $this->config->getDataUploadDisabled($store->getId())
                // Do NOT use the indexer if "full upload strategy" is enabled
                || $this->config->getFullUploadStrategy($store->getId())
            ) {
                continue;
            }
            /** @var InfoHelperInterface $uploadInfoHelper */
            $uploadInfoHelper = $this->uploadInfoHelperFactory->create(['storeId' => $store->getId()]);
            if ($uploadInfoHelper->getState() == InfoHelperInterface::STATE_STOPPED ||
             $uploadInfoHelper->getState() == InfoHelperInterface::STATE_QUEUED ||
             $uploadInfoHelper->getState() == InfoHelperInterface::STATE_PAUSED
            ) {
                $uploadInfoHelper->setState(InfoHelperInterface::STATE_QUEUED);
                $uploadInfoHelper->clearExceptions();
                $this->logger->info(
                    'Product Indexer: Full reindex requested. Queuing initial upload process for store '.
                    $store->getId().'...'
                );
            } else {
                $this->logger->info(
                    'Product Indexer: Full reindex requested. Already running for store '.
                    $store->getId().'...'
                );
            }
        }
    }

    /**
     * Execute partial indexation by ID list
     *
     * @param int[] $ids
     *
     * @return void
     * @throws \Exception
     */
    public function executeList(array $ids)
    {
        $this->logger->info('Product Indexer: Reindex requested.', ['ids' => $ids]);

        try {
            // ...first Collect
            $collectedData = $this->collectDataForProducts($ids);
            // ...then Upload
            $this->uploadCollectedData($collectedData);
        } catch (\Exception $ex) {
            $this->logger->critical('Product Indexer: Reindex for products FAILED!', ['ids' => $ids]);
            throw $ex;
        }
    }

    /**
     * Execute partial indexation by ID
     *
     * @param int $id
     *
     * @return void
     */
    public function executeRow($id)
    {
        $this->executeList([$id]);
    }

    /**
     * @param int[] $ids
     *
     * @return array with following structure:
     * [
     *      "edit" => [
     *          (store_view_id) => [(product data),(product data)],
     *          (store_view_id) => [(product data)]
     *      ],
     *      "delete" => [
     *          (store_view_id) => [(product id),(product id)],
     *          (store_view_id) => [(product id)]
     *      ]
     * ]
     */
    protected function collectDataForProducts($ids)
    {
        $collectedData = [ "update" => [], "delete" => []];

        foreach ($this->getActivatedStoreIds() as $storeId) {
            // Each store needs it's own collection due to visibility settings
            // (1) Load all products, even if not visible of in stock...
            $productCollection = $this->uploadCollectionCreator->execute($storeId, false);
            $productCollection->addIdFilter($ids);

            /** @var \Magento\Catalog\Model\Product $product */
            foreach ($productCollection as $product) {
                if (!array_key_exists($storeId, $collectedData["update"])) {
                    $collectedData["update"][$storeId] = [];
                }
                // (2) ...and check condition here. Only update or delete if the product was eligible for index in the first place
                // TODO: There are edge cases, where this will not index all products correctly. For example products that change visibility may not correctly update/delete.
                if ($this->isProductVisible($product) && $this->checkStockFilter($product)) {
                    // (3) Updated all products that have the correct visibility and stock settings
                    $collectedData["update"][$storeId][] = $this->productConverter->execute($product->getId(), $storeId);
                }
            }

            // (4) Mark all products not in collection for deletion.
            $collectedData["delete"][$storeId] = array_diff($ids, $productCollection->getAllIds());
        }

        return $collectedData;
    }

    /**
     * Is this product visible where it has to be?
     *
     * @param \Magento\Catalog\Model\Product $product
     *
     * @return bool
     */
    private function isProductVisible($product)
    {
        return !empty(array_intersect($product->getVisibleStatuses(), $this->uploadCollectionCreator->getVisibleStatusIds()))
            && array_search($product->getVisibility(), $this->uploadCollectionCreator->getVisibleInSearchIds());
    }

    /**
     * Either show out of stock products or this product has to be in stock
     *
     * @param \Magento\Catalog\Model\Product $product
     *
     * @return bool
     */
    private function checkStockFilter($product)
    {
        return ($this->uploadCollectionCreator->showOutOfStock() || $product->isInStock());
    }

    /**
     * @param array $collectedData For structure @see collectDataForProducts
     *
     * @return void
     */
    protected function uploadCollectedData($collectedData)
    {
        // UPDATE
        foreach ($collectedData["update"] as $storeId => $products) {
            try {
                $this->apiConnection->get($storeId)
                    ->update()
                    ->addProducts($products);
                $this->logger->info('Product Indexer: Updated products for store '. $storeId);
            } catch (ApiException $ex) {
                $this->logger->alert(
                    'Product Indexer: Reindex for some products (see list) in store ' .
                    $storeId . ' FAILED! API communication error. Skipping products... ' . $ex,
                    ['ids' => $products]
                );
            }
        }

        // DELETE
        foreach ($collectedData["delete"] as $storeId => $products) {
            if (empty($products)) {
                continue;
            }
            $updateEndpoint = $this->apiConnection->get($storeId)
                ->update();
            foreach ($collectedData["delete"][$storeId] as $product) {
                try {
                    $updateEndpoint->deleteProduct($product);
                    $this->logger->info('Product Indexer: Deleted product '.$product.' in store '.$storeId);
                } catch (ApiException $ex) {
                    $this->logger->alert(
                        'Product Indexer: Deletion of product '. $product .' in store ' .
                        $storeId . ' FAILED! API communication error. Skipping deletion... ' . $ex
                    );
                }
            }
        }
    }

    /**
     * Returns all stores that the store manager knows of and that are enabled for Search Service use
     * @return array
     */
    protected function getActivatedStoreIds()
    {
        $stores = [];
        foreach ($this->storeManager->getStores() as $store) {
            // Don't collect data for deactivate / unavailable stores
            if (!$this->apiConnection->isAuthenticated($store->getId())
                || $this->config->getDataUploadDisabled($store->getId())) {
                continue;
            }
            $stores[] = $store->getId();
        }
        return $stores;
    }
}
