<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Artsiom Bruneuski <artsiom.bruneuski@tudock.de>
 */
namespace Semknox\Semknox\Model\Layer\Filter;

use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Helper\StateFilter;
use Semknox\Semknox\Library\SearchResult\LibFilterInterface;
use Semknox\Semknox\Model\Layer\FilterList;

class Attribute extends \Magento\Catalog\Model\Layer\Filter\AbstractFilter
{
    /**
     * @var LibFilterInterface
     */
    protected $_filter;

    /**
     * @var \Magento\Framework\Escaper
     */
    protected $_escaper;

    /**
     * @var ApiConnection
     */
    protected $_apiConnection;

    /** @var StateFilter  */
    protected $_stateFilter;

    /**
     * A list of item values that are active
     * @var array
     */
    protected $_markedAsActive;

    /**
     * Attribute constructor.
     * @param \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\Layer $layer
     * @param \Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder
     * @param \Magento\Framework\Escaper $escaper
     * @param LibFilterInterface $filter
     * @param ApiConnection $apiConnection
     * @param StateFilter $stateFilter
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\Layer $layer,
        \Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder,
        \Magento\Framework\Escaper $escaper,
        LibFilterInterface $filter,
        ApiConnection $apiConnection,
        StateFilter $stateFilter,
        array $data = []
    ) {
        $this->_filter = $filter;
        $this->_escaper = $escaper;
        $this->_apiConnection = $apiConnection;
        $this->_stateFilter = $stateFilter;
        $this->_requestVar = StateFilter::SEARCH_REQUEST_PARAM .
            StateFilter::SEARCH_REQUEST_PARAM_DELIMITER .
            $this->_filter->getId();
        $this->_markedAsActive = [];
        parent::__construct($filterItemFactory, $storeManager, $layer, $itemDataBuilder, $data);
    }

    /**
     * Initialize filter items
     *
     * @return  \Magento\Catalog\Model\Layer\Filter\AbstractFilter
     */
    protected function _initItems()
    {
        $data = $this->_getItemsData();
        $items = [];
        foreach ($data as $itemData) {
            // The item is active if it's label was added to the markedAsActive list by "apply".
            $isActive = array_key_exists($itemData['label'], $this->_markedAsActive);
            $item = $this->_createItem($itemData['label'], $itemData['value'], $itemData['count']);
            $item->setData('sknox_active', $isActive);
            $items[] = $item;
        }
        $this->_items = $items;
        return $this;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_filter->getName();
    }

    /**
     * @return array
     */
    protected function _getItemsData()
    {
        $optionVals = $this->_stateFilter->getSearchAppliedFilter($this->_filter);

        foreach ($this->_filter->getOptions() as $option) {
            // The item is active if it's label was added to the markedAsActive list by "apply".
            $isActive = array_key_exists($option->getName(), $this->_markedAsActive);

            if (!$isActive) {
                $optionValuesForItem = array_merge([$option->getId()], $optionVals);
            } else {
                // If active don't add this filter id to the value set, instead remove this filter,
                // so if the button on layered navigation is pressed this item get's removed instead of added.
                $optionValuesForItem = array_filter($optionVals, function ($optionVal) use ($option) {
                    return $optionVal != $option->getId();
                });
            }

            $this->itemDataBuilder->addItemData(
                $this->_escaper->escapeHtml($option->getName()),
                implode(',', $optionValuesForItem),
                $option->getCount()
            );
        }

        return $this->itemDataBuilder->build();
    }

    /**
     * @param \Magento\Framework\App\RequestInterface $request
     *
     * @return $this
     */
    public function apply(\Magento\Framework\App\RequestInterface $request)
    {
        $optionVals = $this->_stateFilter->getSearchAppliedFilter($this->_filter);
        $optionNames = [];

        if (count($optionVals) === 0) {
            return $this;
        }

        foreach ($this->_filter->getOptions() as $option) {
            if (in_array($option->getId(), $optionVals)) {
                $optionNames[] = $option->getName();
                // Add this to a list that marks items as active. Used to build the filter set for the layer navigation
                $this->_markedAsActive[$option->getName()] = $option->getId();
            }
        }

        $this->getLayer()
            ->getState()
            ->addFilter($this->_createItem(implode(', ', $optionNames), implode(',', $optionVals)));

        return parent::apply($request);
    }

    /**
     * @param \Magento\Catalog\Model\Layer\Filter\Item $item
     *
     * @return bool
     */
    public function itemIsApplied($item)
    {
        foreach ($this->getLayer()->getState()->getFilters() as $appliedFilterItem) {
            if ($appliedFilterItem == $item) {
                return true;
            }
        }
        return false;
    }
}
