<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Artsiom Bruneuski <artsiom.bruneuski@tudock.de>
 */

namespace Semknox\Semknox\Model\Layer\Filter;

use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Helper\StateFilter;
use Semknox\Semknox\Library\SearchResult\LibFilterInterface;

class Range extends Attribute
{
    const MIN_RANGE_POWER = 10;

    /** @var \Magento\Framework\Locale\FormatInterface  */
    protected $localeFormat;

    /**
     * Range constructor.
     * @param \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\Layer $layer
     * @param \Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder
     * @param \Magento\Framework\Escaper $escaper
     * @param \Magento\Framework\Locale\FormatInterface $localeFormat
     * @param LibFilterInterface $filter
     * @param ApiConnection $apiConnection
     * @param StateFilter $stateFilter
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Model\Layer\Filter\ItemFactory $filterItemFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\Layer $layer,
        \Magento\Catalog\Model\Layer\Filter\Item\DataBuilder $itemDataBuilder,
        \Magento\Framework\Escaper $escaper,
        \Magento\Framework\Locale\FormatInterface $localeFormat,
        LibFilterInterface $filter,
        ApiConnection $apiConnection,
        StateFilter $stateFilter,
        array $data = []
    ) {
        $this->localeFormat = $localeFormat;
        parent::__construct(
            $filterItemFactory,
            $storeManager,
            $layer,
            $itemDataBuilder,
            $escaper,
            $filter,
            $apiConnection,
            $stateFilter,
            $data
        );
        $this->_requestVar = StateFilter::SEARCH_REQUEST_RANGE_PARAM .
            StateFilter::SEARCH_REQUEST_PARAM_DELIMITER .
            $this->_filter->getId();
    }

    /**
     * @param number $range
     * @return array
     */
    protected function getRanges($range)
    {
        $ranges = [];
        if ($this->_filter->getMax() > $range) {
            $fromRange = ceil($this->_filter->getMin() / $range) * $range;
            $toRange = floor($this->_filter->getMax() / $range) * $range;
            $fromStep = $this->_filter->getMin();
            $toStep = null;

            if ($fromRange < $toRange) {
                foreach (range($fromRange, $toRange, $range) as $toStep) {
                    if ($fromStep != $toStep) {
                        $ranges[$fromStep . ',' . $toStep] =
                            $this->priceCurrency->format($fromStep) . ' - ' .
                            $this->priceCurrency->format($toStep);
                    }
                    $fromStep = $toStep;
                }
            }
            $ranges[$toStep . ',' . $this->_filter->getMax()] =
                $this->priceCurrency->format($toStep) . ' - ' .
                $this->priceCurrency->format($this->_filter->getMax());
        }

        return $ranges;
    }

    /**
     * @return int|number
     */
    protected function getRange()
    {
        $range = 0;
        $index = 1;
        do {
            $range = pow(10, strlen(floor($this->_filter->getMax())) - $index);
            $index++;
        } while ($range > self::MIN_RANGE_POWER && count($this->getRanges($range)) < 2);

        return $range;
    }

    /**
     * Retrieve data for build decimal filter items
     *
     * @return array
     */
    protected function _getItemsData()
    {
        $this->itemDataBuilder->addItemData(
            $this->_filter->getName(),
            'from_to',
            null
        );
        return $this->itemDataBuilder->build();
    }

    /**
     * @param \Magento\Framework\App\RequestInterface $request
     * @return $this
     */
    public function apply(\Magento\Framework\App\RequestInterface $request)
    {
        $optionVals = $this->_stateFilter->getSearchAppliedRangeFilter($this->_filter);
        $optionNames = [];

        if (count($optionVals) === 0) {
            return $this;
        }

        if (count($optionVals) > 0) {
            list($from, $to) = $optionVals;
        }

        $label = $this->localeFormat->getNumber($from) . ' ' . $this->_filter->getUnitName() . ' - ' .
            $this->localeFormat->getNumber($to) . ' ' . $this->_filter->getUnitName();

        $this->getLayer()
            ->getState()
            ->addFilter(
                $this->_createItem(
                    $label,
                    implode(',', $optionVals)
                )
            );

        return parent::apply($request);
    }

    /**
     * @return int
     */
    public function getMax()
    {
        return $this->_filter->getMax();
    }

    /**
     * @return int
     */
    public function getMin()
    {
        return $this->_filter->getMin();
    }

    /**
     * @return int
     */
    public function getFrom()
    {
        $from = $this->getMin();
        $optionVals = $this->_stateFilter->getSearchAppliedRangeFilter($this->_filter);

        if (count($optionVals) > 0) {
            list($from, $to) = $optionVals;
        }

        return $from;
    }

    /**
     * @return int
     */
    public function getTo()
    {
        $to = $this->getMax();
        $optionVals = $this->_stateFilter->getSearchAppliedRangeFilter($this->_filter);

        if (count($optionVals) > 0) {
            list($from, $to) = $optionVals;
        }

        return $to;
    }

    /**
     * @return string
     */
    public function getUnitName()
    {
        return $this->_filter->getUnitName();
    }
}
