<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Plugin\CatalogSearch\Controller\Result;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\StringUtils;
use Magento\Search\Model\QueryFactory;
use Semknox\Semknox\Helper\ConfigInterface;

/**
 * Class Index
 *
 * This plugin for the CatalogSearch Result Index Action adds noroute handling ability to
 * the action.
 *
 * @package Semknox\Semknox\Plugin\CatalogSearch\Controller\Result
 */
class Index
{

    const SEPARATOR = '-';

    /**
     * @var Context
     */
    protected $context;

    /**
     * @var StringUtils
     */
    protected $string;

    /**
     * @var ConfigInterface
     */
    protected $config;

    /**
     * The Context may return another Request object in theory, but in this context, only the HTTP request can
     * be the one that is actually returned.
     * There is no other way of reliably getting the original request data otherwise. The Base Router also assumes this!
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * @var \Magento\Backend\Model\View\Result\ForwardFactory
     */
    protected $resultForwardFactory;

    /**
     * This static variable prevents the shop from being stuck in an endless loop, in case the configured noroute action
     * doesn't exist.
     * @var bool
     */
    protected static $loopDetector = false;

    /**
     * @var string
     */
    protected $queryPrefix;

    /**
     * Index constructor.
     *
     * @param Context                                           $context
     * @param StringUtils                                       $string
     * @param ConfigInterface                                   $config
     * @param \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
     */
    public function __construct(
        Context $context,
        StringUtils $string,
        ConfigInterface $config,
        \Magento\Backend\Model\View\Result\ForwardFactory $resultForwardFactory
    ) {
        $this->context = $context;
        $this->request = $context->getRequest();
        $this->string = $string;
        $this->config = $config;
        $this->resultForwardFactory = $resultForwardFactory;

        $this->queryPrefix = $this->config->getSeoUrlPrefix() . self::SEPARATOR;
    }

    /**
     * Scan the URL, see if it's a noroute action and if so, check if it is supposed to be
     * a search URL. If so, build a query object from that, otherwise redirect to configured noroute.
     *
     * @param \Magento\CatalogSearch\Controller\Result\Index $subject
     *
     * @param callable                                       $proceed
     *
     * @return ResultInterface|null
     * @throws LocalizedException
     */
    public function aroundExecute(\Magento\CatalogSearch\Controller\Result\Index $subject, callable $proceed)
    {
        // noroute fallback (see below)
        if (self::$loopDetector) {
            return $this->fallbackNoroute();
        }
        self::$loopDetector = true;

        $reuqestPathParts = explode('/', trim($this->request->getPathInfo(), '/'));

        if ($this->config->getSeoUrlEnabled() && $this->isSeoUrlSearchRequest($reuqestPathParts)) {
            $this->fillQueryParameters($reuqestPathParts);
            return $proceed();
        } elseif ($this->isOriginalControllerSearchRequest($reuqestPathParts)) {
            return $proceed();
        }

        // else: Do NOT proceed, instead silently forward to the configured noroute.
        // This may forward again into this method, in this case the loop detector above will use a fallback.
        return $this->noroute();
    }

    /**
     * Scan the Request Path Parts and determine if this is a noroute search action or not
     * (url has the form /s-my-search where s is the configured prefix).
     *
     * @param array $reuqestPathParts
     *
     * @return bool
     */
    private function isSeoUrlSearchRequest($reuqestPathParts)
    {
        return $this->queryPrefixGet($reuqestPathParts[0]) == $this->queryPrefix;
    }

    /**
     * Scan the URL and see if it directly points to the Action's route (catalogsearch/result[/index].
     *
     * @param array $reuqestPathParts
     *
     * @return bool
     */
    private function isOriginalControllerSearchRequest($reuqestPathParts)
    {
        return ($reuqestPathParts[0] == $this->request->getRouteName()
                && $reuqestPathParts[1] == $this->request->getControllerName()
                && (count($reuqestPathParts) < 3 || $reuqestPathParts[2] == $this->request->getActionName()));
    }

    /**
     * Fill the HTTP request parameter used for the search term with the data from the request path.
     *
     * @param array $reuqestPathParts
     *
     * @return void
     */
    private function fillQueryParameters($reuqestPathParts)
    {
        $this->request->setParam(
            QueryFactory::QUERY_VAR_NAME,
            $this->queryStringConverter(
                $this->queryPrefixRemove(
                    $reuqestPathParts[0]
                )
            )
        );
    }

    /**
     * Return the part of the query string that should match (==) the configured SEO URL prefix.
     * (If it doesn't then this is not a search action).
     *
     * @param string $queryString
     *
     * @return string
     */
    private function queryPrefixGet($queryString)
    {
        return $this->string->substr(
            $queryString,
            0,
            $this->string->strlen($this->queryPrefix)
        );
    }

    /**
     * Opposite of @see queryPrefixGet
     *
     * @param string $queryString
     *
     * @return string
     */
    private function queryPrefixRemove($queryString)
    {
        return $this->string->substr(
            $queryString,
            $this->string->strlen($this->queryPrefix)
        );
    }

    /**
     * Prepare the query string to be used by the CatalogSearch engine.
     *
     * @param string $queryString
     *
     * @return string
     */
    private function queryStringConverter($queryString)
    {
        // TODO: Might consider adding a way to escape - characters.
        return urldecode(str_replace(
            '-',
            ' ',
            $queryString
        ));
    }

    /**
     * Forward to configured noroute.
     *
     * @return ResultInterface
     */
    private function noroute()
    {
        /** @var \Magento\Framework\Controller\Result\Forward $resultForward */
        $resultForward = $this->resultForwardFactory->create();
        $noRouteInformation = explode('/', $this->config->getNoRoute());
        $resultForward->setModule($noRouteInformation[0]);
        $resultForward->setController($noRouteInformation[1]);
        return $resultForward->forward($noRouteInformation[2]);
    }

    /**
     * Forward to fallback noroute.
     *
     * @return ResultInterface
     */
    private function fallbackNoroute()
    {
        /** @var \Magento\Framework\Controller\Result\Forward $resultForward */
        $resultForward = $this->resultForwardFactory->create();
        $resultForward->setModule('cms');
        $resultForward->setController('index');
        return $resultForward->forward('defaultNoRoute');
    }
}
