<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Plugin\Framework;

use Magento\Framework\App\Helper\Context;
use Magento\Search\Model\QueryFactory;
use Semknox\Semknox\Helper\ConfigInterface;

/**
 * Class Url
 *
 * Plugin for the Url generator, to translate catalogsearch/result/index URLs to SEO URLs if SEO is enabled.
 *
 * @package Semknox\Semknox\Plugin\Framework
 */
class Url
{

    /**
     * @var ConfigInterface
     */
    protected $config;

    /**
     * @var Context
     */
    protected $context;

    /**
     * Url constructor.
     *
     * @param Context $context
     * @param ConfigInterface  $config
     */
    public function __construct(
        Context $context,
        ConfigInterface $config
    ) {
        $this->context = $context;
        $this->config = $config;
    }

    /**
     * Transform the URL if it's for the catalogsearch engine.
     *
     * @param \Magento\Framework\Url $subject
     * @param string                 $result
     *
     * @return string
     */
    public function afterGetUrl(\Magento\Framework\Url $subject, $result)
    {
        if ($this->isForSearchPage($result) && $this->config->getSeoUrlEnabled()) {
            return $this->transformUrl($result);
        }
        return $result;
    }

    /**
     * Detect if the URL was for the catalogsearch endpoint
     *
     * @param string $routePath
     *
     * @return bool
     */
    protected function isForSearchPage($routePath)
    {
        return strpos($routePath, '/catalogsearch/result/index') !== false;
    }

    /**
     * Transform a "vanilla" catalogsearch URL into an optimized SEO URL.
     *
     * @param string $originalUrl
     *
     * @return string
     */
    protected function transformUrl($originalUrl)
    {
        $baseUrl = $this->context->getUrlBuilder()->getBaseUrl();
        $newUrl = str_replace($baseUrl, '', $originalUrl);

        // Take URL apart
        $urlParts = explode('/', $newUrl);
        $queryStringPosition = array_search(QueryFactory::QUERY_VAR_NAME, $urlParts);
        $queryString = $queryStringPosition && array_key_exists($queryStringPosition+1, $urlParts)
            ? $this->convertToQueryString($urlParts[$queryStringPosition+1]) : '';

        // GET-Parameters
        $getParameters = '';
        $getParametersPos = count($urlParts) - 1;
        if ($getParametersPos > 0
            && $urlParts[$getParametersPos] != ''
            && $urlParts[$getParametersPos][0] == '?'
        ) {
            $getParameters = $urlParts[count($urlParts) - 1];
        }

        return $baseUrl . $this->config->getSeoUrlPrefix() . '-' . $queryString . '/' . $getParameters;
    }

    /**
     * Convert a sentence into query string parameters for the SEO search URLs.
     *
     * @param string $urlPart
     *
     * @return string
     */
    protected function convertToQueryString($urlPart)
    {
        return urlencode(str_replace(' ', '-', $urlPart));
    }
}
