<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Artsiom Bruneuski <artsiom.bruneuski@tudock.de>
 */

namespace Semknox\Semknox\Plugin\Search\Adapter\Mysql\Query\Builder;

use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Helper\ApiConnection;
use Magento\Framework\Search\Adapter\Mysql\TemporaryStorageFactory;
use Magento\Framework\Api\Search\DocumentInterfaceFactory;
use Magento\Framework\Api\AttributeInterfaceFactory;

use Magento\Framework\Search\Adapter\Mysql\Query\Builder\Match as MysqlQueryBuilderMatch;
use Magento\Framework\Search\Adapter\Mysql\ScoreBuilder;

class Match
{
    /** @var ApiConnection  */
    protected $apiConnection;

    /** @var TemporaryStorageFactory  */
    protected $temporaryStorageFactory;

    /** @var DocumentInterfaceFactory  */
    protected $documentFactory;

    /** @var AttributeInterfaceFactory  */
    protected $attributeFactory;

    /** @var \Psr\Log\LoggerInterface */
    protected $logger;

    /**
     * Match constructor.
     *
     * @param ApiConnection             $apiConnection
     * @param TemporaryStorageFactory   $temporaryStorageFactory
     * @param DocumentInterfaceFactory  $documentFactory
     * @param AttributeInterfaceFactory $attributeFactory
     * @param \Psr\Log\LoggerInterface  $logger
     */
    public function __construct(
        ApiConnection $apiConnection,
        TemporaryStorageFactory $temporaryStorageFactory,
        DocumentInterfaceFactory $documentFactory,
        AttributeInterfaceFactory $attributeFactory,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->apiConnection = $apiConnection;
        $this->temporaryStorageFactory = $temporaryStorageFactory;
        $this->documentFactory = $documentFactory;
        $this->attributeFactory = $attributeFactory;
        $this->logger = $logger;
    }

    /**
     * @param MysqlQueryBuilderMatch $builderMatch
     * @param \Closure $proceed
     * @param ScoreBuilder $scoreBuilder
     * @param \Magento\Framework\DB\Select $select
     * @param \Magento\Framework\Search\Request\Query\Match $query
     * @param string $conditionType
     * @return \Magento\Framework\DB\Select
     */
    public function aroundBuild(
        MysqlQueryBuilderMatch $builderMatch,
        \Closure $proceed,
        ScoreBuilder $scoreBuilder,
        \Magento\Framework\DB\Select $select,
        \Magento\Framework\Search\Request\Query\Match $query,
        $conditionType
    ) {

        try {
            if ($search = $this->apiConnection->getSearch()) {
                $fieldIds = [];

                if (count($search->getResultProductIds()) > 0) {
                    foreach ($search->getResultProductIds() as $result) {
                        $fieldIds[] = $result;
                    }
                }

                $table = $this->storeInTemporaryTable($fieldIds);

                $select->joinInner(
                    [
                        'semknox_index' => $table->getName(),
                    ],
                    'search_index.entity_id = semknox_index.' .
                    \Magento\Framework\Search\Adapter\Mysql\TemporaryStorage::FIELD_ENTITY_ID,
                    []
                );

                $scoreBuilder->addCondition(
                    'semknox_index.' . \Magento\Framework\Search\Adapter\Mysql\TemporaryStorage::FIELD_SCORE,
                    false
                );

                return $select;
            } else {
                return $proceed($scoreBuilder, $select, $query, $conditionType);
            }
        } catch (ApiException $apiException) {
            // fall back to original MySQL search on errors
            $this->logger->warning(
                'Server connection failed (see attached exception).' .
                'The fallback search engine was used instead.',
                [
                    'exception' => $apiException
                ]
            );
            return $proceed($scoreBuilder, $select, $query, $conditionType);
        }
    }

    /**
     * Write data from array to mysql temporary table
     * @param array $fieldIds
     * @return \Magento\Framework\DB\Ddl\Table
     */
    protected function storeInTemporaryTable(array $fieldIds)
    {
        $documents = [];

        $score = count($fieldIds);

        foreach ($fieldIds as $fieldId) {

            /** @var \Magento\Framework\Api\AttributeInterface $attribute */
            $attribute = $this->attributeFactory->create();

            $attribute->setAttributeCode('score')
                ->setValue($score);

            /** @var \Magento\Framework\Api\Search\DocumentInterface $document */
            $document = $this->documentFactory->create()
                ->setId($fieldId);

            $document->setCustomAttribute('score', $attribute);

            $documents[] = $document;
            $score--;
        }

        $temporaryStorage = $this->temporaryStorageFactory->create();
        return $temporaryStorage->storeApiDocuments($documents);
    }
}
