<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service\Converter;

use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\Entity\Attribute;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Catalog\Pricing\Price\FinalPrice;
use Magento\Framework\App\State;
use Magento\Framework\Exception\LocalizedException;
use Magento\Setup\Exception;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use \Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Semknox\Semknox\Service\Converter\ProductConverterInterface;
use Semknox\Semknox\Helper\Config;

/**
 * Class ProductConverter
 *
 * Converts a product model to an array compliant to the Search Service API.
 *
 * @package Semknox\Semknox\Service
 */
class ProductConverter implements ProductConverterInterface
{

    const ALWAYS_VISIBLE_ATTRIBUTES = ['sku', 'manufacturer', 'brand'];
    protected static $wasStoreEmulationWarned = false;

    /**
     * A cache to store store label information to increase performance
     */
    protected static $storeLabelCache = [];

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var string Store Currency Code
     */
    protected $currency;

    /**
     * @var \Magento\Catalog\Helper\Image
     */
    protected $imageHelper;

    /**
     * @var CollectionFactory
     */
    protected $categoryCollectionFactory;

    /**
     * @var Configurable
     */
    protected $configurable;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var \Magento\Store\Model\App\Emulation
     */
    protected $emulation;

    /**
     * @var State
     */
    protected $appState;

    /**
     * ProductConverter constructor.
     *
     * @param StoreManagerInterface         $storeManager
     * @param \Magento\Catalog\Helper\Image $imageHelper
     * @param CollectionFactory             $categoryCollectionFactory
     * @param LoggerInterface               $logger
     * @param Configurable                  $configurable
     * @param ProductRepositoryInterface    $productRepository
     * @param Config                        $config
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        \Magento\Catalog\Helper\Image $imageHelper,
        CollectionFactory $categoryCollectionFactory,
        LoggerInterface $logger,
        Configurable $configurable,
        ProductRepositoryInterface $productRepository,
        Config $config,
        \Magento\Store\Model\App\Emulation $emulation,
        State $appState
    ) {
        $this->storeManager = $storeManager;
        $this->imageHelper = $imageHelper;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->logger = $logger;
        $this->configurable = $configurable;
        $this->productRepository = $productRepository;
        $this->config = $config;
        $this->emulation = $emulation;
        $this->appState = $appState;
    }

    /**
     * Get the store label information from a cache array
     *
     * @param string        $attributeName
     * @param Attribute     $attribute
     * @param int           $storeId
     *
     * @return mixed
     */
    private static function getAttributeLabelFromCache($attributeName, $attribute, $storeId)
    {
        if (!array_key_exists($storeId, self::$storeLabelCache)) {
            self::$storeLabelCache[$storeId] = [];
        }
        if (!array_key_exists($attributeName, self::$storeLabelCache[$storeId])) {
            self::$storeLabelCache[$storeId][$attributeName] =
                ($attribute->getStoreLabel()) ? $attribute->getStoreLabel() : '';
        }
        return self::$storeLabelCache[$storeId][$attributeName];
    }

    /**
     * Initialize the conversion process.
     * @param int $storeId
     * @return void
     */
    private function init($storeId)
    {
        /** @var Store $store */
        $store = $this->storeManager->getStore($storeId);
        $this->currency =  $store->getCurrentCurrencyCode();

        // TODO: Implement? See note at execute.
        //$this->_passOnAttributes = $this->_helper->semknoxPassOnAttributes();
    }

    /**
     * Convert a single product to Search Service compatible data.
     *
     * @param int     $productId
     * @param int     $storeId
     *
     * @return array A Search Service-API compliant product data array
     */
    public function execute($productId, $storeId)
    {
        $this->init($storeId);

        // Reload product to get all data
        /** @var Product $product */
        $product = $this->productRepository->getById($productId, false, $storeId);

        $url = $product->getProductUrl();

        // The price can only be accurately be retrieved when using Store emulation
        // Store emulation does not work when using Magerun 2, due to it's App State Emulation, which conflicts with store emulation
        // We ignore store emulation issues here and log them once.
        // see: N98/Magento/Command/System/Cron/RunCommand.php@58
        //      vendor/magento/framework/App/State.php::emulateAreaCode
        //      vendor/magento/module-theme/Model/Theme.php::getArea
        if (!$this->appState->isAreaCodeEmulated()
            || $this->appState->getMode() != State::MODE_DEVELOPER
        ) {
            $this->emulation->startEnvironmentEmulation($storeId);
            $price = number_format($product->getPriceInfo()->getPrice(FinalPrice::PRICE_CODE)->getMinimalPrice()->getValue(), 2, ',', '');
            $this->emulation->stopEnvironmentEmulation();
        } {
            // Only allow not using store emulation in developer mode
        if (!self::$wasStoreEmulationWarned) {
            $this->logger->warning(
                "Tudock Search Product Converter: 
                    Could not use store emulation. Price info might be incorrect. 
                    Store emulation does not work when using Magerun2 or App State emulation. 
                    This is only allowed in developer mode.
                    This warning will not be repeated."
            );
            self::$wasStoreEmulationWarned = true;
        }
            $price = number_format($product->getPriceInfo()->getPrice(FinalPrice::PRICE_CODE)->getMinimalPrice()->getValue(), 2, ',', '');
        }

        $data = [
            'name'              => $product->getName() ?: '',
            'articleNumber'     => $productId,
            'price'             => $price . '' . $this->currency,
            'image'             => $this->getMainProductImage($product, $storeId),
            'description'       => $product->getDescription(),
            'deeplink'          => $url,
            'link'              => $url,
            'attributes'        => $this->getAttributes($product, $storeId),
            'category'          => $this->getCategoryName($product),
            'passOn'            => [
                [
                    "key" => "typeId",
                    "value" => $product->getTypeId()
                ]
            ]
        ];

        $this->processSpecialRules($product, $data);

        return $data;
    }

    /**
     * TODO
     *
     * @param Product $product
     *
     * @param         $storeId
     *
     * @return array|string Image URL
     */
    private function getMainProductImage($product, $storeId)
    {
        $cdn = false;
        /* TODO: Implement
        if($this->_helper->semknoxImageCdn() && strlen($this->_helper->semknoxImageCdn()) > 5)
        {
        $cdn = $this->_helper->semknoxImageCdn();
        }*/

        try {
            $width = 210;

            $image = $this->imageHelper->init(
                $product,
                'thumbnail',
                ['type' => 'small_image']
            )
            ->keepAspectRatio(true)
            ->constrainOnly(false)
            ->keepFrame(false)
            ->resize($width, $width)
            ->getUrl();

            if ($this->config->getRemovePubFromUrl($storeId)) {
                $image = str_replace('pub/', '', $image);
            }

            if ($cdn) {
                $image = explode('/media/', $image, 2);
                if (count($image) == 2) {
                    $image[0] = $cdn;
                }
                $image = implode('/media/', $image);
            }
        } catch (\Exception $e) {
            $image = '';
        }

        return $image;
    }

    /**
     * Get all attributes for the current product
     *
     * @param Product $product
     * @param int     $storeId
     *
     * @return array
     * @todo Is it really necessary to load all attributes again?
     * @todo Merge this and translateAttributeKeys since iterating over everything twice is nonsense.
     */
    protected function getAttributes($product, $storeId)
    {
        $result = [];
        $attributes = $product->getAttributes();

        foreach ($attributes as $attributeName => $attribute) {
            /** @var Attribute $attribute */

            // This attribute is broken: https://github.com/magento/magento2/issues/7080
            if ($attributeName == 'quantity_and_stock_status') {
                continue;
            }

            $key   = $attributeName;
            $value = $attribute->getFrontend()->getValue($product);

            // add to result
            if ($value) {
                $valueArray = $this->prepareAttributeValueArray($value);

                $visible = $this->isAttributeVisible($attributeName, $attribute);
                $storeLabel = self::getAttributeLabelFromCache($attributeName, $attribute, $storeId);

                foreach ($valueArray as $idx => $value) {
                    $resultAttribute['value'] = $value;
                    $resultAttribute['storeLabel'] = $storeLabel;
                    $resultAttribute['visible'] = $visible;
                    $resultAttribute['key'] = $key;
                    $resultAttribute['idx'] = $idx;

                    $result[$key.$idx] = $resultAttribute;
                }
            }
        }

        $result = $this->translateAttributeKeys($result);

        return $result;
    }

    /**
     * TODO
     *
     * @param mixed $value
     *
     * @return array
     */
    protected function prepareAttributeValueArray($value)
    {
        if (is_array($value)) {
            return [];
        }

        $valueArray = preg_split('/<br[^>]*>/i', $value);
        if (!is_array($valueArray)) {
            $valueArray[] = $value;
        }

        return $valueArray;
    }

    /**
     * TODO
     *
     * @param string    $attributeName
     * @param Attribute $attribute
     *
     * @return bool
     */
    protected function isAttributeVisible($attributeName, $attribute)
    {
        // Explicitly visible attributes, other may be visible by being visible on front or may be added via pass on.
        $allowedAttributeNames = array_merge(self::ALWAYS_VISIBLE_ATTRIBUTES);

        return $attribute->getIsVisibleOnFront() or in_array($attributeName, $allowedAttributeNames);
    }

    /**
     * Convert the attribute information to be understandable by the Search Service.
     *
     * Generally, this is the transformation:
     *  - Input: "attribute_name":
     *      {"value":"a value","storeLabel":"Name In Store","visible":boolean,"key":"attribute_name","idx":number}
     *  - Output: "Name In Store": "value"
     *
     * If idx is > 0, the output key will get a number appended:
     *  - Input: "attribute_name":
     *      {"value":"a value","storeLabel":"Name In Store","visible":boolean,"key":"attribute_name","idx":1}
     *  - Output: "Name In Store 2": "value"
     *
     * If invisible,  sxNotVisible will be appended to the code.
     *
     * @param array $attributes
     *
     * @deprecated @todo Merge this with getAttributes
     *
     * @return array
     *
     */
    protected function translateAttributeKeys($attributes)
    {
        // TODO: Implement matching?

        //$matching = Mage::getStoreConfig('semknoxps/advancedsettings/attributeMatch');
        //$matchingRaw = unserialize($matching);
        //$matching = array();

        //foreach((array) $matchingRaw as $data) $matching[$data['magento']] = $data['api'];

        $sxNotVisible = [];
        $sxVisible = [];

        foreach ($attributes as $k => $value) {
            $code = $value['key'];
            //$attribute_code = $code;

            // is set in matching table?
            //if(isset($matching[$attribute_code])) {
            //
            //	$code = $matching[$attribute_code];
            //}
            // is translated via viewName?
            /*else*/
            if (strlen($value['storeLabel'])) {
                $code = $value['storeLabel'];
            }

            $code .= ($value['idx'] > 0) ? ' '.($value['idx'] + 1) : '';

            if (isset($value['visible']) && $value['visible'] === false) {
                $sxNotVisible[$code. ' sxNotVisible'] = $value['value'];
            } else {
                $sxVisible[$code] = $value['value'];
            }
        }

        $result = array_merge($sxVisible, $sxNotVisible);

        return $result;
    }

    /**
     * Get the category name for API for product.
     *
     * @param Product $product
     *
     * @return string category name(s) separated by #.
     */
    protected function getCategoryName($product)
    {
        $categoryNames = [];
        $categoryIds = $product->getCategoryIds();

        if (count($categoryIds)) {
            $categoryCollection = $this->categoryCollectionFactory
             ->create()
             ->addAttributeToSelect('name')
             ->addAttributeToFilter('entity_id', $categoryIds)
             ->addIsActiveFilter();

            /** @var Category $category */
            foreach ($categoryCollection as $category) {
                $categoryNames[] = $category->getName();
            }

            return join('#', $categoryNames);
        } else {
            return '';
        }
    }

    /**
     * Apply some more specialized rules to the result
     *
     * @param Product $product
     * @param array   $data
     * @return void
     */
    protected function processSpecialRules($product, &$data)
    {

        $this->processSimpleProductsSetGroupId($product, $data);
        $this->processConfigurableProductDescription($product, $data);
    }

    /**
     * If simple products of a configurable product are visible, set the groupId
     *
     * @param Product $product
     * @param array   $data
     * @return void
     */
    protected function processSimpleProductsSetGroupId($product, &$data)
    {
        if ($product->getTypeId() != 'simple') {
            return;
        }

        $parentIds = $this->configurable->getParentIdsByChild($product->getId());
        if (!$parentIds) {
            $parentIds = $this->configurable->getParentIdsByChild($product->getId());
        }

        if (isset($parentIds[0])) {
            $parent = $this->productRepository->getById($parentIds[0]);

            $data['description'] = (strlen($data['description']) > 0) ?
                $data['description'] : $parent->getDescription();
            $data['groupId'] = $parentIds[0];
        }
    }

    /**
     * If only the configurable product is visible, but description is not set, take
     * the description from the first child
     *
     * @param Product $product
     * @param array   $data
     * @return void
     *
     */
    protected function processConfigurableProductDescription($product, &$data)
    {
        if ($product->getTypeId() != 'configurable' || strlen($data['description']) > 0) {
            return;
        }

        $firstChild = $this->configurable
        ->getUsedProductCollection($product)
        ->addAttributeToSelect('description')
        ->setPageSize(1)
        ->getFirstItem();

        if ($firstChild && $childDescription = $firstChild->getDescription()) {
            $data['description'] = $childDescription;
        }
    }
}
