<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service;

use Semknox\Semknox\Helper\ConfigInterface;
use Semknox\Semknox\Helper\InitialUpload\InfoHelper;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use Semknox\Semknox\Service\InitialUpload\DataCollector;
use Semknox\Semknox\Service\InitialUpload\DataUploader;
use Semknox\Semknox\Service\InitialUpload\TaskExecuter;

/**
 * Class InitialUpload
 *
 * Runs the initial upload process.
 *
 * @package Semknox\Semknox\Service
 */
class InitialUpload
{

    /**
     * A number of maximum iterations in the runner's while statement.
     *
     * This is used as an emergency break in case a logic error
     * would trap the loop in an infinitely looping state.
     */
    const MAX_WHILE_ITERATIONS = 999999;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var TaskExecuter
     */
    protected $taskExecuter;

    /**
     * @var ConfigInterface
     */
    protected $config;

    /**
     * InitialUpload constructor.
     *
     * @param Logger          $logger
     * @param TaskExecuter    $taskExecuter
     * @param ConfigInterface $config
     */
    public function __construct(
        Logger $logger,
        TaskExecuter $taskExecuter,
        ConfigInterface $config
    ) {
        $this->logger = $logger;
        $this->taskExecuter = $taskExecuter;
        $this->config = $config;
    }

    /**
     * Loops the TaskExecuter of the upload process, until the state of Initial Upload is no longer RUNNING.
     * Sets the state to RUNNING before starting.
     *
     * @param InfoHelperInterface $uploadInfoHelper The store specific flag helper
     * @return void
     */
    public function execute($uploadInfoHelper)
    {
        // The process is now running.
        $uploadInfoHelper->setState(InfoHelperInterface::STATE_RUNNING);
        $this->logger->info('Upload process service was started.', $uploadInfoHelper);

        try {
            // Run until any sub-process [or the user] has changed the status.
            $emergencyBreak = 0;
            while ($uploadInfoHelper->getState() == InfoHelperInterface::STATE_RUNNING
                && $emergencyBreak <= self::MAX_WHILE_ITERATIONS
                && !$this->config->getDataUploadDisabled($uploadInfoHelper->getStoreId())
            ) {
                $this->taskExecuter->execute($uploadInfoHelper);
                // Reload the flag data, in case an external process has also made changes.
                $uploadInfoHelper->refreshFlagData();
                $emergencyBreak += 1;
            }

            // If the data upload was disabled via backend, abort the upload process
            if ($this->config->getDataUploadDisabled($uploadInfoHelper->getStoreId())) {
                $uploadInfoHelper->setState(InfoHelperInterface::STATE_STOPPED);
            }

            // If the process was stopped, reset most flags to initial values again,
            // in case they were modified in this cycle.
            if ($uploadInfoHelper->getState() == InfoHelperInterface::STATE_STOPPED) {
                $uploadInfoHelper->setToInitial();
            }

            if ($emergencyBreak >= self::MAX_WHILE_ITERATIONS) {
                $this->logger->error(
                    'MAXIMUM NUMBER OF ITERATIONS REACHED! There is probably a logic error in the runner.',
                    $uploadInfoHelper
                );
            }

            $this->logger->info('Process finished or was stopped or paused. Exiting...', $uploadInfoHelper);
        } catch (\Exception $ex) {
            $this->logger->error('ABORTING! Exception during process: '.$ex->__toString(), $uploadInfoHelper);
            throw $ex;
        }
    }
}
