<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2018 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service\InitialUpload\Task;

use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Framework\App\State;
use Magento\Framework\Exception\LocalizedException;
use Psr\Log\LoggerInterface;
use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Service\InitialUpload\TaskInterface;
use Semknox\Semknox\Library\LibEntrypoint;
use Semknox\Semknox\Library\Semknox\Query\UpdateCcss;
use Magento\Store\Model\StoreManager;

/**
 * Class CategoryUploading
 *
 * Collects category data and sends it to the Ccss endpoint.
 *
 * @package Semknox\Semknox\Service\InitialUpload\Task
 */
class CategoryUploading implements TaskInterface
{

    /**
     * @var ApiConnection
     */
    protected $apiConnection;

    /**
     * @var CollectionFactory
     */
    protected $categoryCollectionFactory;


    /**
     * @var StoreManager
     */
    protected $storeManager;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\Store\Model\App\Emulation
     */
    protected $emulation;

    /**
     * @var State
     */
    protected $appState;

    /**
     * CategoryUploading constructor.
     *
     * @param ApiConnection                      $apiConnection
     * @param CollectionFactory                  $categoryCollectionFactory
     * @param StoreManager                       $storeManager
     * @param LoggerInterface                    $logger
     * @param \Magento\Store\Model\App\Emulation $emulation
     * @param State                              $appState
     */
    public function __construct(
        ApiConnection $apiConnection,
        CollectionFactory $categoryCollectionFactory,
        StoreManager $storeManager,
        LoggerInterface $logger,
        \Magento\Store\Model\App\Emulation $emulation,
        State $appState
    ) {
        $this->apiConnection = $apiConnection;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->storeManager = $storeManager;
        $this->logger = $logger;
        $this->emulation = $emulation;
        $this->appState = $appState;
    }

    /**
     * Perform the initial upload process task
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return mixed
     * @throws ApiException
     */
    public function execute($uploadInfoHelper)
    {
        $storeId = $uploadInfoHelper->getStoreId();
        /** @var LibEntrypoint $semknoxLib */
        $semknoxLib = $this->apiConnection->get($storeId);


        // URL rewrites for categories can only be retrieved using store emulation.
        // Store emulation does not work when using Magerun 2. See ProductConverter.
        $usesEmulation = false;
        if (!$this->appState->isAreaCodeEmulated()
            || $this->appState->getMode() != State::MODE_DEVELOPER
        ) {
            $usesEmulation = true;
            $this->emulation->startEnvironmentEmulation($storeId);
        } else {
            $this->logger->warning(
                "Tudock Search Category Uploader: 
                Could not use store emulation. URLs might be incorrect.
                Store emulation does not work when using Magerun2 or App State emulation. 
                This is only allowed in developer mode."
            );
        }

        $categoryData = [];

        try {
            /** @var \Magento\Catalog\Model\ResourceModel\Category\Collection $categories */
            $categories = $this->categoryCollectionFactory->create();
            $categories
                ->addAttributeToSelect(['entity_id', 'name', 'image', 'request_path'])
                ->addAttributeToFilter('is_active', '1')
                ->setStore($this->storeManager->getStore($uploadInfoHelper->getStoreId()))
                ->addUrlRewriteToResult();

            /** @var Category $category */
            foreach ($categories as $category) {
                if ($category->getName() != null) {
                    $categoryData[] = [
                        "type" => UpdateCcss::CCSS_TYPE_CATEGORY,
                        "categoryId" => $category->getId(),
                        "categoryWeight" => $category->getLevel(),
                        "imageUrl" => $category->getImageUrl() ? $category->getImageUrl() : '',
                        "url" => $category->getUrl(),
                        "name" => $category->getName()
                    ];
                }
            }

            $semknoxLib->semknoxApi()->updateCcss()->viaArray($categoryData);
        } catch (LocalizedException $e) {
            throw new ApiException("Error during category processing", null, $e);
        } finally {
            if ($usesEmulation) {
                $this->emulation->stopEnvironmentEmulation();
            }
        }

        return 100;
    }

    /**
     * This function get's called before the task is run for the first time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskStart($uploadInfoHelper)
    {
        // Is NOT incorruptible
        $uploadInfoHelper->setInterruptable(false);
    }

    /**
     * This function get's called after the task is run for the last time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskEnd($uploadInfoHelper)
    {
        // Nothing to do
    }
}
