<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service\InitialUpload\Task;

use Magento\Framework\Filesystem\Io\File;
use Magento\Catalog\Model\Product;
use Magento\Store\Model\StoreManager;
use Semknox\Semknox\Helper\InitialUpload\FileHelper;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use Semknox\Semknox\Service\InitialUpload\TaskInterface;
use Semknox\Semknox\Service\Converter\ProductConverterInterface;
use Semknox\Semknox\Service\UploadCollectionCreator;

/**
 * Class DataCollector
 *
 * Collects product data and converts + saves it for the Search Service.
 *
 * @package Semknox\Semknox\Service\InitialUpload\Task
 */
class ProductCollecting implements TaskInterface
{

    const BATCH_SIZE = 50;
    const BATCHES_PER_FILE = 100;
    const FILE_PROCESS_NAME = 'product';

    /**
     * @var FileHelper
     */
    protected $fileHelper;

    /**
     * @var File
     */
    protected $ioFile;

    /**
     * @var ProductConverterInterface
     */
    protected $productConverter;

    /**
     * @var StoreManager
     */
    protected $storeManager;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var UploadCollectionCreator
     */
    protected $uploadCollectionCreator;

    /**
     * DataCollector constructor.
     *
     * @param FileHelper                                 $fileHelper
     * @param File                                       $ioFile
     * @param ProductConverterInterface                  $productConverter
     * @param Logger                                     $logger
     * @param UploadCollectionCreator                    $uploadCollectionCreator
     * @param StoreManager                               $storeManager
     */
    public function __construct(
        FileHelper $fileHelper,
        File $ioFile,
        ProductConverterInterface $productConverter,
        Logger $logger,
        UploadCollectionCreator $uploadCollectionCreator,
        StoreManager $storeManager
    ) {
        $this->fileHelper = $fileHelper;
        $this->ioFile = $ioFile;
        $this->productConverter = $productConverter;
        $this->logger = $logger;
        $this->uploadCollectionCreator = $uploadCollectionCreator;
        $this->storeManager = $storeManager;
    }

    /**
     * Collect one batch of shop products and convert them to Search Service API products,
     * returns progress information.
     * The batch that should be collected is gathered from the info helper, and is not advanced after processing!
     *
     * @param InfoHelperInterface $uploadInfoHelper The store specific flag helper
     *
     * @return float|int Current progress in percent. 100 means the process is finished.
     */
    public function execute($uploadInfoHelper)
    {
        $productData = [];

        // If first batch, recreate the export directory
        if ($uploadInfoHelper->getBatch() == 1) {
            $this->fileHelper->resetDirectoryFor(self::FILE_PROCESS_NAME);
        }

        $productCollection = $this->uploadCollectionCreator
            ->execute($this->storeManager->getStore($uploadInfoHelper->getStoreId()));

        $this->logger->debug('Collecting batch '.$uploadInfoHelper->getBatch(), $uploadInfoHelper);

        $productCollection
            ->setCurPage($uploadInfoHelper->getBatch())
            ->setPageSize(self::BATCH_SIZE);

        // Sadly, the resource iterator doesn't work with pagination.
        foreach ($productCollection as $product) {
            /** @var Product $product */
            $this->logger->debug('Processing product '.$product->getId(), $uploadInfoHelper);
            $productData[] = $this->productConverter->execute($product->getId(), $uploadInfoHelper->getStoreId());
        }

        $this->saveToFile(
            $uploadInfoHelper->getStoreId(),
            floor($uploadInfoHelper->getBatch() /(self::BATCHES_PER_FILE + 1)),
            $productData
        );

        return $this->collectProgress($uploadInfoHelper, $productCollection->getLastPageNumber());
    }

    /**
     * Write the product data to the end of the file.
     *
     * @param int       $storeId
     * @param int       $fileNumber
     * @param array     $productData
     *
     * @return void
     */
    private function saveToFile($storeId, $fileNumber, $productData)
    {
        // $this->ioFile does not provide a method to use
        // ftruncate so we're using non-Magento functionality here

        // @codingStandardsIgnoreStart
        $handle = fopen($this->fileHelper->getFileName(self::FILE_PROCESS_NAME, $storeId, $fileNumber), 'a+b');

        // lock for writing
        flock($handle, LOCK_EX);

        // get information about file
        $stat = fstat($handle);

        $content = json_encode($productData);

        // if more than 15 characters were written in the file (there's at least one product written already)
        // remove the closing "]" from the file, replace the starting [ from our json string with a comma and append
        if ($stat['size'] > 15) {
            // remove last character from file
            ftruncate($handle, $stat['size']-1);

            // replace the starting [ from our json
            // data with a comma to correctly format json
            $content[0] = ',';
        }

        // write content
        fwrite($handle, $content);

        // unlock file
        flock($handle, LOCK_UN);

        fclose($handle);
        // @codingStandardsIgnoreEnd
    }

    /**
     * Return the progress based on the number of collection pages.
     *
     * @param InfoHelperInterface $uploadInfoHelper
     * @param int                 $numberOfPages
     *
     * @return float|int
     */
    private function collectProgress($uploadInfoHelper, $numberOfPages)
    {
        // the current batch upload progress
        return ($uploadInfoHelper->getBatch() / ($numberOfPages)) * 100;
    }

    /**
     * This function get's called before the task is run for the first time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskStart($uploadInfoHelper)
    {
        // Is incorruptible
        $uploadInfoHelper->setInterruptable(true);
    }

    /**
     * This function get's called after the task is run for the last time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskEnd($uploadInfoHelper)
    {
        // Nothing to do
    }
}
