<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service\InitialUpload\Task;

use Magento\Framework\Filesystem\Io\File;
use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Helper\ConfigInterface;
use Semknox\Semknox\Helper\InitialUpload\FileHelper;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use Semknox\Semknox\Service\InitialUpload\TaskInterface;

/**
 * Class DataUploader
 *
 * Uploads the data collected by DataCollector to the API.
 *
 * @package Semknox\Semknox\Service\InitialUpload\Task
 */
class ProductUploading implements TaskInterface
{

    const FILE_PROCESS_NAME = 'product';

    /**
     * @var FileHelper
     */
    protected $fileHelper;

    /**
     * @var File
     */
    protected $ioFile;

    /**
     * @var ApiConnection
     */
    protected $apiConnection;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * DataCollector constructor.
     *
     * @param FileHelper      $fileHelper
     * @param File            $ioFile
     * @param ApiConnection   $apiConnection
     * @param Logger          $logger
     * @param ConfigInterface $config
     */
    public function __construct(
        FileHelper $fileHelper,
        File $ioFile,
        ApiConnection $apiConnection,
        Logger $logger,
        ConfigInterface $config
    ) {
        $this->fileHelper = $fileHelper;
        $this->ioFile = $ioFile;
        $this->apiConnection = $apiConnection;
        $this->logger = $logger;
        $this->configInterface = $config;
    }

    /**
     * Uploads the file specified by the storeId, batch number for InfoHelper and the Filename got from the FileHelper
     * to the Search Service API. If this is the first file, deletes all other entries first.
     *
     * @param InfoHelperInterface $uploadInfoHelper The store specific flag helper
     *
     * @return int The current progress of upload, where 0 is no progress and 100 is finished
     */
    public function execute($uploadInfoHelper)
    {
        // TODO: Cleanup method

        $dir = $this->fileHelper->getDirectoryFor(self::FILE_PROCESS_NAME);
        $fileName = $this->fileHelper->getFileName(
            self::FILE_PROCESS_NAME,
            $uploadInfoHelper->getStoreId(),
            $uploadInfoHelper->getBatch()-1
        );

        $this->logger->debug('Uploading file '.$uploadInfoHelper->getBatch(), $uploadInfoHelper);

        // Only continue, if upload was not disabled
        if ($this->configInterface->getDataUploadDisabled($uploadInfoHelper->getStoreId())) {
            $this->logger->error(
                'Could not continue: Data Upload was disabled via Backend setting.',
                $uploadInfoHelper
            );
            return 100;
        }

        $numberFiles = count(array_filter(scandir($dir), function ($element) {
            return substr($element, -5) == '.json';
        }));

        try {
            $fileContents = $this->ioFile->read($fileName);
            // TODO: less generic exception
            if ($fileContents === false) {
                throw new \Exception('File could not be read.');
            }
        } catch (\Exception $ex) {
            $this->logger->error(
                'Could not open '.$fileName.' - Skipping and continuing with next file. Exception was: '.$ex,
                $uploadInfoHelper
            );
            return $this->collectProgress($uploadInfoHelper, $numberFiles);
        }

        $updateEndpoint = $this->apiConnection->get($uploadInfoHelper->getStoreId())->update();

        if ($uploadInfoHelper->getBatch() == 1) {
            // FIRST set of products. Initialize process.
            $updateEndpoint->replaceProductsBegin();
        }

        $updateEndpoint->replaceProductsAddByJson($fileContents);

        if ($uploadInfoHelper->getBatch() == $numberFiles) {
            // LAST set of products. Commit.
            $updateEndpoint->replaceProductsCommit();
        }

        $progress = $this->collectProgress($uploadInfoHelper, $numberFiles);

        return $progress;
    }

    /**
     * Returns the progress, based on the number of JSON files in export directory.
     *
     * @param InfoHelperInterface $uploadInfoHelper
     * @param int                 $numberFiles
     *
     * @return float|int
     */
    protected function collectProgress($uploadInfoHelper, $numberFiles)
    {
        return ($uploadInfoHelper->getBatch() / ($numberFiles)) * 100;
    }

    /**
     * This function get's called before the task is run for the first time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskStart($uploadInfoHelper)
    {
        // Is NOT incorruptible
        $uploadInfoHelper->setInterruptable(false);
    }

    /**
     * This function get's called after the task is run for the last time in an upload process
     *
     * @param InfoHelperInterface $uploadInfoHelper
     *
     * @return void
     */
    public function taskEnd($uploadInfoHelper)
    {
        // Nothing to do
    }
}
