<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Service\InitialUpload;

use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use Semknox\Semknox\Library\Exception\ApiException;

/**
 * Class TaskExecuter
 *
 * @package Semknox\Semknox\Test\Unit\Service
 * @covers \Semknox\Semknox\Service\InitialUpload\TaskExecuter
 */
class TaskExecuter
{
    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var array
     */
    protected $schedule;

    /**
     * Cached schedule shifted to the current task. Run getCurrentTaskObjectFromSchedule to reset.
     * @var array
     */
    protected $scheduleShifted;

    /**
     * InitialUpload constructor.
     *
     * @param Logger            $logger
     * @param ScheduleInterface $schedule
     */
    public function __construct(
        Logger $logger,
        ScheduleInterface $schedule
    ) {
        $this->logger = $logger;
        $this->schedule = $schedule->getSchedule();
    }

    /**
     * Decides the next initial upload task based on the current runnerState and switches state if task is done.
     *
     * Must only be called if the overall upload state is RUNNING.
     *
     * @param InfoHelperInterface $uploadInfoHelper The store specific flag helper
     *
     * @internal Should only be used by {@see InitialUpload} and never be called directly,
     *           to ensure correct control flow.
     *
     * @return void
     */
    public function execute($uploadInfoHelper)
    {
        $this->logger->debug('Executing next action...', $uploadInfoHelper);
        // Executing a single state / action
        if (($state = $uploadInfoHelper->getRunnerState()) == InfoHelperInterface::RUNNER_WAITING) {
            $this->startFirstTask($uploadInfoHelper);
            return; // RETURNS HERE! This is only for initialising, none of the code below is relevant for this case!
        }
        $taskObj = $this->getCurrentTaskObjectFromSchedule($state);

        // EXECUTE TASK
        $stateProgress = $taskObj->execute($uploadInfoHelper);

        // Setting the current progress and proceeding depending on current state progress
        $this->setGlobalProgress($uploadInfoHelper, $state, $stateProgress);
        if ($stateProgress >= 100) {
            // Task is done!
            // End this task
            $taskObj->taskEnd($uploadInfoHelper);
            // getCurrentTaskObjectFromSchedule create a copy of the
            // schedule shifted to the current task. We use this now
            array_shift($this->scheduleShifted);
            // ARE THERE MORE TASKs?
            if (!empty($this->scheduleShifted)) {
                // Start next task
                $nextTask = $this->scheduleShifted[0];
                /** @var TaskInterface $nextTaskObj */
                $nextTaskObj = $nextTask['factory']->create();
                $nextTaskObj->taskStart($uploadInfoHelper);
                // Set to next task
                $uploadInfoHelper->setRunnerState($nextTask['key']);
            } else {
                // NO MORE TASKS - We are done
                $uploadInfoHelper->setState(InfoHelperInterface::STATE_STOPPED);
                $uploadInfoHelper->clearExceptions();
                $uploadInfoHelper->setLastSuccess(new \DateTime('now'));
            }
            // Set batch back to 1
            $uploadInfoHelper->setBatch(1);
        } else {
            // Advance batch
            $uploadInfoHelper->setBatch($uploadInfoHelper->getBatch()+1);
        }
    }

    /**
     * Depending on the current state and the sub-progress of the state,
     * calculate the current progress
     *
     * @param InfoHelperInterface $uploadInfoHelper The store specific flag helper
     * @param string              $currentState     The name of the current state for context
     * @param int                 $stateProgress    The progress of that state
     *
     * @return void
     */
    private function setGlobalProgress($uploadInfoHelper, $currentState, $stateProgress)
    {
        $progressUntilNow = 0;
        foreach ($this->schedule as $taskInSchedule) {
            if ($taskInSchedule['key'] == $currentState) {
                $progressUntilNow += $stateProgress / 100 * $taskInSchedule['weight'] ;
                break;
            }
            $progressUntilNow += $taskInSchedule['weight'];
        }
        $uploadInfoHelper->setProgress($progressUntilNow);
    }

    /**
     * Give the current task object and set the scheduleShifted field to be shifted current entry in the schedule
     *
     * @param string $state
     *
     * @return TaskInterface
     * @throws ApiException     When invalid schedule
     */
    private function getCurrentTaskObjectFromSchedule($state)
    {
        $this->scheduleShifted = $this->schedule;
        foreach ($this->schedule as $taskInSchedule) {
            if ($taskInSchedule['key'] == $state) {
                return $taskInSchedule['factory']->create();
            }
            array_shift($this->scheduleShifted);
        }
        throw new ApiException('Invalid Library Schedule. does not contain '.$state);
    }

    /**
     * @param InfoHelperInterface $uploadInfoHelper
     * @return void
     */
    private function startFirstTask($uploadInfoHelper)
    {
        // 1- Started process, reset flags
        $uploadInfoHelper->setToInitial();
        $uploadInfoHelper->setRunnerState($this->schedule[0]['key']);
        // Send start signal to first task
        $this->getCurrentTaskObjectFromSchedule($this->schedule[0]['key'])->taskStart($uploadInfoHelper);
    }
}
