<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Controller\Adminhtml\System\Config;

use Semknox\Semknox\Controller\Adminhtml\System\Config\Upload;
use Semknox\Semknox\Exception\InvalidStoreException;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperFactory;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

class UploadTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @var Upload
     */
    protected $uploadController;

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var InfoHelperFactory|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $uploadInfoHelperFactoryMock;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Backend\App\Action\Context
     */
    protected $context;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Framework\Controller\Result\Json
     */
    protected $json;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->request = $this->createMock('Magento\Framework\App\RequestInterface');

        $this->context = $this->getMockBuilder('\Magento\Backend\App\Action\Context')
            ->disableOriginalConstructor()
            ->getMock();

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId(1);

        $this->uploadInfoHelperFactoryMock =
            $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\InfoHelperFactory')
                ->setMethods(['create'])
                ->disableOriginalConstructor()
                ->getMock();

        $this->uploadInfoHelperFactoryMock->expects(self::once())
            ->method('create')
            ->willReturnCallback(
                function ($data) {
                    if ($data['storeId'] == 1) {
                        return $this->infoHelperStub;
                    }
                    throw new InvalidStoreException("Invalid store");
                }
            );

        $this->context->expects($this->any())
            ->method('getRequest')
            ->will($this->returnValue($this->request));

        $this->json = $this->getMockBuilder('\Magento\Framework\Controller\Result\Json')
            ->disableOriginalConstructor()
            ->getMock();

        $jsonFactory = $this->getMockBuilder('\Magento\Framework\Controller\Result\JsonFactory')
            ->disableOriginalConstructor()
            ->getMock();
        $jsonFactory->expects(self::once())
            ->method('create')
            ->willReturn($this->json);

        $this->uploadController = $objectManager->getObject(
            'Semknox\Semknox\Controller\Adminhtml\System\Config\Upload',
            [
                'resultJsonFactory'       => $jsonFactory,
                'context'                 => $this->context,
                'uploadInfoHelperFactory' => $this->uploadInfoHelperFactoryMock
            ]
        );
    }

    /**
     * @covers \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     */
    public function testInvalidRequest()
    {
        $this->request->expects(self::exactly(2))
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, 'invalid'],
                    ['storeId', -1, 1]
                ]
            );

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) {

                        $this->assertEquals(
                            [
                                'error' => 1,
                                'msg'   => 'Invalid Request'
                            ],
                            $array,
                            'On invalid request, return invalid request json'
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );
    }

    /**
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::pollAction
     * @dataProvider invalidStoreDataProvider
     */
    public function testInvalidStore($action, $storeId)
    {
        $this->request->expects(self::once())
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, $action],
                    ['storeId', -1, $storeId],
                ]
            );

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) {

                        $this->assertEquals(
                            [
                                'error' => 1,
                                'msg'   => 'Invalid Request'
                            ],
                            $array,
                            'On invalid store, return invalid request json'
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );
    }

    public function invalidStoreDataProvider()
    {
        return [
            ["start", 2],
            ["stop", 2],
            ["poll", 2],
            ["pause", 2],
            ["start", null],
            ["stop", null],
            ["poll", null],
            ["pause", null],
        ];
    }

    /**
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::pollAction
     * @dataProvider pollActionDataProvider
     */
    public function testPollAction($state, $progress, $lastSuccess, $expectedResponse)
    {
        $this->request->expects(self::any())
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, 'poll'],
                    ['storeId', -1, 1],
                ]
            );

        $this->infoHelperStub->setState($state);
        $this->infoHelperStub->setProgress($progress);
        $this->infoHelperStub->setLastSuccess((new \DateTime())->setTimestamp($lastSuccess));

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) use ($expectedResponse) {

                        $this->assertEquals(
                            $expectedResponse,
                            $array,
                            'On poll action, the controller has to return the state that 
                            it was provided by the InfoHelper.'
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );
    }

    public function pollActionDataProvider()
    {
        // [ state, progress, expected response]
        return [
            [
                'a completely invalid state',
                null,
                456,
                ['state' => 'a completely invalid state', 'lastSuccess' => 456, 'exceptions' => []]
            ],

            [
                InfoHelperInterface::STATE_STOPPED,
                10,
                123,
                ['state' => InfoHelperInterface::STATE_STOPPED, 'lastSuccess' => 123, 'exceptions' => []]
            ],

            [
                InfoHelperInterface::STATE_QUEUED,
                20,
                987,
                ['state' => InfoHelperInterface::STATE_QUEUED, 'progress' => 20, 'exceptions' => []]
            ],

            [
                InfoHelperInterface::STATE_PAUSED,
                30,
                654,
                ['state' => InfoHelperInterface::STATE_PAUSED, 'progress' => 30, 'exceptions' => []]
            ],

            [
                InfoHelperInterface::STATE_RUNNING,
                40,
                321,
                ['state' => InfoHelperInterface::STATE_RUNNING, 'progress' => 40, 'exceptions' => []]
            ],

            [
                InfoHelperInterface::STATE_RUNNING,
                'anything',
                101,
                ['state' => InfoHelperInterface::STATE_RUNNING, 'progress' => 'anything', 'exceptions' => []]
            ],
        ];
    }

    /**
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::startAction
     * @dataProvider startActionDataProvider
     */
    public function testStartAction($state, $expectedResponse, $isSuccess)
    {
        $this->request->expects(self::any())
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, 'start'],
                    ['storeId', -1, 1]
                ]
            );

        $this->infoHelperStub->setState($state);

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) use ($state, $expectedResponse) {

                        $this->assertEquals(
                            $expectedResponse,
                            $array,
                            'On start action with state ' . $state . ' the server has to respond ' . json_encode(
                                $expectedResponse
                            )
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );

        // If the conditions are met, setStateFlag has to be called to set the state to QUEUED.
        // Otherwise it may not be set!
        // Conditions met => InfoHelper::getStateFlag returns either STOPPED or QUEUED
        if ($isSuccess) {
            $this->assertEquals(
                InfoHelperInterface::STATE_QUEUED,
                $this->infoHelperStub->getState(),
                'On start action with state ' . $state . ' the state has to be set to ' .
                    InfoHelperInterface::STATE_QUEUED
            );
        } else {
            $this->assertEquals(
                $state,
                $this->infoHelperStub->getState(),
                'On start action with state ' . $state . ' the state has to be not changed'
            );
        }
    }

    public function startActionDataProvider()
    {
        $responseSuccess = [
            'error' => 0
        ];
        $responseFailure = [
            'error'   => 1,
            'message' => 'An upload is already running.'
        ];

        return [
            [InfoHelperInterface::STATE_STOPPED, $responseSuccess, true],
            [InfoHelperInterface::STATE_QUEUED, $responseSuccess, true],
            [InfoHelperInterface::STATE_PAUSED, $responseSuccess, true],
            [InfoHelperInterface::STATE_RUNNING, $responseFailure, false],
        ];
    }

    /**
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::stopAction
     * @dataProvider stopAndPauseActionDataProvider
     */
    public function testStopAction($state, $interruptable, $expectedResponse)
    {
        $this->request->expects(self::any())
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, 'stop'],
                    ['storeId', -1, 1],
                ]
            );

        $this->infoHelperStub->setState($state);
        $this->infoHelperStub->setInterruptable($interruptable);

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) use ($state, $expectedResponse, $interruptable) {

                        $this->assertEquals(
                            $expectedResponse,
                            $array,
                            'On stop action with state ' . $state . ' and interruptable set to ' . $interruptable . ' 
                the server has to respond ' . json_encode($expectedResponse)
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );

        // If the conditions are met, setStateFlag has to be called to set the state to STOPPED.
        // Otherwise it may not be set!
        // Conditions met => InfoHelper::isInterruptable returns true
        if ($interruptable) {
            $this->assertEquals(
                InfoHelperInterface::STATE_STOPPED,
                $this->infoHelperStub->getState(),
                'On stop action with state ' . $state . ' the state has to be set to ' .
                    InfoHelperInterface::STATE_STOPPED
            );
            $this->assertTrue(
                $this->infoHelperStub->wasSetToInitial(),
                'On stop action with state ' . $state . ' the setToInitial method has to be called'
            );
        } else {
            $this->assertEquals(
                $state,
                $this->infoHelperStub->getState(),
                'On stop action with state ' . $state . ' the state has to be not changed'
            );
            $this->assertFalse(
                $this->infoHelperStub->wasSetToInitial(),
                'On stop action with state ' . $state . ' the setToInitial method may not be called'
            );
        }
    }

    /**
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::execute
     * @covers       \Semknox\Semknox\Controller\Adminhtml\System\Config\Upload::pauseAction
     * @dataProvider stopAndPauseActionDataProvider
     */
    public function testPauseAction($state, $interruptable, $expectedResponse)
    {
        $this->request->expects(self::any())
            ->method('getParam')
            ->willReturnMap(
                [
                    ['action', null, 'pause'],
                    ['storeId', -1, 1],
                ]
            );

        $this->infoHelperStub->setState($state);
        $this->infoHelperStub->setInterruptable($interruptable);

        $this->json->expects(self::once())
            ->method('setData')
            ->with(
                $this->callback(
                    function ($array) use ($state, $expectedResponse, $interruptable) {

                        $this->assertEquals(
                            $expectedResponse,
                            $array,
                            'On pause action with state ' . $state . ' and interruptable set to ' . $interruptable . ' 
                the server has to respond ' . json_encode($expectedResponse)
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->assertEquals(
            $this->json,
            $this->uploadController->execute(),
            'The execute action must return the JSON object created by the JSON factory'
        );

        // If the conditions are met, setStateFlag has to be called to set the state to STOPPED.
        // Otherwise it may not be set!
        // Conditions met => InfoHelper::isInterruptable returns true
        if ($interruptable) {
            $this->assertEquals(
                InfoHelperInterface::STATE_PAUSED,
                $this->infoHelperStub->getState(),
                'On stop action with state ' . $state . ' the state has to be set to ' .
                    InfoHelperInterface::STATE_PAUSED
            );
        } else {
            $this->assertEquals(
                $state,
                $this->infoHelperStub->getState(),
                'On stop action with state ' . $state . ' the state has to be not changed'
            );
        }
    }

    public function stopAndPauseActionDataProvider()
    {
        $responseSuccess = [
            'error' => 0
        ];
        $responseFailure = [
            'error'   => 1,
            'message' => 'The proccess is currently not interruptable.'
        ];

        return [
            [InfoHelperInterface::STATE_STOPPED, true, $responseSuccess],
            [InfoHelperInterface::STATE_QUEUED, true, $responseSuccess],
            [InfoHelperInterface::STATE_PAUSED, true, $responseSuccess],
            [InfoHelperInterface::STATE_RUNNING, true, $responseSuccess],

            [InfoHelperInterface::STATE_STOPPED, false, $responseFailure],
            [InfoHelperInterface::STATE_QUEUED, false, $responseFailure],
            [InfoHelperInterface::STATE_PAUSED, false, $responseFailure],
            [InfoHelperInterface::STATE_RUNNING, false, $responseFailure],
        ];
    }
}
