<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Cron\InitialUpload;

use org\bovigo\vfs\vfsStream;
use org\bovigo\vfs\vfsStreamContainer;
use org\bovigo\vfs\vfsStreamDirectory;
use org\bovigo\vfs\vfsStreamFile;
use org\bovigo\vfs\vfsStreamWrapper;
use Semknox\Semknox\Cron\InitialUpload\Runner;
use Semknox\Semknox\Helper\InitialUpload\FileHelper;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperFactory;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Service\InitialUpload;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

class RunnerTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var InfoHelperFactory|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $uploadInfoHelperFactoryMock;

    /**
     * @var InitialUpload|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $initialUploadMock;

    /**
     * \Magento\Store\Model\StoreManagerInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeManagerMock;

    /**
     * @var Runner
     */
    protected $runner;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|FileHelper
     */
    protected $fileHelperMock;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Framework\Filesystem\Io\File
     */
    protected $ioFileMock;

    /**
     * @var vfsStreamContainer
     */
    protected $root;

    protected function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->initialUploadMock = $this->getMockBuilder('\Semknox\Semknox\Service\InitialUpload')
            ->disableOriginalConstructor()
            ->getMock();

        $storeMock = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $storeMock->expects(self::any())
            ->method('getId')
            ->willReturn(1);

        $storeMockTwo = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $storeMockTwo->expects(self::any())
            ->method('getId')
            ->willReturn(2);

        $this->storeManagerMock = $this->getMockBuilder('\Magento\Store\Model\StoreManagerInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $this->storeManagerMock->expects(self::any())
            ->method('getStores')
            ->willReturn(
                [
                    $storeMock,
                    $storeMockTwo
                ]
            );

        $this->infoHelperStub              = new InfoHelperStub();
        $this->uploadInfoHelperFactoryMock =
            $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\InfoHelperFactory')
                ->setMethods(['create'])
                ->disableOriginalConstructor()
                ->getMock();

        // Filesystem mocking
        vfsStreamWrapper::register();
        $this->root = vfsStreamWrapper::setRoot(new vfsStreamDirectory('__unit_test'));

        $this->fileHelperMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\FileHelper')
            ->setMethods(['getDirectory', 'getFilename'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->fileHelperMock->expects(self::atLeastOnce())
            ->method('getDirectory')
            ->willReturnMap(
                [
                    [vfsStream::url('__unit_test')]
                ]
            );

        $this->ioFileMock = $this->getMockBuilder('\Magento\Framework\Filesystem\Io\File')
            ->setMethods(['checkAndCreateFolder', 'fileExists', 'rm'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->runner = $objectManager->getObject(
            'Semknox\Semknox\Cron\InitialUpload\Runner',
            [
                'uploadInfoHelperFactory' => $this->uploadInfoHelperFactoryMock,
                'initialUpload'           => $this->initialUploadMock,
                'storeManager'            => $this->storeManagerMock,
                'fileHelper'              => $this->fileHelperMock,
                'ioFile'                  => $this->ioFileMock
            ]
        );
    }

    /**
     * Execute get's a store manager with exactly two stores in the same state.
     *
     * @covers       \Semknox\Semknox\Cron\InitialUpload\Runner::execute
     * @dataProvider executeDataProvider
     */
    public function testExecute($state, $shouldRun)
    {

        if ($this->root->hasChild('cron.lock')) {
            $this->markTestSkipped('Can not continue, cron.lock file was not cleared after last test.');
        }

        $this->ioFileMock->expects(self::once())
            ->method('fileExists')
            ->with(vfsStream::url('__unit_test') . DIRECTORY_SEPARATOR . 'cron.lock')
            ->willReturn(false);

        $this->ioFileMock->expects(self::once())
            ->method('checkAndCreateFolder');

        $this->ioFileMock->expects(self::once())
            ->method('rm')
            ->with(vfsStream::url('__unit_test') . DIRECTORY_SEPARATOR . 'cron.lock')
            ->willReturnCallback(
                function ($file) {
                    try {
                        unlink($file);
                    } catch (\Exception $e) {
                    }
                }
            );

        $this->uploadInfoHelperFactoryMock->expects(self::exactly(2))
            ->method('create')
            ->willReturn($this->infoHelperStub);

        $this->infoHelperStub->setState($state);

        if ($shouldRun) {
            $this->initialUploadMock->expects(self::exactly(2))
                ->method('execute')
                ->willReturnCallback(
                    function () {
                        $this->assertTrue($this->root->hasChild('cron.lock'), 'The runner has to create a lock file.');

                        return;
                    }
                );
        } else {
            $this->initialUploadMock->expects(self::never())
                ->method('execute');
        }

        $this->runner->execute();

        $this->assertFalse($this->root->hasChild('cron.lock'), 'The runner has to delete the lock file.');
    }

    /**
     * The initial upload process throws an exception: Error handling
     *
     * @covers       \Semknox\Semknox\Cron\InitialUpload\Runner::execute
     */
    public function testExecuteException()
    {

        $this->uploadInfoHelperFactoryMock->expects(self::exactly(2))
            ->method('create')
            ->willReturn($this->infoHelperStub);

        $this->infoHelperStub->setState(InfoHelperInterface::STATE_RUNNING);

        $ex = new \Exception("Dummy from test");
        $this->initialUploadMock->expects(self::exactly(2))
            ->method('execute')
            ->willThrowException($ex);

        $this->runner->execute();

        $this->assertEquals(
            [$ex, $ex],
            $this->infoHelperStub->getExceptionsAsObjects(),
            "The initial upload process has to set the info flag on exception"
        );
    }

    /**
     * Execute get's a store manager with exactly two stores in different states.
     *
     * @covers \Semknox\Semknox\Cron\InitialUpload\Runner::execute
     */
    public function testExecuteDifferentStates()
    {

        $this->infoHelperStub->setState(InfoHelperInterface::STATE_STOPPED);

        $infoHelperStub2 = new InfoHelperStub();
        $infoHelperStub2->setState(InfoHelperInterface::STATE_RUNNING);

        $this->uploadInfoHelperFactoryMock->expects(self::exactly(2))
            ->method('create')
            ->willReturnMap(
                [
                    [["storeId" => 1], $this->infoHelperStub],
                    [["storeId" => 2], $infoHelperStub2]
                ]
            );

        $this->initialUploadMock->expects(self::once())
            ->method('execute');

        $this->runner->execute();
    }

    /**
     * Try to execute, when the cron runner is already running (lockfile exists),
     * should not execute.
     *
     * @covers \Semknox\Semknox\Cron\InitialUpload\Runner::execute
     */
    public function testExecuteLockfileExists()
    {

        if ($this->root->hasChild('cron.lock')) {
            $this->markTestSkipped('Can not continue, cron.lock file was not cleared after last test.');
        }

        touch(vfsStream::url('__unit_test') . DIRECTORY_SEPARATOR . 'cron.lock');

        $this->ioFileMock->expects(self::once())
            ->method('fileExists')
            ->with(vfsStream::url('__unit_test') . DIRECTORY_SEPARATOR . 'cron.lock')
            ->willReturn(true);

        $this->ioFileMock->expects(self::once())
            ->method('checkAndCreateFolder');

        $this->ioFileMock->expects(self::never())
            ->method('rm');
        $this->uploadInfoHelperFactoryMock->expects(self::never())
            ->method('create');
        $this->initialUploadMock->expects(self::never())
            ->method('execute');

        $this->infoHelperStub->setState(InfoHelperInterface::STATE_RUNNING);

        $this->runner->execute();

        $this->assertTrue(
            $this->root->hasChild('cron.lock'),
            'The runner has NOT to delete the lock file in this situation.'
        );
    }

    public function executeDataProvider()
    {
        return [
            [InfoHelperInterface::STATE_PAUSED, false],
            [InfoHelperInterface::STATE_QUEUED, true],
            [InfoHelperInterface::STATE_RUNNING, true],
            [InfoHelperInterface::STATE_STOPPED, false],
        ];
    }
}
