<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Test\Unit\Helper;

use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException;
use Semknox\Semknox\Helper\ApiConnection;
use Semknox\Semknox\Test\Unit\Helper\ConfigStub;
use Semknox\Semknox\Test\Unit\Library\LibAuthenticationStub;

class ApiConnectionTest extends \PHPUnit\Framework\TestCase
{

    const STORE_ID_DEFAULT = 10000;

    /**
     * @var ConfigStub
     */
    protected $configStub;

    /**
     * @var \Magento\Framework\App\Helper\Context|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $context;

    /**
     * @var ApiConnection
     */
    protected $apiConnection;

    /**
     * @var \Semknox\Semknox\Library\LibEntrypointInterfaceFactory|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $libEntrypointFactoryMock;

    /**
     * @var \Semknox\Semknox\Library\LibEntrypointInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $libEntrypointMock;

    /**
     * \Magento\Store\Model\StoreManagerInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeManagerMock;

    /**
     * @var \Magento\Store\Api\Data\StoreInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeMock;

    /**
     * @var LibAuthenticationStub
     */
    protected $libAuthenticationStub;

    protected $configForLib;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->mockContext();

        $this->libAuthenticationStub = new LibAuthenticationStub();

        $this->libEntrypointMock = $this->getMockBuilder('\Semknox\Semknox\Library\LibEntrypointInterface')
            ->disableOriginalConstructor()
            ->getMock();

        $this->libEntrypointMock->expects(self::any())
            ->method('auth')
            ->willReturn($this->libAuthenticationStub);

        $this->libEntrypointFactoryMock = $this->getMockBuilder('\Semknox\Semknox\Library\LibEntrypointInterfaceFactory')
            ->disableOriginalConstructor()
            ->setMethods(['create'])
            ->getMock();
        $this->libEntrypointFactoryMock->expects(self::any())
            ->method('create')
            ->willReturn($this->libEntrypointMock);

        // These are used to test setting user, password and serverUrl
        $this->configForLib = ["bla" => random_int(0, 100)];

        $this->storeMock = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $this->storeMock->expects(self::any())
            ->method('getId')
            ->willReturn(self::STORE_ID_DEFAULT);

        $this->configStub = new ConfigStub();
        $this->configStub->createStore(self::STORE_ID_DEFAULT);

        $this->storeManagerMock = $this->getMockBuilder('\Magento\Store\Model\StoreManagerInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $this->storeManagerMock->expects(self::any())
            ->method('getStore')
            ->willReturn($this->storeMock);

        $this->apiConnection = $objectManager->getObject(
            'Semknox\Semknox\Helper\ApiConnection',
            [
                'libEntrypointFactory'   => $this->libEntrypointFactoryMock,
                'context'                => $this->context,
                'storeManager'           => $this->storeManagerMock,
                'config'                 => $this->configStub
            ]
        );
    }

    private function mockContext()
    {
        $this->context = $this->getMockBuilder('Magento\Framework\App\Helper\Context')
            ->disableOriginalConstructor()
            ->getMock();
    }

    /**
     * Test without requiring to set a server URL or username and password
     *
     * @covers       \Semknox\Semknox\Helper\ApiConnection::get
     * @dataProvider getDataProvider
     */
    public function testGet($storeIdPassedIntoGet, $storeIdExcpectedToBeChecked)
    {
        $this->configStub->createStore($storeIdExcpectedToBeChecked);
        $this->configStub->setConfigurationForLibrary($storeIdExcpectedToBeChecked, $this->configForLib);

        // The configuration has to be set
        $this->assertEquals(
            $this->configForLib,
            $this->apiConnection->get($storeIdPassedIntoGet)->auth()->getConfiguration(),
            "The configuration for LibAuthentication has to be set to the config returned by ConfigInterface::dump..."
        );

        $this->assertInstanceOf(
            '\Semknox\Semknox\Library\LibEntrypointInterface',
            $this->apiConnection->get($storeIdPassedIntoGet),
            'The API Connection hs to return the LibEntrypoint'
        );

        // Check if a call to get again results in any of the create methods being called again.
        $this->apiConnection->get($storeIdPassedIntoGet);
    }

    public function getDataProvider()
    {
        return [[null, self::STORE_ID_DEFAULT], [1, 1], [1000, 1000], ['apfel', 'apfel']];
    }

    public function commonIsAuthenticated($testConnectionReturnValue)
    {
        $this->libAuthenticationStub->testConnectionShouldReturn($testConnectionReturnValue);
    }

    public function testIsAuthenticatedPositive()
    {

        $this->commonIsAuthenticated('something');

        $this->assertTrue(
            $this->apiConnection->isAuthenticated(-1),
            "The check for API accessibility needs to report true, if a result is returned."
        );
    }

    public function testIsAuthenticatedNegative()
    {
        $this->commonIsAuthenticated(new ApiNotAuthenticatedException('Something happened'));

        $this->assertFalse(
            $this->apiConnection->isAuthenticated(-1),
            "The check for API accessibility needs to report false, if if a ApiNotAtuhenticatedException is thrown."
        );
    }

    /**
     * The check for API accessibility needs to pass the ApiException on errors
     * @expectedException \Semknox\Semknox\Library\Exception\ApiException
     */
    public function testIsAuthenticatedError()
    {
        $this->commonIsAuthenticated(new ApiException('Something happened'));

        $this->apiConnection->isAuthenticated();
    }

    public function testIsAuthenticatedError400()
    {
        $this->commonIsAuthenticated(new ApiException('Something happened', '', 400));

        $this->assertFalse(
            $this->apiConnection->isAuthenticated(),
            "The check for API accessibility needs to report false, if if a ApiException with code 400 is thrown."
        );
    }
}
