<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Helper;

use Semknox\Semknox\Helper\ConfigInterface;

/**
 * Class ConfigStub
 *
 * A simple data container implementation of the ConfigInterface for testing.
 *
 * @package Semknox\Semknox\Test\Unit\Helper
 */
class ConfigStub implements ConfigInterface
{

    const DEFAULT_STORE = -1;

    protected $data = [
        self::DEFAULT_STORE => [
            'seo_url_enabled' => true,
            'seo_url_prefix' => 's',
            'no_route' => 'this/is_a/noroute',
            'data_upload_disabled' => false,
            'config_to_be_dumped' => null,
            'full_upload_strategy' => false,
            'are_robots_allowed_on_search_results' => true
        ]
    ];

    /**
     * Is the options for SEO URLs currently enabled?
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getSeoUrlEnabled($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['seo_url_enabled'];
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setSeoUrlEnabled($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['seo_url_enabled'] = $newValue;
    }

    /**
     * Get the prefix to be used for SEO urls (bla ==> /bla-my-search)
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getSeoUrlPrefix($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['seo_url_prefix'];
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setSeoUrlPrefix($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['seo_url_prefix'] = $newValue;
    }

    /**
     * Get the noroute. Format: route/controller/action
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return string
     */
    public function getNoRoute($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['no_route'];
    }

    /**
     * Check whether the product upload is enabled or not
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getDataUploadDisabled($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['data_upload_disabled'];
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setDataUploadDisabled($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['data_upload_disabled'] = $newValue;
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setNoRoute($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['no_route'] = $newValue;
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setFullUploadStrategy($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['full_upload_strategy'] = $newValue;
    }

    /**
     * If false:
     * - Use indexer to process partial product updates
     * If true:
     * - The indexer does nothing
     * - Schedule a full product upload every night
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return bool
     */
    public function getFullUploadStrategy($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['full_upload_strategy'];
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setAreRobotsAllowedOnSearchResults($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['are_robots_allowed_on_search_results'] = $newValue;
    }

    /**
     * Whether the meta-robots tag für CatalogSearch result pages is set to:
     * (true) => INDEX,FOLLOW
     * (false) => NOINDEX, NOFOLLOW
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return mixed
     */
    public function getAreRobotsAllowedOnSearchResults($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['are_robots_allowed_on_search_results'];
    }

    /**
     * Dump the configuration needed for the LibAuthenticationInterface
     * implementation. The actual needed configuration depends on the library used.
     *
     * @see \Semknox\Semknox\Library\LibAuthenticationInterface::setConfiguration
     *
     * @param int $storeId Optional, explicitly set the store to get the settings from
     *
     * @return array
     */
    public function dumpConfigurationForLibrary($storeId = null)
    {
        return $this->data[$this->resolve($storeId)]['config_to_be_dumped'];
    }

    /**
     * Set a value in the data array
     *
     * @param int    $storeId
     * @param mixed  $newValue
     */
    public function setConfigurationForLibrary($storeId = null, $newValue)
    {
        $this->data[$this->resolve($storeId)]['config_to_be_dumped'] = $newValue;
    }

    /**
     * Resolve a storeId to either default or the specified.
     *
     * @param $storeId
     *
     * @return int
     */
    private function resolve($storeId)
    {
        return $storeId ? $storeId : self::DEFAULT_STORE;
    }

    /**
     * Create store config for $storeId based on the current default store settings.
     * @param int $storeId
     */
    public function createStore($storeId)
    {
        $this->data[$storeId] = $this->data[self::DEFAULT_STORE];
    }
}
