<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Helper\InitialUpload;

use http\Exception\InvalidArgumentException;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;

/**
 * Class InfoHelperStub
 *
 * Simple stub for info helper, used for testing. Writes and reads data from an array.
 *
 * @package Semknox\Semknox\Test\Unit\Helper\InitialUpload
 * @codeCoverageIgnore
 */
class InfoHelperStub implements InfoHelperInterface
{

    // Defaults
    protected $data = [
        'storeId' => -1,
        'state' => InfoHelperInterface::STATE_STOPPED,
        'interruptable' => true,
        'progress' => 0,
        'lastSuccess' => 0,
        'batch' => 1,
        'runnerState' => InfoHelperInterface::RUNNER_WAITING,
        '_setToInitialCalled' => false,
        '_flagDataRefreshed' => 0,
        'exceptions' => [],
        '_exceptionsCleared' => false
    ];

    /**
     * Returns a copy of the data array, without internal components
     */
    public function dump()
    {
        $dataCopy = $this->data;
        unset($dataCopy['_setToInitialCalled']);
        unset($dataCopy['_flagDataRefreshed']);
        return $dataCopy;
    }

    /**
     * Reload the flag data.
     */
    public function refreshFlagData()
    {
        $this->data['_flagDataRefreshed']++;
    }

    public function howOftenWasFlagDataRefreshed()
    {
        return $this->data['_flagDataRefreshed'];
    }

    public function getStoreId()
    {
        return $this->data['storeId'];
    }

    public function setStoreId($storeId)
    {
        $this->data['storeId'] = $storeId;
    }

    /**
     * Return the current flag code
     *
     * If the flag code got corrupted somehow (or isn't set yet), returns the STATE_STOPPED code.
     *
     * @return string returns a value from the state variables
     */
    public function getState()
    {
        return $this->data['state'];
    }

    /**
     * Set the flag code, setting the value to anything other than
     * the constants of this class will thrown an exception.
     *
     * @param $flagCode
     */
    public function setState($flagCode)
    {
        $this->data['state'] = $flagCode;
    }

    /**
     * Get the interruptable state.
     */
    public function isInterruptable()
    {
        return $this->data['interruptable'];
    }

    /**
     * Set the interruptable state.
     *
     * @param boolean $interruptable
     */
    public function setInterruptable($interruptable)
    {
        $this->data['interruptable'] = $interruptable;
    }

    /**
     * Get the progress.
     */
    public function getProgress()
    {
        return $this->data['progress'];
    }

    /**
     * Set the progress.
     *
     * @param number $progress
     */
    public function setProgress($progress)
    {
        $this->data['progress'] = $progress;
    }

    /**
     * Get last success.
     */
    public function getLastSuccess()
    {
        return (new \DateTime())->setTimestamp($this->data['lastSuccess']);
    }

    /**
     * Set last success
     *
     * @param \DateTime $lastSuccess
     */
    public function setLastSuccess(\DateTime $lastSuccess)
    {
        $this->data['lastSuccess'] = $lastSuccess->getTimestamp();
    }

    /**
     * Get the batch id.
     */
    public function getBatch()
    {
        return $this->data['batch'];
    }

    /**
     * Set the batch id
     *
     * @param number $batch
     */
    public function setBatch($batch)
    {
        $this->data['batch'] = $batch;
    }

    /**
     * Get the runner state.
     */
    public function getRunnerState()
    {
        return $this->data['runnerState'];
    }

    /**
     * Set the runner state
     *
     * @param string $runnerState
     */
    public function setRunnerState($runnerState)
    {
        $this->data['runnerState'] = $runnerState;
    }

    /**
     * Resets all upload related flags to a point that represents the initial state of the process.
     * This does however not modify the overall state flag.
     */
    public function setToInitial()
    {
        // Same as implementation
        // This MUST be the same as in the defaults for $this->data!
        $this->setRunnerState(InfoHelperInterface::RUNNER_WAITING);
        $this->setBatch(1);
        $this->setProgress(0);
        $this->setInterruptable(true);
        $this->data['_setToInitialCalled'] = true;
    }

    public function wasSetToInitial()
    {
        return $this->data['_setToInitialCalled'];
    }

    /**
     * Adds an exception to the upload state.
     * On the third added exception an upload process is stopped.
     *
     * @param \Exception $exception
     */
    public function addException($exception)
    {
        $this->data['exceptions'][] = $exception;
    }

    /**
     * Clears all exceptions of the upload state and resets the exception counter to zero.
     */
    public function clearExceptions()
    {
        $this->data['exceptions'] = [];
        $this->data['_exceptionsCleared'] = true;
    }

    /**
     * Get's the exceptions from this run.
     *
     * @return array [['trace' => string, 'message' => string]]
     */
    public function getExceptions()
    {
        return []; // Not implemented in Test yet;
    }

    /**
     * Get's the exceptions from this run as objects
     */
    public function getExceptionsAsObjects()
    {
        return $this->data['exceptions'];
    }

    public function wereExceptionsCleared()
    {
        return $this->data['_exceptionsCleared'];
    }
}
