<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Helper\InitialUpload;

use Magento\Framework\Exception\NoSuchEntityException;
use Semknox\Semknox\Exception\InvalidStoreException;
use Semknox\Semknox\Helper\InitialUpload\InfoHelper;

class InfoHelperTest extends \PHPUnit\Framework\TestCase
{

    const STATE_RUNNING = 'running';
    const STATE_QUEUED = 'queued';
    const STATE_STOPPED = 'stopped';
    const STATE_PAUSED = 'paused';

    const RUNNER_WAITING = 'runner_waiting';
    const RUNNER_COLLECTING = 'runner_collection';
    const RUNNER_READY_FOR_UPLOAD = 'runner_ready_upload';

    /**
     * @var \Semknox\Semknox\Model\InitialUpload\StateInfoFlagFactory|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $stateInfoFlagFactoryMock;

    /**
     * @var \Semknox\Semknox\Model\InitialUpload\StateInfoFlag|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $stateInfoFlagMock;

    /**
     * @var InfoHelper
     */
    protected $infoHelper;

    /**
     * @var array
     */
    protected $flagData;

    /**
     * \Magento\Store\Model\StoreManagerInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeManagerMock;

    public function setUp()
    {

        $this->stateInfoFlagMock = $this->getMockBuilder('\Semknox\Semknox\Model\InitialUpload\StateInfoFlag')
            ->disableOriginalConstructor()
            ->getMock();

        $this->flagData = [];

        $storeMock = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $storeMock->expects(self::any())
            ->method('getId')
            ->willReturn(1);

        $this->storeManagerMock = $this->getMockBuilder('\Magento\Store\Model\StoreManagerInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $this->storeManagerMock->expects(self::any())
            ->method('getStore')
            ->willReturnCallback(
                function ($storeId) use ($storeMock) {
                    if ($storeId == 1) {
                        return $storeMock;
                    } else {
                        throw new NoSuchEntityException();
                    }
                }
            );

        $this->stateInfoFlagMock->expects(self::any())
            ->method('getFlagData')
            ->will($this->returnCallback([$this, 'callbackGetFlagData']));

        $this->stateInfoFlagFactoryMock = $this->getMockBuilder(
            '\Semknox\Semknox\Model\InitialUpload\StateInfoFlagFactory'
        )
            ->setMethods(['create'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->stateInfoFlagFactoryMock->expects(self::any())
            ->method('create')
            ->willReturn($this->stateInfoFlagMock);
    }

    /**
     * Reinitializes the Info Helper
     */
    protected function reconstructObject()
    {
        // loadSelf has to be called once to load the flag model
        $this->stateInfoFlagMock->expects(self::atLeastOnce())
            ->method('loadSelf')
            ->willReturnSelf();

        $objectManager    = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);
        $this->infoHelper = $objectManager->getObject(
            'Semknox\Semknox\Helper\InitialUpload\InfoHelper',
            [
                'storeId'              => 1,
                'stateInfoFlagFactory' => $this->stateInfoFlagFactoryMock,
                'storeManager'         => $this->storeManagerMock,
                'dateTimeFactory'      => $objectManager->getObject('Magento\Framework\Intl\DateTimeFactory')
            ]
        );
    }

    /**
     * Callback to return the current contents of flagData
     * when getFlagData on the mock is called
     *
     * @return string
     */
    public function callbackGetFlagData()
    {
        return json_encode([1 => $this->flagData]);
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::__construct
     * InfoHelper on non-existent stores is not allowed.
     * @expectedException \Semknox\Semknox\Exception\InvalidStoreException
     */
    public function testInvalidStore()
    {

        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);


        $this->stateInfoFlagMock->expects(self::never())
            ->method('loadSelf');

        $this->infoHelper = $objectManager->getObject(
            'Semknox\Semknox\Helper\InitialUpload\InfoHelper',
            [
                'storeId'              => 2,
                'stateInfoFlagFactory' => $this->stateInfoFlagFactoryMock,
                'storeManager'         => $this->storeManagerMock
            ]
        );
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::__construct
     */
    public function testNoDataInDatabaseYet()
    {

        $this->flagData = null;

        // We just test if no exception is thrown, default checks are in the tests below.
        $this->reconstructObject();
        $this->infoHelper->getState();
    }

    /**
     *
     * A common getter test
     *
     * @param      $fieldName
     * @param      $state
     * @param      $expectedOutput
     * @param bool $isInsteadOfGet
     */
    public function commonGetTest($fieldName, $state, $expectedOutput, $isInsteadOfGet = false)
    {
        $methodName = ($isInsteadOfGet ? 'is' : 'get') . ucfirst($fieldName);

        $this->flagData = [$fieldName => $state];
        $this->reconstructObject();
        $expectedOutputString = ($expectedOutput instanceof \DateTime) ? $expectedOutput->getTimestamp(
        ) : $expectedOutput;
        $this->assertEquals(
            $expectedOutput,
            $this->infoHelper->$methodName(),
            "The method has to return $expectedOutputString if $state is set"
        );
    }

    /**
     * A common setter test
     *
     * @param   string $fieldName Field name
     * @param   mixed  $data      Input data for setter
     * @param   mixed  $expected  What result was expected to be stored? Defaults to $data
     */
    public function commonSetTest($fieldName, $data, $expected = null)
    {
        $methodName = 'set' . ucfirst($fieldName);
        if ($expected == null) {
            $expected = $data;
        }

        // Before setting anyhting, the InfoHelper has to reload the flag
        $this->flagData['__test_reload'] = 'you did not reload';
        $this->reconstructObject();
        $this->flagData['__test_reload'] = 'you reloaded';

        $this->stateInfoFlagMock->expects(self::once())
            ->method('setFlagData')
            ->with(
                $this->callback(
                    function ($newData) use ($fieldName) {
                        $newDataArray = json_decode($newData, true);

                        $this->assertEquals(
                            $this->flagData[$fieldName],
                            $newDataArray[1][$fieldName],
                            'On any valid value, the modified value that was passed in has 
                            to be saved to the flag model.'
                        );

                        $this->assertEquals(
                            'you reloaded',
                            $newDataArray[1]['__test_reload'],
                            'You must reload the flag data before setting any value in the InfoHelper!'
                        );

                        return true;
                    }
                )
            )
            ->willReturnSelf();

        $this->stateInfoFlagMock->expects(self::once())
            ->method('save');

        $this->flagData[$fieldName] = $expected;
        $this->infoHelper->$methodName($data);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::getState
     * @dataProvider getStateFlagDataProvider
     */
    public function testGetStateFlag($state, $expectedOutput)
    {
        $this->commonGetTest('state', $state, $expectedOutput);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::isInterruptable
     * @dataProvider isInterruptableDataProvider
     */
    public function testIsInterruptable($value, $expected)
    {
        $this->commonGetTest('interruptable', $value, $expected, true);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::getProgress
     * @dataProvider getProgressDataProvider
     */
    public function testGetProgress($value, $expected)
    {
        $this->commonGetTest('progress', $value, $expected);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::getLastSuccess
     * @dataProvider getLastSuccessDataProvider
     */
    public function testGetLastSuccess($value, \DateTime $expected)
    {
        $this->commonGetTest('lastSuccess', $value, $expected);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::getBatch
     * @dataProvider getProgressDataProvider
     */
    public function testGetBatch($value, $expected)
    {
        $this->commonGetTest('batch', $value, $expected);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::getRunnerState
     * @dataProvider getRunnerStateDataProvider
     */
    public function testGetRunnerState($value, $expected)
    {
        $this->commonGetTest('runnerState', $value, $expected);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setState
     * @dataProvider setStateFlagDataProvider
     */
    public function testSetStateFlag($state)
    {
        $this->commonSetTest('state', $state);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setInterruptable
     * @dataProvider setInterruptableDataProvider
     */
    public function testSetInterruptable($value)
    {
        $this->commonSetTest('interruptable', $value);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setProgress
     * @dataProvider setProgressDataProvider
     */
    public function testSetProgress($value)
    {
        $this->commonSetTest('progress', $value);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setLastSuccess
     * @dataProvider setLastSuccessDataProvider
     */
    public function testSetLastSuccess(\DateTime $value)
    {
        $this->commonSetTest('lastSuccess', $value, $value->getTimestamp());
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setBatch
     * @dataProvider setProgressDataProvider
     */
    public function testSetBatch($value)
    {
        $this->commonSetTest('batch', $value);
    }

    /**
     * @covers       \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setRunnerState
     * @dataProvider setRunnerStateDataProvider
     */
    public function testSetRunnerState($runnerState)
    {
        $this->commonSetTest('runnerState', $runnerState);
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setState
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage The flagCode must be a STATE_* value
     */
    public function testSetStateFlagInvalid()
    {

        $this->reconstructObject();

        $this->stateInfoFlagMock->expects(self::never())
            ->method('setFlagData');

        $this->stateInfoFlagMock->expects(self::never())
            ->method('save');

        $this->infoHelper->setState('invalid');
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setInterruptable
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage The interruptable value can only be set to true or false.
     */
    public function testSetInterruptableInvalid()
    {

        $this->reconstructObject();

        $this->stateInfoFlagMock->expects(self::never())
            ->method('setFlagData');

        $this->stateInfoFlagMock->expects(self::never())
            ->method('save');

        $this->infoHelper->setInterruptable('invalid');
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setProgress
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage The progress can only be set to numeric values.
     */
    public function testSetProgressInvalid()
    {

        $this->reconstructObject();

        $this->stateInfoFlagMock->expects(self::never())
            ->method('setFlagData');

        $this->stateInfoFlagMock->expects(self::never())
            ->method('save');

        $this->infoHelper->setProgress('invalid');
    }

    /**
     * @covers \Semknox\Semknox\Helper\InitialUpload\InfoHelper::setBatch
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage The batch id can only be set to numeric values.
     */
    public function testSetBatchInvalid()
    {

        $this->reconstructObject();

        $this->stateInfoFlagMock->expects(self::never())
            ->method('setFlagData');

        $this->stateInfoFlagMock->expects(self::never())
            ->method('save');

        $this->infoHelper->setBatch('invalid');
    }

    public function getStateFlagDataProvider()
    {
        return [
            [self::STATE_STOPPED, self::STATE_STOPPED],
            [self::STATE_QUEUED, self::STATE_QUEUED],
            [self::STATE_PAUSED, self::STATE_PAUSED],
            [self::STATE_RUNNING, self::STATE_RUNNING],
            [null /* nothing */, self::STATE_STOPPED],
            ['a totally invalid state', self::STATE_STOPPED]
        ];
    }

    public function setStateFlagDataProvider()
    {
        return [
            [self::STATE_STOPPED],
            [self::STATE_QUEUED],
            [self::STATE_PAUSED],
            [self::STATE_RUNNING]
        ];
    }

    public function isInterruptableDataProvider()
    {
        return [
            [true, true],
            [false, false],
            [null, true],
            ['invalid', true]
        ];
    }

    public function setInterruptableDataProvider()
    {
        return [
            [true],
            [false]
        ];
    }

    public function getProgressDataProvider()
    {
        return [
            [10, 10],
            [22, 22],
            [0, 0],
            [100, 100],
            [12.321, 12.321],
            [null, 0],
            ['invalid', 0]
        ];
    }

    public function setProgressDataProvider()
    {
        return [
            [10],
            [22],
            [0],
            [100],
            [12.321]
        ];
    }

    public function getLastSuccessDataProvider()
    {
        return [
            [0, (new \DateTime())->setTimestamp(0)],
            [12345, (new \DateTime())->setTimestamp(12345)],
            [42, (new \DateTime())->setTimestamp(42)],
            [null, (new \DateTime())->setTimestamp(0)],
            ['invalid', (new \DateTime())->setTimestamp(0)]
        ];
    }

    public function setLastSuccessDataProvider()
    {
        return [
            [new \DateTime('now')],
            [new \DateTime('10 years ago')]
        ];
    }

    public function getRunnerStateDataProvider()
    {
        return [
            [self::RUNNER_WAITING, self::RUNNER_WAITING],
            [self::RUNNER_COLLECTING, self::RUNNER_COLLECTING],
            [self::RUNNER_READY_FOR_UPLOAD, self::RUNNER_READY_FOR_UPLOAD],
            // runner state is no longer checked for "valid" values because arbitrary states are supported
            [null /* nothing */, null],
            ['a totally invalid state', 'a totally invalid state']
        ];
    }

    public function setRunnerStateDataProvider()
    {
        return [
            [self::RUNNER_WAITING],
            [self::RUNNER_COLLECTING],
            [self::RUNNER_READY_FOR_UPLOAD]
        ];
    }
}
