<?php
/**
 * SEMKNOX API - PHP Library
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */
namespace Semknox\Semknox\Test\Unit\Library\Semknox\Test;

use Semknox\Semknox\Library\Semknox\ApiCommunicator;
use Semknox\Semknox\Library\Exception\ApiException;
use Semknox\Semknox\Library\Exception\ApiNotAuthenticatedException;
use Semknox\Semknox\Library\Semknox\ApiConfig;

class ApiCommunicatorTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @var ApiConfig|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $userCredentialsMock;

    /**
     * @var \Zend\Http\Client
     */
    protected $zendHttpClient;

    /**
     * @var ApiCommunicator
     */
    protected $apiCommunicator;

    /**
     * @var int
     */
    protected $customerId;

    /**
     * @var string
     */
    protected $apiKey;

    /**
     * @var string
     */
    protected $apiUrl;

    /**
     * @var string
     */
    protected $defaultContentType;

    /**
     * Test setup
     * @return void
     */
    public function setUp()
    {
        $this->userCredentialsMock = $this->getMockBuilder(
            '\Semknox\Semknox\Library\Semknox\ApiConfig'
        )
        ->disableOriginalConstructor()
        ->getMock();

        $this->customerId = -123;
        $this->apiKey = 'test';
        $this->apiUrl = "https://semknox.example/";
        $this->defaultContentType = 'application/x-www-form-urlencoded';

        $this->userCredentialsMock->expects(self::any())->method('getCustomerId')->willReturn($this->customerId);
        $this->userCredentialsMock->expects(self::any())->method('getApiKey')->willReturn($this->apiKey);
        $this->userCredentialsMock->expects(self::any())->method('getApiUrl')->willReturn($this->apiUrl);

        $this->zendHttpClient = new \Zend\Http\Client();

        $this->apiCommunicator = new ApiCommunicator(
            $this->userCredentialsMock,
            $this->zendHttpClient
        );
    }

    /**
     * Set the adapter on the HTTP client.
     *
     * @param \Zend\Http\Client\Adapter\Test $adapter
     * @return void
     */
    protected function setAdapter(\Zend\Http\Client\Adapter\Test $adapter)
    {
        $this->zendHttpClient->setAdapter($adapter);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet200()
    {

        $adapter = new \Zend\Http\Client\Adapter\Test();

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{"test": 123}'
        );

        $this->setAdapter($adapter);

        $requestAnswer = $this->apiCommunicator->requestGet('/--/:customerId/test', ["testkey" => "testvalue"]);

        $this->assertEquals(
            ["test" => 123],
            $requestAnswer,
            "On successful transmission the returned JSON content must match the HTTP response"
        );

        $expected = [
        "testkey" => "testvalue",
        "customerId" => $this->customerId,
        "apiKey" => $this->apiKey
        ];
        $actual = $this->zendHttpClient->getRequest()->getQuery()->toArray();
        $this->assertEquals(
            $expected,
            $actual,
            "The transmitted parameters must include all specified parameters
            (in this case 'testkey'=>'testvalue' and the user information."
        );

        $this->assertEquals(
            $this->defaultContentType,
            $this->zendHttpClient->getRequest()->getHeader('Content-Type')->getFieldValue(),
            "The content type of the request must match the default content type"
        );

        $this->assertEquals(
            "/--/".$this->customerId."/test",
            str_replace($this->apiUrl, "", $this->zendHttpClient->getRequest()->getUri()),
            ":customerId has to be replaced with the customer ID ".$this->customerId
        );

        $this->assertEquals(
            'get',
            strtolower($this->zendHttpClient->getRequest()->getMethod()),
            'The method for this request has to be GET'
        );
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet200Invalid()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // Non-JSON responses have to throw an ApiException.
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            'invalid response'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestGet('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet404()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 404 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 404 Not Found"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestGet('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet401()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 401 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 401 Unauthorized"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestGet('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet403()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 403 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 403 Forbidden"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestGet('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestGet
     * @return void
     */
    public function testRequestGet503()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 503 Status code should throw ApiException
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 503 Internal Server Error"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestGet('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete200()
    {

        $adapter = new \Zend\Http\Client\Adapter\Test();

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{"test": 123}'
        );

        $this->setAdapter($adapter);

        $requestAnswer = $this->apiCommunicator->requestDelete('/--/:customerId/test', ["testkey" => "testvalue"]);

        $this->assertEquals(
            ["test" => 123],
            $requestAnswer,
            "On successful transmission the returned JSON content must match the HTTP response"
        );

        $expected = [
            "testkey" => "testvalue",
            "customerId" => $this->customerId,
            "apiKey" => $this->apiKey
        ];
        $actual = $this->zendHttpClient->getRequest()->getQuery()->toArray();
        $this->assertEquals(
            $expected,
            $actual,
            "The transmitted parameters must include all specified parameters
            (in this case 'testkey'=>'testvalue' and the user information."
        );

        $this->assertEquals(
            $this->defaultContentType,
            $this->zendHttpClient->getRequest()->getHeader('Content-Type')->getFieldValue(),
            "The content type of the request must match the default content type"
        );

        $this->assertEquals(
            "/--/".$this->customerId."/test",
            str_replace($this->apiUrl, "", $this->zendHttpClient->getRequest()->getUri()),
            ":customerId has to be replaced with the customer ID ".$this->customerId
        );

        $this->assertEquals(
            'delete',
            strtolower($this->zendHttpClient->getRequest()->getMethod()),
            'The method for this request has to be GET'
        );
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete200Invalid()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // Non-JSON responses have to throw an ApiException.
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            'invalid response'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestDelete('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete404()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 404 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 404 Not Found"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestDelete('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete401()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 401 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 401 Unauthorized"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestDelete('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete403()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 403 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 403 Forbidden"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestDelete('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestDelete
     * @return void
     */
    public function testRequestDelete503()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 503 Status code should throw ApiException
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 503 Internal Server Error"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestDelete('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut200()
    {

        $adapter = new \Zend\Http\Client\Adapter\Test();

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{"test": 123}'
        );

        $this->setAdapter($adapter);

        $requestAnswer = $this->apiCommunicator->requestPut('/--/:customerId/test', ["testkey" => "testvalue"]);

        $this->assertEquals(
            ["test" => 123],
            $requestAnswer,
            "On successful transmission the returned JSON content must match the HTTP response"
        );

        $expected = [
        "testkey" => "testvalue",
        "customerId" => $this->customerId,
        "apiKey" => $this->apiKey
        ];
        $actual = [];
        parse_str($this->zendHttpClient->getRequest()->getContent(), $actual);
        $this->assertEquals(
            $expected,
            $actual,
            "The transmitted POST-style parameters must include all specified parameters 
            (in this case 'testkey'=>'testvalue' and the user information."
        );

        $this->assertEquals(
            $this->defaultContentType,
            $this->zendHttpClient->getRequest()->getHeader('Content-Type')->getFieldValue(),
            "The content type of the request must match the default content type"
        );

        $this->assertEquals(
            "/--/".$this->customerId."/test",
            str_replace($this->apiUrl, "", $this->zendHttpClient->getRequest()->getUri()),
            ":customerId has to be replaced with the customer ID ".$this->customerId
        );

        $this->assertEquals(
            'put',
            strtolower($this->zendHttpClient->getRequest()->getMethod()),
            'The method for this request has to be PUT'
        );
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut200Invalid()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // Non-JSON responses have to throw an ApiException.
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            'invalid response'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPut('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut404()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 404 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 404 Not Found"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPut('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut401()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 401 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 401 Unauthorized"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPut('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut403()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 403 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 403 Forbidden"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPut('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPut
     * @return void
     */
    public function testRequestPut503()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 503 Status code should throw ApiException
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 503 Internal Server Error"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPut('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost200()
    {

        $adapter = new \Zend\Http\Client\Adapter\Test();

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{"test": 123}'
        );

        $this->setAdapter($adapter);

        $requestAnswer = $this->apiCommunicator->requestPost('/--/:customerId/test', ["testkey" => "testvalue"]);

        $this->assertEquals(
            ["test" => 123],
            $requestAnswer,
            "On successful transmission the returned JSON content must match the HTTP response"
        );

        $expected = [
            "testkey" => "testvalue",
            "customerId" => $this->customerId,
            "apiKey" => $this->apiKey
        ];
        $actual = [];
        parse_str($this->zendHttpClient->getRequest()->getContent(), $actual);
        $this->assertEquals(
            $expected,
            $actual,
            "The transmitted POST-style parameters must include all specified parameters 
            (in this case 'testkey'=>'testvalue' and the user information."
        );

        $this->assertEquals(
            $this->defaultContentType,
            $this->zendHttpClient->getRequest()->getHeader('Content-Type')->getFieldValue(),
            "The content type of the request must match the default content type"
        );

        $this->assertEquals(
            "/--/".$this->customerId."/test",
            str_replace($this->apiUrl, "", $this->zendHttpClient->getRequest()->getUri()),
            ":customerId has to be replaced with the customer ID ".$this->customerId
        );

        $this->assertEquals(
            'post',
            strtolower($this->zendHttpClient->getRequest()->getMethod()),
            'The method for this request has to be POST'
        );
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost200Invalid()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // Non-JSON responses have to throw an ApiException.
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            'invalid response'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPost('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost404()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 404 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 404 Not Found"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPost('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost401()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 401 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 401 Unauthorized"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPost('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost403()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 403 Status code should throw ApiNotAuthenticatedException
        $this->setExpectedException(ApiNotAuthenticatedException::class);

        $adapter->setResponse(
            "HTTP/1.1 403 Forbidden"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPost('/--', []);
    }

    /**
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPost503()
    {
        $adapter = new \Zend\Http\Client\Adapter\Test();

        // 503 Status code should throw ApiException
        $this->setExpectedException(ApiException::class);

        $adapter->setResponse(
            "HTTP/1.1 503 Internal Server Error"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{}'
        );

        $this->setAdapter($adapter);

        $this->apiCommunicator->requestPost('/--', []);
    }

    /**
     * Tests sending a custom POST body with a custom content-type
     * @covers \Semknox\Semknox\Library\ApiCommunicator::requestPost
     * @return void
     */
    public function testRequestPostStringBody()
    {

        $stringBodyText = "this is a string body";
        $contentType = "test/testing";

        $adapter = new \Zend\Http\Client\Adapter\Test();

        $adapter->setResponse(
            "HTTP/1.1 200 OK"         . "\r\n" .
            "Content-Type: text/json" . "\r\n" .
            "\r\n" .
            '{"test": 123}'
        );

        $this->setAdapter($adapter);

        $requestAnswer = $this->apiCommunicator->requestPost('/--/:customerId/test', $stringBodyText, $contentType);

        $this->assertEquals(
            ["test" => 123],
            $requestAnswer,
            "On successful transmission the returned JSON content must match the HTTP response"
        );


        $expected = [
            "customerId" => $this->customerId,
            "apiKey" => $this->apiKey
        ];
        $actual = $this->zendHttpClient->getRequest()->getQuery()->toArray();
        $this->assertEquals(
            $expected,
            $actual,
            "The transmitted GET-style parameters must include all specified parameters (user information)."
        );

        $this->assertEquals(
            $stringBodyText,
            $this->zendHttpClient->getRequest()->getContent(),
            "The transmitted POST-style parameters must include the provided body text"
        );

        $this->assertEquals(
            $contentType,
            $this->zendHttpClient->getRequest()->getHeader('Content-Type')->getFieldValue(),
            "The content type of the request must match the content type provided"
        );

        $this->assertEquals(
            "/--/".$this->customerId."/test",
            str_replace($this->apiUrl, "", $this->zendHttpClient->getRequest()->getUri()),
            ":customerId has to be replaced with the customer ID ".$this->customerId
        );

        $this->assertEquals(
            'post',
            strtolower($this->zendHttpClient->getRequest()->getMethod()),
            'The method for this request has to be POST'
        );
    }
}
