<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Service\Converter;

use Magento\Framework\App\State;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\App\Emulation;
use Magento\Store\Model\StoreManager;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;
use Semknox\Semknox\Library\Semknox\Query\UpdateCcss;
use Semknox\Semknox\Service\InitialUpload\Task\CategoryUploading;

class CategoryUploadingTest extends \PHPUnit\Framework\TestCase
{

    const STORE_ID = 100;

    /**
     * @var CategoryUploading
     */
    protected $categoryUploading;

    /**
     * Mock one: Has Image, Name and Url
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\Category
     */
    protected $categoryMock1;

    /**
     * Mock two: Has Image and Url but no name
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\Category
     */
    protected $categoryMock2;

    /**
     * Mock three: Has Name and Url but no image
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\Category
     */
    protected $categoryMock3;

    /**
     * Input category data for the model mocks
     * @var array
     */
    protected $catgeoryDataIn;

    /**
     * Expected output array that is going to be sent to API
     * @var array
     */
    protected $catgeoryDataOutExpected;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\ResourceModel\Category\Collection
     */
    protected $collectionMock;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var StoreManager|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeManagerMock;

    /**
     * @var StoreInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeMock;

    /**
     * @var UpdateCcss|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $updateCcssMock;

    /** @var array */
    protected $filterAttributes;

    /**
     * @var Emulation|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $emulationMock;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->storeId                      = self::STORE_ID;

        $this->catgeoryDataIn = [
            1 => ['name' => 'Hello 1', 'image' => 'image1', 'url' => 'url1'],
            2 => ['name' =>  null    , 'image' => 'image2', 'url' => 'url2'],
            3 => ['name' => 'Hello 3', 'image' => null    , 'url' => 'url3'],
        ];

        $this->filterAttributes = ['entity_id', 'name', 'image', 'request_path'];

        $this->catgeoryDataOutExpected = [
            [
                "type" => UpdateCcss::CCSS_TYPE_CATEGORY,
                "categoryId" => 1,
                "categoryWeight" => 100,
                "imageUrl" => $this->catgeoryDataIn[1]["image"],
                "url" => $this->catgeoryDataIn[1]["url"],
                "name" => $this->catgeoryDataIn[1]["name"]
            ],
            // Skip 2 because it has no name.
            [
                "type" => UpdateCcss::CCSS_TYPE_CATEGORY,
                "categoryId" => 3,
                "categoryWeight" => 100,
                "imageUrl" => '', // empty string because image is null
                "url" => $this->catgeoryDataIn[3]["url"],
                "name" => $this->catgeoryDataIn[3]["name"]
            ]
        ];

        $this->categoryMock1 = $this->getMockBuilder('\Magento\Catalog\Model\ResourceModel\Category')
            ->setMethods(['getId', 'getImageUrl', 'getUrl', 'getName'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->categoryMock1->expects(self::any())->method('getId')->willReturn(1);
        $this->categoryMock1->expects(self::any())->method('getName')->willReturn($this->catgeoryDataIn[1]['name']);
        $this->categoryMock1->expects(self::any())->method('getImageUrl')->willReturn($this->catgeoryDataIn[1]['image']);
        $this->categoryMock1->expects(self::any())->method('getUrl')->willReturn($this->catgeoryDataIn[1]['url']);

        $this->categoryMock2 = $this->getMockBuilder('\Magento\Catalog\Model\ResourceModel\Category')
            ->setMethods(['getId', 'getImageUrl', 'getUrl', 'getName'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->categoryMock2->expects(self::any())->method('getId')->willReturn(2);
        $this->categoryMock2->expects(self::any())->method('getName')->willReturn($this->catgeoryDataIn[2]['name']);
        $this->categoryMock2->expects(self::any())->method('getImageUrl')->willReturn($this->catgeoryDataIn[2]['image']);
        $this->categoryMock2->expects(self::any())->method('getUrl')->willReturn($this->catgeoryDataIn[2]['url']);

        $this->categoryMock3 = $this->getMockBuilder('\Magento\Catalog\Model\ResourceModel\Category')
            ->setMethods(['getId', 'getImageUrl', 'getUrl', 'getName'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->categoryMock3->expects(self::any())->method('getId')->willReturn(3);
        $this->categoryMock3->expects(self::any())->method('getName')->willReturn($this->catgeoryDataIn[3]['name']);
        $this->categoryMock3->expects(self::any())->method('getImageUrl')->willReturn($this->catgeoryDataIn[3]['image']);
        $this->categoryMock3->expects(self::any())->method('getUrl')->willReturn($this->catgeoryDataIn[3]['url']);

        $this->collectionMock = $objectManager->getCollectionMock(
            '\Magento\Catalog\Model\ResourceModel\Category\Collection',
            [$this->categoryMock1, $this->categoryMock2, $this->categoryMock3]
        );

        $categoryCollectionFactoryMock = $this->getMockBuilder('\Magento\Catalog\Model\ResourceModel\Category\CollectionFactory')
            ->setMethods(['create'])
            ->disableOriginalConstructor()
            ->getMock();
        $categoryCollectionFactoryMock->expects(self::any())->method('create')->willReturn($this->collectionMock);

        $this->storeMock = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->setMethods([])
            ->disableOriginalConstructor()
            ->getMock();

        $this->storeManagerMock = $this->getMockBuilder('\Magento\Store\Model\StoreManager')
            ->setMethods(['getStore'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->storeManagerMock
            ->expects(self::any())
            ->method('getStore')
            ->willReturn($this->storeMock);

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId($this->storeId);

        $this->updateCcssMock = $this->getMockBuilder('\Semknox\Semknox\Library\Semknox\Query\UpdateCcss')
            ->setMethods(['viaArray'])
            ->disableOriginalConstructor()
            ->getMock();

        $semknoxApiMock = $this->getMockBuilder('\Semknox\Semknox\Library\Semknox\SemknoxApi')
            ->disableOriginalConstructor()
            ->getMock();
        $semknoxApiMock->expects(self::any())
            ->method('updateCcss')
            ->willReturn($this->updateCcssMock);

        $apiMock = $this->getMockBuilder('\Semknox\Semknox\Library\LibEntrypoint')
            ->disableOriginalConstructor()
            ->getMock();
        $apiMock->expects(self::any())
            ->method('semknoxApi')
            ->willReturn($semknoxApiMock);

        $apiConnectionMock = $this->getMockBuilder('\Semknox\Semknox\Helper\ApiConnection')
            ->disableOriginalConstructor()
            ->getMock();
        $apiConnectionMock->expects(self::any())
            ->method('get')
            ->with($this->storeId)
            ->willReturn($apiMock);

        $loggerMock = $this->getMockBuilder('\Psr\Log\LoggerInterface')
            ->disableOriginalConstructor()
            ->getMock();

        $appStateMock = $this->getMockBuilder('\Magento\Framework\App\State')
            ->disableOriginalConstructor()
            ->getMock();
        $appStateMock->expects(self::any())
            ->method('isAreaCodeEmulated')
            ->willReturn(false);
        $appStateMock->expects(self::any())
            ->method('getMode')
            ->willReturn(State::MODE_PRODUCTION);

        $this->emulationMock = $this->getMockBuilder('\Magento\Store\Model\App\Emulation')
            ->disableOriginalConstructor()
            ->getMock();

        $this->categoryUploading = $objectManager->getObject(
            'Semknox\Semknox\Service\InitialUpload\Task\CategoryUploading',
            [
                'apiConnection'                => $apiConnectionMock,
                'categoryCollectionFactory'    => $categoryCollectionFactoryMock,
                'storeManager'                 => $this->storeManagerMock,
                'logger'                       => $loggerMock,
                'appState'                     => $appStateMock,
                'emulation'                    => $this->emulationMock,
            ]
        );
    }

    public function testExecute()
    {
        // Collection assertions
        $this->collectionMock
            ->expects(self::once())
            ->method('addAttributeToSelect')
            ->with($this->filterAttributes)
            ->willReturnSelf();
        $this->collectionMock
            ->expects(self::once())
            ->method('addAttributeToFilter')
            ->with('is_active', '1')
            ->willReturnSelf();
        $this->collectionMock
            ->expects(self::once())
            ->method('setStore')
            ->with($this->storeMock)
            ->willReturnSelf();

        // Assert that emulation was started and ended.
        $this->emulationMock->expects(self::once())
            ->method('startEnvironmentEmulation')
            ->with($this->storeId);
        $this->emulationMock->expects(self::once())
            ->method('stopEnvironmentEmulation');

        // Category data assertions
        $this->updateCcssMock
            ->expects(self::once())
            ->method('viaArray')
            ->with($this->catgeoryDataOutExpected);

        $this->categoryUploading->execute($this->infoHelperStub);
    }
}
