<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Service\InitialUpload;

use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\StoreManager;
use org\bovigo\vfs\vfsStream;
use org\bovigo\vfs\vfsStreamDirectory;
use org\bovigo\vfs\vfsStreamWrapper;
use Semknox\Semknox\Helper\InitialUpload\FileHelper;
use Semknox\Semknox\Service\Converter\ProductConverter;
use Semknox\Semknox\Service\InitialUpload\Task\ProductCollecting;
use Semknox\Semknox\Service\UploadCollectionCreator;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

/**
 * Class DataCollectorTest
 *
 * @package Semknox\Semknox\Test\Unit\Service\InitialUpload
 * @covers  \Semknox\Semknox\Service\InitialUpload\Task\ProductCollecting
 */
class ProductCollectingTest extends \PHPUnit\Framework\TestCase
{

    const STORE_ID = 100;

    /**
     * @var \Semknox\Semknox\Service\InitialUpload\Task\ProductCollecting
     */
    protected $productCollecting;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\Product
     */
    protected $productMock;

    /**
     * @var array
     */
    protected $conversionResult = ["this is a" => "test"];

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Catalog\Model\ResourceModel\Product\Collection
     */
    protected $collectionMock;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|ProductConverter
     */
    protected $converterMock;

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|FileHelper
     */
    protected $fileHelperMock;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|\Magento\Framework\Filesystem\Io\File
     */
    protected $ioFileMock;

    /**
     * @var int
     */
    protected $numberOfPages = ProductCollecting::BATCHES_PER_FILE * 2;

    /**
     * @var string
     */
    protected static $fileVarRoot;

    /**
     * @var string
     */
    protected static $dir;

    /**
     * @var string
     */
    protected static $resultFileNamePrefix;

    /**
     * @var int
     */
    protected $productMockId;

    /**
     * @var UploadCollectionCreator||\PHPUnit_Framework_MockObject_MockObject
     */
    protected $uploadCollectionCreatorMock;

    /**
     * @var StoreManager|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeManagerMock;

    /**
     * @var StoreInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $storeMock;

    /**
     * WARNING: The filesystem is consistent between tests
     * and doesn't get reset. @see executeDataProvider @see subTestSaving
     */
    public static function setUpBeforeClass()
    {
        // Filesystem mocking
        self::$fileVarRoot = vfsStream::url('__unit_test');
        self::$dir = self::$fileVarRoot;
        self::$resultFileNamePrefix = self::$dir . DIRECTORY_SEPARATOR . 'running-upload-data-store' . self::STORE_ID;
        vfsStreamWrapper::register();
        vfsStreamWrapper::setRoot(new vfsStreamDirectory('__unit_test'));
    }

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->storeId                      = self::STORE_ID;
        $this->productMockId                = 42;

        $this->productMock = $this->getMockBuilder('\Magento\Catalog\Model\ResourceModel\Product')
            ->setMethods(['getId'])
            ->disableOriginalConstructor()
            ->getMock();
        $this->productMock->expects(self::any())
            ->method('getId')
            ->willReturn($this->productMockId);

        $this->collectionMock = $objectManager->getCollectionMock(
            '\Magento\Catalog\Model\ResourceModel\Product\Collection',
            [$this->productMock]
        );

        $this->converterMock = $this->getMockBuilder('\Semknox\Semknox\Service\Converter\ProductConverterInterface')
            ->setMethods(['execute'])
            ->disableOriginalConstructor()
            ->getMock();

        $loggerMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\Logger')
            ->setMethods(['info', 'error', 'debug'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->fileHelperMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\FileHelper')
            ->setMethods(['resetDirectoryFor', 'getFilename'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->ioFileMock = $this->getMockBuilder('\Magento\Framework\Filesystem\Io\File')
            ->setMethods(['checkAndCreateFolder', 'fileExists', 'rmdir'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->storeMock = $this->getMockBuilder('\Magento\Store\Api\Data\StoreInterface')
            ->setMethods([])
            ->disableOriginalConstructor()
            ->getMock();

        $this->storeManagerMock = $this->getMockBuilder('\Magento\Store\Model\StoreManager')
            ->setMethods(['getStore'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->uploadCollectionCreatorMock = $this->getMockBuilder(
            '\Semknox\Semknox\Service\UploadCollectionCreator'
        )
            ->setMethods(['execute'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId($this->storeId);

        $this->productCollecting = $objectManager->getObject(
            'Semknox\Semknox\Service\InitialUpload\Task\ProductCollecting',
            [
                'productConverter'         => $this->converterMock,
                'logger'                   => $loggerMock,
                'fileHelper'               => $this->fileHelperMock,
                'ioFile'                   => $this->ioFileMock,
                'uploadCollectionCreator'  => $this->uploadCollectionCreatorMock,
                'storeManager'             => $this->storeManagerMock
            ]
        );
    }

    /**
     * @dataProvider executeDataProvider
     */
    public function testExecute($batchNumber, $expectedProgress, $expectedFileContents, $fileNameSuffix)
    {
        $this->collectionMock->expects(self::once())
            ->method('getLastPageNumber')
            ->willReturn($this->numberOfPages);
        $this->storeManagerMock->expects(self::once())
            ->method('getStore')
            ->with($this->storeId)
            ->willReturn($this->storeMock);
        $this->uploadCollectionCreatorMock->expects(self::once())
            ->method('execute')
            ->with($this->storeMock)
            ->willReturn($this->collectionMock);

        // Init tests
        $this->fileHelperMock->expects(self::any())
            ->method('getFileName')
            ->willReturnCallback(
                function ($type, $storeId, $fileNumber) {
                    $this->assertEquals(
                        'product',
                        $type,
                        'The service must ask for the filename in the product context.'
                    );
                    $this->assertEquals(
                        $this->storeId,
                        $storeId,
                        'The service may only ask for files for the current Store.'
                    );

                    return self::$resultFileNamePrefix . '-' . $fileNumber . '.json';
                }
            );

        $this->infoHelperStub->setBatch($batchNumber);

        // Test creation of product collection
        $this->subTestCollection($batchNumber);

        // Test data conversion and correct batch
        $this->subTestIfConverted();

        // Test removing file on fist batch
        $this->subTestRemovingFile($batchNumber);

        // Test return value (progress)
        $this->subTestReturnValue($expectedProgress);

        // Test saving to file
        $this->subTestSaving($expectedFileContents, $batchNumber, self::$resultFileNamePrefix . $fileNameSuffix);
    }

    /**
     * WARNING: The third data set parameter assumes that the file won't get reset between tests.
     * Please @see subTestSaving.
     *
     * @return array
     */
    public function executeDataProvider()
    {
        $newFileStart = ProductCollecting::BATCHES_PER_FILE + 1;

        return [
            [1, 1 / $this->numberOfPages * 100, json_encode(array_fill(0, 1, $this->conversionResult)), '-0.json'],
            [2, 2 / $this->numberOfPages * 100, json_encode(array_fill(0, 2, $this->conversionResult)), '-0.json'],
            [3, 3 / $this->numberOfPages * 100, json_encode(array_fill(0, 3, $this->conversionResult)), '-0.json'],

            // make sure that this batch is still in the first file
            [
                $newFileStart - 1,
                ($newFileStart - 1) / $this->numberOfPages * 100,
                json_encode(array_fill(0, 4, $this->conversionResult)),
                '-0.json'
            ],

            // all batches from now on have to be written to new files
            [
                $newFileStart,
                $newFileStart / $this->numberOfPages * 100,
                json_encode(array_fill(0, 1, $this->conversionResult)),
                '-1.json'
            ],
            [
                $newFileStart + 1,
                ($newFileStart + 1) / $this->numberOfPages * 100,
                json_encode(array_fill(0, 2, $this->conversionResult)),
                '-1.json'
            ],

            [
                $newFileStart * 2 - 1,
                ($newFileStart * 2 - 1) / $this->numberOfPages * 100,
                json_encode(array_fill(0, 3, $this->conversionResult)),
                '-1.json'
            ],

            [
                $newFileStart * 2,
                $newFileStart * 2 / $this->numberOfPages * 100,
                json_encode(array_fill(0, 1, $this->conversionResult)),
                '-2.json'
            ],
            [
                $newFileStart * 2 + 1,
                ($newFileStart * 2 + 1) / $this->numberOfPages * 100,
                json_encode(array_fill(0, 2, $this->conversionResult)),
                '-2.json'
            ]
        ];
    }

    /**
     * The collection has to be filtered by exactly these rules.
     *
     * @param $batchNumber
     */
    protected function subTestCollection($batchNumber)
    {
        $this->collectionMock->expects(self::once())
            ->method('setCurPage')
            ->with($batchNumber)
            ->willReturnSelf();
        $this->collectionMock->expects(self::once())
            ->method('setPageSize')
            ->with(ProductCollecting::BATCH_SIZE)
            ->willReturnSelf();
    }

    /**
     * Test if execute is called for the one product mocked.
     */
    protected function subTestIfConverted()
    {
        $this->converterMock->expects(self::once())
            ->method('execute')
            ->willReturnMap([[$this->productMockId, $this->storeId, $this->conversionResult]]);
    }

    protected function subTestRemovingFile($batchNumber)
    {
        if ($batchNumber == 1) {
            $this->fileHelperMock->expects(self::once())
                ->method('resetDirectoryFor')
                ->with('product');
        } else {
            $this->fileHelperMock->expects(self::never())
                ->method('resetDirectoryFor');
        }
    }

    /**
     * Test saving.
     * WARNING: This test tests the behavior across multiple runs of testExecute().
     * The filesystem does not get reset after the first set from the dataProvider is run.
     * This means that the content from the first batch is still in the file, when the second set is run.
     * This is intentional and supposed to be tested here!
     *
     * @param $expectedFileContents
     * @param $batchNumber
     * @param $fileName
     */
    protected function subTestSaving($expectedFileContents, $batchNumber, $fileName)
    {
        $this->assertEquals(
            $expectedFileContents,
            file_get_contents($fileName),
            'After saving to the result file ' . $fileName . ' ' . $batchNumber % ProductCollecting::BATCHES_PER_FILE .
            ' times the content has to be as described. With each write the same JSON content is expected to be written,
             which is [{"this is a":"test"}]. After BATCHES_PER_FILE times, a new file is started.'
        );
    }

    protected function subTestReturnValue($expectedProgress)
    {
        $this->assertEquals($expectedProgress, $this->productCollecting->execute($this->infoHelperStub));
    }

    public function testTaskStart()
    {
        $this->infoHelperStub->setInterruptable('INVALID');

        $expectedData = $this->infoHelperStub->dump();
        $expectedData['interruptable'] = true;

        $this->productCollecting->taskStart($this->infoHelperStub);

        self::assertEquals(
            $expectedData,
            $this->infoHelperStub->dump(),
            "The interruptable flag has to be set to true on start. 
             Nothing else may be changed."
        );
    }
}
