<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Service\InitialUpload;

use Magento\Framework\Filesystem\Io\File;
use org\bovigo\vfs\vfsStream;
use org\bovigo\vfs\vfsStreamContainer;
use org\bovigo\vfs\vfsStreamDirectory;
use org\bovigo\vfs\vfsStreamWrapper;
use Semknox\Semknox\Helper\InitialUpload\FileHelper;
use Semknox\Semknox\Service\InitialUpload\Task\ProductUploading;
use Semknox\Semknox\Test\Unit\Helper\ConfigStub;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

/**
 * Class DataUploaderTest
 *
 * @package Semknox\Semknox\Test\Unit\Service\InitialUpload
 * @covers  \Semknox\Semknox\Service\InitialUpload\Task\ProductUploading
 */
class ProductUploadingTest extends \PHPUnit\Framework\TestCase
{

    const STORE_ID = 100;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|FileHelper
     */
    protected $fileHelperMock;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|File
     */
    protected $ioFileMock;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var \PHPUnit_Framework_MockObject_MockObject|LibUpdateInterface
     */
    protected $libUpdateMock;

    /**
     * @var ProductUploading
     */
    protected $productUploading;

    /**
     * @var string
     */
    protected $dir;

    /**
     * @var ConfigStub
     */
    protected $configStub;

    /**
     * @var vfsStreamContainer
     */
    protected $root;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->storeId = self::STORE_ID;

        $loggerMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\Logger')
            ->setMethods(['info', 'error', 'debug'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->fileHelperMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\FileHelper')
            ->setMethods(['getDirectoryFor', 'getFilename'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->ioFileMock = $this->getMockBuilder('\Magento\Framework\Filesystem\Io\File')
            ->setMethods(['read'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId($this->storeId);

        $this->libUpdateMock = $this->getMockBuilder('\Semknox\Semknox\Library\LibUpdateInterface')
            ->disableOriginalConstructor()
            ->getMock();

        $apiMock = $this->getMockBuilder('\Semknox\Semknox\Library\LibEntrypointInterface')
            ->disableOriginalConstructor()
            ->getMock();
        $apiMock->expects(self::any())
            ->method('update')
            ->willReturn($this->libUpdateMock);

        $apiConnectionMock = $this->getMockBuilder('\Semknox\Semknox\Helper\ApiConnection')
            ->disableOriginalConstructor()
            ->getMock();
        $apiConnectionMock->expects(self::any())
            ->method('get')
            ->with($this->storeId)
            ->willReturn($apiMock);

        $this->configStub = new ConfigStub();
        $this->configStub->setDataUploadDisabled($this->storeId, false);

        $this->productUploading = $objectManager->getObject(
            'Semknox\Semknox\Service\InitialUpload\Task\ProductUploading',
            [
                'logger'        => $loggerMock,
                'fileHelper'    => $this->fileHelperMock,
                'apiConnection' => $apiConnectionMock,
                'ioFile'        => $this->ioFileMock,
                'config'        => $this->configStub
            ]
        );

        // Filesystem mocking
        $this->dir = vfsStream::url('__unit_test');
        vfsStreamWrapper::register();
        $this->root = vfsStreamWrapper::setRoot(new vfsStreamDirectory('__unit_test'));
    }

    /**
     * @dataProvider dataProvider
     */
    public function testExecute($batchId, $fileContents, $countFilesInDir, $expectedProgress)
    {
        $this->fileHelperMock->expects(self::atLeastOnce())
            ->method('getDirectoryFor')
            ->with('product')
            ->willReturn($this->dir);

        $fileName = $this->dir . DIRECTORY_SEPARATOR . 'doesntReallyMatter.json';

        // Init tests
        $this->fileHelperMock->expects(self::atLeastOnce())
            ->method('getFileName')
            ->willReturnCallback(
                function ($type, $storeId, $fileNumber) use ($batchId, $fileName) {
                    $this->assertEquals(
                        'product',
                        $type,
                        'The service must ask for the filename in the product context.'
                    );
                    $this->assertEquals(
                        $this->storeId,
                        $storeId,
                        'The service may only ask for files for the current Store.'
                    );
                    $this->assertEquals(
                        $batchId - 1,
                        $fileNumber,
                        'The service may only ask for files with the number of the current batch - 1 (eg. 1 => 0).'
                    );

                    return $fileName;
                }
            );
        $this->ioFileMock->expects(self::once())
            ->method('read')
            ->with($fileName)
            ->willReturn($fileContents);

        $this->infoHelperStub->setBatch($batchId);
        $this->infoHelperStub->setStoreId($this->storeId);

        // Make $countFilesInDir files in the directory and later see, if the progress matches this count (see below)
        for ($i = 0; $i < $countFilesInDir; $i++) {
            touch($this->dir . DIRECTORY_SEPARATOR . $i . '.json');
        }

        // If batch id is 1, replaceProductsBegin has to be called
        if ($batchId == 1) {
            $this->libUpdateMock->expects(self::once())
                ->method('replaceProductsBegin');
        } else {
            $this->libUpdateMock->expects(self::never())
                ->method('replaceProductsBegin');
        }
        $this->libUpdateMock->expects(self::once())
            ->method('replaceProductsAddByJson')
            ->with($fileContents);
        // If batch id is last batch id, replaceProductsCommit has to be called
        if ($batchId == $countFilesInDir) {
            $this->libUpdateMock->expects(self::once())
                ->method('replaceProductsCommit');
        } else {
            $this->libUpdateMock->expects(self::never())
                ->method('replaceProductsCommit');
        }

        $this->assertEquals(
            $expectedProgress,
            $this->productUploading->execute($this->infoHelperStub),
            'The progress has to be calculated using the formula [batchId / numberOfFilesInDirectory * 100].'
        );
    }

    public function testUploadDisabled()
    {
        $this->configStub->setDataUploadDisabled($this->storeId, true);

        $this->fileHelperMock->expects(self::atLeastOnce())
            ->method('getFileName')
            ->willReturn('doesntMatter');

        // If the data upload is disabled, the method must not all any uploading methods.
        $this->libUpdateMock->expects(self::never())
            ->method('replaceProductsBegin');
        $this->libUpdateMock->expects(self::never())
            ->method('replaceProductsCommit');
        $this->libUpdateMock->expects(self::never())
            ->method('replaceProductsAddByJson');

        $this->assertEquals(
            100,
            $this->productUploading->execute($this->infoHelperStub),
            'If the data upload is disabled 100% should be reported back.'
        );
    }

    public function dataProvider()
    {
        return [
            [1, '{"whatever"=>null}', 12, 1 / 12 * 100],
            [2, 'doesnt even have to be valid json }', 7, 2 / 7 * 100],
            [3, 'doesnt even have to be valid json }', 3, 100],
        ];
    }

    public function testTaskStart()
    {
        $this->infoHelperStub->setInterruptable('INVALID');

        $expectedData = $this->infoHelperStub->dump();
        $expectedData['interruptable'] = false;

        $this->productUploading->taskStart($this->infoHelperStub);

        self::assertEquals(
            $expectedData,
            $this->infoHelperStub->dump(),
            "The interruptable flag has to be set to false on start. 
            Nothing else may be changed."
        );
    }
}
