<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Service\InitialUpload;

use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Service\InitialUpload\ScheduleInterface;
use Semknox\Semknox\Service\InitialUpload\Task\ProductCollecting;
use Semknox\Semknox\Service\InitialUpload\Task\ProductUploading;
use Semknox\Semknox\Service\InitialUpload\Task\CategoryCollecting;
use Semknox\Semknox\Service\InitialUpload\Task\CategoryUploading;
use Semknox\Semknox\Service\InitialUpload\TaskExecuter;
use Semknox\Semknox\Service\InitialUpload\TaskInterface;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

class TaskExecuterTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var TaskExecuter
     */
    protected $taskExecuter;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var ScheduleInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $scheduleMock;

    /**
     * @var TaskInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $taskOneMock;

    /**
     * @var TaskInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $taskTwoMock;

    /**
     * @var TaskInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $taskThreeMock;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->storeId = 100;

        $loggerMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\Logger')
            ->setMethods(['info', 'error', 'debug'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->scheduleMock = $this->getMockBuilder('\Semknox\Semknox\Service\InitialUpload\ScheduleInterface')
            ->setMethods(['getSchedule'])
            ->disableOriginalConstructor()
            ->getMock();

        $taskBuilder = $this->getMockBuilder('\Semknox\Semknox\Service\InitialUpload\TaskInterface')
            ->disableOriginalConstructor();

        $taskFactoryBuilder = $this->getMockBuilder('\Semknox\Semknox\Service\InitialUpload\TaskInterfaceFactory')
            ->setMethods(['create'])
            ->disableOriginalConstructor();

        $this->taskOneMock = $taskBuilder->getMock();
        $this->taskTwoMock = $taskBuilder->getMock();
        $this->taskThreeMock = $taskBuilder->getMock();

        $taskFactoryOneMock = $taskFactoryBuilder->getMock();
        $taskFactoryOneMock->expects(self::any())
            ->method('create')
            ->willReturn($this->taskOneMock);

        $taskFactoryTwoMock = $taskFactoryBuilder->getMock();
        $taskFactoryTwoMock->expects(self::any())
            ->method('create')
            ->willReturn($this->taskTwoMock);

        $taskFactoryThreeMock = $taskFactoryBuilder->getMock();
        $taskFactoryThreeMock->expects(self::any())
            ->method('create')
            ->willReturn($this->taskThreeMock);

        $this->scheduleMock->expects(self::any())
            ->method('getSchedule')
            ->willReturn([
                [
                    "key" => 'task1',
                    "weight" => 50,
                    "factory" => $taskFactoryOneMock
                ],
                [
                    "key" => 'task2',
                    "weight" => 25,
                    "factory" => $taskFactoryTwoMock
                ],
                [
                    "key" => 'task3',
                    "weight" => 25,
                    "factory" => $taskFactoryThreeMock
                ]
            ]);

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId($this->storeId);

        $this->taskExecuter = $objectManager->getObject(
            'Semknox\Semknox\Service\InitialUpload\TaskExecuter',
            [
                'logger'        => $loggerMock,
                'schedule'      => $this->scheduleMock
            ]
        );
    }

    public function testTaskWaiting()
    {
        $this->infoHelperStub->setRunnerState(InfoHelperInterface::RUNNER_WAITING);

        $dataExpected                = $this->infoHelperStub->dump();
        $dataExpected['runnerState'] = 'task1';

        // When state is WAITING the taskStart of the first task in the schedule has to be called
        $this->taskOneMock->expects(self::once())
            ->method('taskStart');

        $this->taskExecuter->execute($this->infoHelperStub);

        $this->assertTrue($this->infoHelperStub->wasSetToInitial(), 'The service must call setToInitial');
        $this->assertEquals(
            $dataExpected,
            $this->infoHelperStub->dump(),
            'The service must not change anything in the info flag directly 
            but the runnerState to the first task in the schedule'
        );
    }

    public function testTaskRunning()
    {
        $this->infoHelperStub->setRunnerState('task2');

        $this->taskTwoMock->expects(self::once())
            ->method('execute')
            ->willReturn(42);

        $this->infoHelperStub->setBatch(41);

        $dataExpected             = $this->infoHelperStub->dump();
        $dataExpected['progress'] = 60.5; // (((50 [from task1]))) + (((42 / 100 * 25 [ from task2 weight])))
        $dataExpected['batch']    = 42; // 41 + 1

        $this->taskExecuter->execute($this->infoHelperStub);

        $this->assertFalse(
            $this->infoHelperStub->wasSetToInitial(),
            'The service must not call setToInitial when not in WAITING state'
        );
        $this->assertEquals(
            $dataExpected,
            $this->infoHelperStub->dump(),
            'The service must only change progress
             [sum of weight of previous tasks + current task progress scaled by current task weight] 
             and batch number (+=1) in the info flag, when running a task and not done.'
        );
    }

    public function testTaskFinishedButMoreStatesExist()
    {
        $this->infoHelperStub->setRunnerState('task2');

        $this->taskTwoMock->expects(self::once())
            ->method('execute')
            ->willReturn(100);

        $this->infoHelperStub->setBatch(41);

        $dataExpected             = $this->infoHelperStub->dump();
        $dataExpected['progress'] = 75; // (((50 [from task1]))) + ((( 25 [ from task2])))
        $dataExpected['batch']    = 1; // reset to 1
        $dataExpected['runnerState'] = 'task3'; // Next task

        // After a task has finished, call taskEnd on the old task and taskStart on the new
        $this->taskTwoMock->expects(self::once())
            ->method('taskEnd');
        $this->taskThreeMock->expects(self::once())
            ->method('taskStart');

        $this->taskExecuter->execute($this->infoHelperStub);

        $this->assertFalse(
            $this->infoHelperStub->wasSetToInitial(),
            'The service must not call setToInitial when not in WAITING state'
        );
        $this->assertEquals(
            $dataExpected,
            $this->infoHelperStub->dump(),
            'The service must only change progress
             [sum of weight of previous tasks + current task progress scaled by current task weight] 
             and batch number (to 1) in the info flag, when running a task and  done. It must also process to next task'
        );
    }

    public function testSaskFinishedWasLastTask()
    {
        $this->infoHelperStub->setRunnerState('task3');

        $this->taskThreeMock->expects(self::once())
            ->method('execute')
            ->willReturn(100);

        $this->infoHelperStub->setBatch(41);

        $dataExpected             = $this->infoHelperStub->dump();
        $dataExpected['progress']      = 100; // Done [50 + 25 + 25]
        $dataExpected['lastSuccess']   = time(); // XXX: This may fail on slow computers!
        $dataExpected['state']         = InfoHelperInterface::STATE_STOPPED;
        $dataExpected['batch']         = 1;
        $dataExpected['_exceptionsCleared'] = true;

        // After a task has finished, call taskEnd on the old task and taskStart on the new
        $this->taskThreeMock->expects(self::once())
            ->method('taskEnd');

        $this->taskExecuter->execute($this->infoHelperStub);

        $this->assertFalse(
            $this->infoHelperStub->wasSetToInitial(),
            'The service must not call setToInitial when not in WAITING state'
        );
        $this->assertEquals(
            $dataExpected,
            $this->infoHelperStub->dump(),
            'The service must only change progress [to 100], lastSucess to current time, and state to stopped
            when the last task from the schedule is finished.'
        );
    }
}
