<?php
/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

namespace Semknox\Semknox\Test\Unit\Service;

use Semknox\Semknox\Helper\InitialUpload\InfoHelper;
use Semknox\Semknox\Helper\InitialUpload\InfoHelperInterface;
use Semknox\Semknox\Helper\InitialUpload\Logger;
use Semknox\Semknox\Service\InitialUpload;
use Semknox\Semknox\Test\Unit\Helper\ConfigStub;
use Semknox\Semknox\Test\Unit\Helper\InitialUpload\InfoHelperStub;

/**
 * Class InitialUploadTest
 *
 * @package Semknox\Semknox\Test\Unit\Service
 * @covers  \Semknox\Semknox\Service\InitialUpload
 */
class InitialUploadTest extends \PHPUnit\Framework\TestCase
{

    /**
     * @var InitialUpload
     */
    protected $initialUpload;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var InfoHelperStub
     */
    protected $infoHelperStub;

    /**
     * @var InitialUpload\TaskExecuter|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $taskExecuterMock;

    /**
     * @var Logger|\PHPUnit_Framework_MockObject_MockObject
     */
    protected $loggerMock;

    /**
     * @var ConfigStub
     */
    protected $configStub;

    public function setUp()
    {
        $objectManager = new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);

        $this->storeId = 100;

        $this->loggerMock = $this->getMockBuilder('\Semknox\Semknox\Helper\InitialUpload\Logger')
            ->setMethods(['info', 'error', 'debug'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->taskExecuterMock = $this->getMockBuilder('\Semknox\Semknox\Service\InitialUpload\TaskExecuter')
            ->setMethods(['execute'])
            ->disableOriginalConstructor()
            ->getMock();

        $this->infoHelperStub = new InfoHelperStub();
        $this->infoHelperStub->setStoreId($this->storeId);

        $this->configStub = new ConfigStub();
        $this->configStub->setDataUploadDisabled($this->storeId, false);

        $this->initialUpload = $objectManager->getObject(
            'Semknox\Semknox\Service\InitialUpload',
            [
                'logger' => $this->loggerMock,
                'taskExecuter' => $this->taskExecuterMock,
                'config' => $this->configStub
            ]
        );
    }

    /**
     * Make sure, a loop exists, that continuously runs the upload logic,
     * but only if the state is set to 'running'.
     *
     * The loop always has to run at least once, since the service is supposed to set the state to running.
     * After the first loop, the test sets the state to no longer running, so the action should only be executed once
     * and no infinite loop must occur.
     */
    public function testWhileBreak()
    {
        $this->taskExecuterMock->expects(self::once())// Loop should only execute once
        ->method('execute')
            ->with(
                self::callback(
                    function () {
                        $this->infoHelperStub->setState('something else than running');

                        return true;
                    }
                )
            );

        $this->loggerMock->expects(self::never())
            ->method('error');

        $this->initialUpload->execute($this->infoHelperStub);
        $this->assertFalse(
            $this->infoHelperStub->wasSetToInitial(),
            'If the state was not set to stopped, the service must not call setToInitial'
        );
        $this->assertEquals(
            1,
            $this->infoHelperStub->howOftenWasFlagDataRefreshed(),
            'After each loop, the flag data has to be refreshed.'
        );
    }

    /**
     * Make sure there really is a loop, and the code doesn't just run one time.
     */
    public function testWhileBreakMultiple()
    {
        $counter = 0;

        $this->taskExecuterMock->expects(self::exactly(4))// Loop should be executed four times.
        ->method('execute')
            ->with(
                self::callback(
                    function () use (&$counter) {
                        if (++$counter == 4) {
                            $this->infoHelperStub->setState('something else than running');
                        }

                        return true;
                    }
                )
            );

        $this->loggerMock->expects(self::never())
            ->method('error');

        $this->initialUpload->execute($this->infoHelperStub);
        $this->assertFalse(
            $this->infoHelperStub->wasSetToInitial(),
            'If the state was not set to stopped, the service must not call setToInitial'
        );
        $this->assertEquals(
            4,
            $this->infoHelperStub->howOftenWasFlagDataRefreshed(),
            'After each loop, the flag data has to be refreshed.'
        );
    }

    /**
     * Test that the uploader resets to initial values, if it was stopped
     */
    public function testProcessWasStopped()
    {
        $this->taskExecuterMock->expects(self::once())
            ->method('execute')
            ->with(
                self::callback(
                    function () {
                        $this->infoHelperStub->setState(InfoHelperInterface::STATE_STOPPED);

                        return true;
                    }
                )
            );

        $this->loggerMock->expects(self::never())
            ->method('error');

        $this->initialUpload->execute($this->infoHelperStub);
        $this->assertTrue(
            $this->infoHelperStub->wasSetToInitial(),
            'If the state was set to stopped, the service has to call setToInitial'
        );
        $this->assertEquals(
            1,
            $this->infoHelperStub->howOftenWasFlagDataRefreshed(),
            'After each loop, the flag data has to be refreshed.'
        );
    }

    /**
     * Test that the uploader loop stops if the data upload was disabled
     */
    public function testUploadDisabled()
    {

        $this->infoHelperStub->setState(InfoHelper::STATE_RUNNING);

        $this->loggerMock->expects(self::never())
            ->method('error');

        // The task executor will run once and then disable the data upload, which should prevent it from running again.
        $this->taskExecuterMock->expects(self::once())
            ->method('execute')
            ->with(
                self::callback(
                    function () {
                        $this->configStub->setDataUploadDisabled($this->storeId, true);

                        return true;
                    }
                )
            );

        $this->initialUpload->execute($this->infoHelperStub);

        $this->assertEquals(
            InfoHelperInterface::STATE_STOPPED,
            $this->infoHelperStub->getState(),
            'If the data upload was disabled, the state has to be set to STOPPED.'
        );
    }

    /**
     * Test that the uploader also stops, if a fixed amount of iterations is reached
     */
    public function testEmergencyBreak()
    {
        $this->markTestSkipped(
            'This test has to be rewritten to somehow set the MAX_WHILE_ITERATIONS 
            to something low, so this test doesn\'t take minutes.'
        );
        /*$this->taskExecuterMock->expects(self::exactly(InitialUpload::MAX_WHILE_ITERATIONS))
        ->method('execute');

        $this->initialUpload->execute($this->infoHelperStub);*/
    }

    /**
     * Test exception handling
     * @expectedException \Exception
     * @expectedExceptionMessage Oh no!
     */
    public function testExceptionHandling()
    {
        $ex = new \Exception('Oh no!');
        $this->taskExecuterMock->expects(self::once())
            ->method('execute')
            ->willThrowException($ex);

        $this->loggerMock->expects(self::once())
            ->method('error');

        $this->initialUpload->execute($this->infoHelperStub);
    }
}
