/**
 * SEMKNOX Magento 2 Module
 *
 * @copyright   Copyright (c) 2017 Tudock GmbH (http://www.tudock.de)
 * @author      Marco Köpcke <marco.koepcke@tudock.de>
 */

/**
 * A callback for the upload._request method
 *
 * @callback requestCallback
 * @param {boolean}     success             Whether the call was successful or not
 * @param {object}      responseObject      The response JSON decoded object or error information
 */

define([
    'mage/translate',
    'jquery',
    'Magento_Ui/js/modal/modal',
    'prototype',
], function($t, $){

    /**
     * Upload constructor
     *
     * Client logic for the initial uploader
     *
     * @param config            A configuration object, @see upload.config
     */
    var Upload = function(config) {
        this.config = config != null ? config : {
            'initialState' : '',
            'progressContainerId' : '',
            'startButtonId' : '',
            'stopButtonId' : '',
            'pauseButtonId' : '',
            'resumeButtonId' : '',
            'ajaxUrl': '',
            'errorMessageId': '',
            'exceptionDisplayId' : '',
            'exceptionMessages': {
                'duringLastStopped': '',
                'duringThis': '',
                'dialogTitle': '',
                'dialogClose': ''
            },
            'storeId' : 1
        };

        this.states = {
            INIT: 'init',
            QUEUED: 'queued',
            STOPPED: 'stopped',
            RUNNING: 'running',
            PAUSED: 'paused'
        };

        this.currentState = null;

        this.progressContainer = document.getElementById(this.config.progressContainerId);
        this.startButton = document.getElementById(this.config.startButtonId);
        this.stopButton = document.getElementById(this.config.stopButtonId);
        this.pauseButton = document.getElementById(this.config.pauseButtonId);
        this.resumeButton = document.getElementById(this.config.resumeButtonId);
        this.errorMessage = document.getElementById(this.config.errorMessageId);
        this.exceptionDisplay = document.getElementById(this.config.exceptionDisplayId);

        this._setState(this.config.initialState);

        this._poll();

        return this;
    };

    Upload.prototype = {
        /**
         * Set the state of the uploader
         * The state determines the state of buttons and progress display
         *
         * @param {string} state a value from upload.states
         * @private
         */
        _setState: function (state) {
            if (state == this.currentState) return;
            this.startButton.disabled = state == this.states.INIT;
            this._clearError();
            switch (state) {
                case this.states.STOPPED:
                    this.stopButton.classList.add('sx-hidden');
                    this.pauseButton.classList.add('sx-hidden');
                    this.startButton.classList.remove('sx-hidden');
                    this.resumeButton.classList.add('sx-hidden');
                    this.progressContainer.classList.add('sx-hidden');
                    this.currentState = this.states.STOPPED;
                    break;
                case this.states.QUEUED:
                    this.stopButton.classList.remove('sx-hidden');
                    this.pauseButton.classList.add('sx-hidden');
                    this.startButton.classList.add('sx-hidden');
                    this.resumeButton.classList.add('sx-hidden');
                    this.progressContainer.classList.remove('sx-hidden');
                    var errMsg = $t("SKNOX_The Uplaod is now scheduled and will begin within the next minute. If the upload doesn't start, make sure you have %1Cron%2 configured correctly.");
                    errMsg = errMsg.replace('%1', '<a href="http://devdocs.magento.com/guides/v2.0/config-guide/cli/config-cli-subcommands-cron.html" target="_blank">').replace('%2', '</a>');
                    this._setError(errMsg, 'info');
                    this.currentState = this.states.QUEUED;
                    break;
                case this.states.RUNNING:
                    this.stopButton.classList.remove('sx-hidden');
                    this.pauseButton.classList.remove('sx-hidden');
                    this.startButton.classList.add('sx-hidden');
                    this.resumeButton.classList.add('sx-hidden');
                    this.progressContainer.classList.remove('sx-hidden');
                    this.currentState = this.states.RUNNING;
                    break;
                case this.states.PAUSED:
                    this.stopButton.classList.remove('sx-hidden');
                    this.pauseButton.classList.add('sx-hidden');
                    this.startButton.classList.add('sx-hidden');
                    this.resumeButton.classList.remove('sx-hidden');
                    this.progressContainer.classList.remove('sx-hidden');
                    this.currentState = this.states.PAUSED;
                    break;
            }
        },

        /**
         * Set an error message,
         * if an error message exists, this will reset the form to INIT state
         * and display the error message to the user.
         *
         * @param {string} errMsg
         * @param {string} errType    The type of error. Possible values are 'error', 'info', 'warning'. 'error' is default.
         * @private
         */
        _setError: function (errMsg, errType) {
            if (errType == null) {
                errType = 'error';
            }
            // Display error message
            this.errorMessage.className = 'message message-'+errType+' '+errType;
            this.errorMessage.childNodes[0].innerHTML = errMsg;
        },

        _clearError: function() {
            this.errorMessage.classList.add('sx-hidden');
        },

        /**
         * Updates the display that shows last errors of the full upload
         * @param exceptionList array
         * @private
         */
        _setExceptions: function(exceptionList) {
            if (exceptionList.length === 0) {
                this.exceptionDisplay.classList.add('sx-hidden');
                return;
            } else {
                this.exceptionDisplay.classList.remove('sx-hidden');
            }
            if (this.currentState === this.states.STOPPED) {
                this.exceptionDisplay.getElementsByClassName("sx-exd-header")[0].innerHTML = this.config.exceptionMessages.duringLastStopped;
                this.exceptionDisplay.classList.add('message-error');
                this.exceptionDisplay.classList.remove('message-warning');
            } else {
                this.exceptionDisplay.getElementsByClassName("sx-exd-header")[0].innerHTML = this.config.exceptionMessages.duringThis;
                this.exceptionDisplay.classList.remove('message-error');
                this.exceptionDisplay.classList.add('message-warning');
            }

            var listEl = this.exceptionDisplay.getElementsByClassName("sx-exd-list")[0];
            while( listEl.firstChild ){
                listEl.removeChild( listEl.firstChild );
            }
            var self = this;
            for (var i = 0; i < exceptionList.length; i++) {
                var exception = exceptionList[i];
                var li = document.createElement("li");
                var a = document.createElement("a");
                (function() {
                    var exceptionCapture = exception;
                    a.onclick = function() {
                        $('<div />').html('<pre>' + exceptionCapture.message + '\n\n' + exceptionCapture.trace + '</pre>')
                            .modal({
                                title: self.config.exceptionMessages.dialogTitle,
                                autoOpen: true,
                                buttons: [{
                                    text: self.config.exceptionMessages.dialogClose
                                }]
                            });
                    }
                })();
                a.href = "#";
                a.innerHTML = exception.message;
                li.appendChild(a);
                listEl.appendChild(li);
            }

        },

        _setProgress: function(progress) {
            this.progressContainer.querySelector('.sx-progress-bar').style.width = progress+'%';
            this.progressContainer.querySelector('.sx-progress-label').innerHTML = progress+' %';
        },

        /**
         * Perform an ajax request on the upload controller.
         * @param   {object}            params                  Parameters for the request
         * @param   {requestCallback}   callback                The callback function
         * @param   {boolean}           [disableLoadingMask]    Optional, if set the loading mask will be disabled
         * @private
         */
        _request: function (params, callback, disableLoadingMask) {
            params.storeId = this.config.storeId;
            var options = {
                parameters: params,
                onComplete: function(response) {
                    if (!response.responseText) {
                        return callback(false, {'response': response});
                    }
                    try {
                        if (response.responseText.isJSON()) {
                            return callback(true, response.responseText.evalJSON());
                        } else {
                            return callback(true, {result: response});
                        }
                    } catch (e) {
                        return callback(false, {'exception': e});
                    }
                }
            };
            if (disableLoadingMask) {
                options.loaderArea = false;
            }
            new Ajax.Request(this.config.ajaxUrl, options);
        },

        /**
         * Get the current uploader status from the server and calls itself 1500 after initial request
         * @private
         */
        _poll: function() {
            var self = this;
            this._request({action: 'poll'}, function(success, response) {
                switch (response.state) {
                    case self.states.STOPPED:
                    case self.states.RUNNING:
                    case self.states.PAUSED:
                    case self.states.QUEUED:
                        self._setState(response.state);
                        self._setExceptions(response.exceptions);
                        if (response.progress != null) {
                            self._setProgress(Math.round(response.progress));
                        }
                        if (response.state == self.states.STOPPED) {
                            var oneMinuteAgo = new Date();
                            oneMinuteAgo.setMinutes(oneMinuteAgo.getMinutes()-1);
                            if (new Date(response.lastSuccess * 1000) > oneMinuteAgo) {
                                self._setError($t("SKNOX_The upload was successful. Please note that it can take a few minutes until the products are updated."), 'success');
                            }
                        }
                        break;
                    default:
                        console.error("UPLOAD:  Polling error: ");
                        console.error(response);
                        break;
                }
                window.setTimeout(function() {self._poll()}, 3000);
            }, true);
        },

        start: function() {
            var self = this;

            console.log("UPLOAD:  Start Request sent");
            this._request({action: 'start'}, function(success, response) {
                switch (response.error) {
                    case 0:
                        console.log("UPLOAD:  Upload queued");
                        self._setState(self.states.QUEUED);
                        break;
                    default:
                        console.log("UPLOAD:  The upload could not be queued: "+response.message);
                        self._setError($t('SKNOX_The upload could not be queued.'));
                        break;
                }
            });
        },

        stop: function() {
            var self = this;

            console.log("UPLOAD:  Stop Request sent");
            this._request({action: 'stop'}, function(success, response) {
                    switch (response.error) {
                        case 0:
                            console.log("UPLOAD:  Upload stopped");
                            self._setProgress(0);
                            self._setState(self.states.STOPPED);
                            break;
                        default:
                            console.log("UPLOAD:  The upload can not be stopped right now: "+response.message);
                            self._setError($t('SKNOX_The upload can not be stopped right now.'));
                            break;
                    }
            });
        },

        pause: function() {
            var self = this;

            console.log("UPLOAD:  Pause Request sent");
            this._request({action: 'pause'}, function(success, response) {
                switch (response.error) {
                    case 0:
                        console.log("UPLOAD:  Upload paused");
                        self._setState(self.states.PAUSED);
                        break;
                    default:
                        self._setError($t('SKNOX_The upload can not be paused right now.'));
                        break;
                }
            });
        }
    };

    return Upload;
});